// 
// Confidential property of Sybase, Inc.
// (c) Copyright Sybase, Inc. 2000.
// All rights reserved
// 
package com.sybase.jaguar.management.jagtool.commands;

import java.io.*;
import java.text.*;
import java.util.*;
import com.sybase.jaguar.management.*;
import com.sybase.jaguar.management.jagtool.*;
import com.sybase.jaguar.system.Repository;
import com.sybase.jaguar.system.RepositoryHelper;
import com.sybase.jaguar.util.*;

/**
* Deletes an entity from the Jaguar repository.
*/
public class DeleteCommand extends BaseCommand
{
    public static final String ENTITY_PROP = "entity";    

    public static final String[] VALID_ENTITY_TYPES =
    {
        EntityIdentifier.APP_TYPE,
        EntityIdentifier.CACHE_TYPE,
        EntityIdentifier.CLUSTER_TYPE,
        EntityIdentifier.COMP_TYPE,
        EntityIdentifier.PACKAGE_TYPE,
        EntityIdentifier.ROLE_TYPE,
        EntityIdentifier.SERVER_TYPE,
        EntityIdentifier.SERVLET_TYPE,
        EntityIdentifier.WEBAPP_TYPE
    };

    private static JTResourceManager _resources =
    	new JTResourceManager(DeleteCommand.class);

    private EntityIdentifier _entity;

    public String getName()
    {
        return _resources.getString("deleteName");
    }

    public String getDescription()
    {
        return _resources.getString("deleteDesc");
    }

    public String getUsage()
    {
        return _resources.getString("deleteUsage");
    }

    public void setEntity(String entityProp) throws Exception
    {
        try
        {
            _entity = new EntityIdentifier(entityProp);
        }
        catch (EntityIdentifierException ex)
        {
            throw new UsageException(Utils.formatInvalidValueMsg(ENTITY_PROP, entityProp), this);
        }
    }

    public void setArgs(String[] args) throws Exception
    {
        if (args.length == 0)
        {
            throw new UsageException(this);
        }
        else
        {
            String arg = args[0];

            if (arg.startsWith("-"))
            {
                throw new UsageException(Utils.formatUnknownOptionMsg(arg), this);
            }
            else
            {
                setEntity(arg);
            }
        }
    }

    public void run() throws Exception
    {
        JTContext context = getContext();

        Utils.checkEntityIsSet(this, ENTITY_PROP, _entity);
        Utils.checkEntityType(this, _entity, VALID_ENTITY_TYPES);

        boolean exists = context.getRepository().exists(_entity.getType(), _entity.getName());

        if (exists == false)
        {
            println(_resources.getString("entityNotExistMsg",
                new String[] { _entity.getType(), _entity.getName() }));
            return;
        }

        try
        {
            JRepository repository =
                new JRepository(context.getUserName(),
                                context.getPassword(),
                                context.getHostName(),
                                context.getPort(),
                                "rw");

            if (_entity.getType().equals(EntityIdentifier.APP_TYPE))
            {
                deleteApp(repository, _entity.getName());
            }
            else if (_entity.getType().equals(EntityIdentifier.CACHE_TYPE))
            {
                deleteConnCache(repository, _entity.getName());
            }
            else if (_entity.getType().equals(EntityIdentifier.CLUSTER_TYPE))
            {
                deleteCluster(repository, _entity.getName());
            }
            else if (_entity.getType().equals(EntityIdentifier.COMP_TYPE))
            {
                deleteComponent(repository, _entity.getName());
            }
            else if (_entity.getType().equals(EntityIdentifier.PACKAGE_TYPE))
            {
                deletePackage(repository, _entity.getName());
            }
            else if (_entity.getType().equals(EntityIdentifier.SERVER_TYPE))
            {
                deleteServer(repository, _entity.getName());
            }
            else if (_entity.getType().equals(EntityIdentifier.SERVLET_TYPE))
            {
                deleteServlet(repository, _entity.getName());
            }
            else if (_entity.getType().equals(EntityIdentifier.WEBAPP_TYPE))
            {
                deleteWebApp(repository, _entity.getName());
            }
            else
            {
                Assert.assert(this, false,
                    MessageFormat.format("Unexpected entity type: {0}",
                        new String[] { _entity.getType() }));
            }
        }
        catch (JException jex)
        {
            throw new JTException(_resources.getString("deleteFailedMsg",
                new String[] { _entity.getType(), _entity.getName() }), this, jex);
        }

        println(_resources.getString("deletedEntityMsg",
            new String[] { _entity.getType(), _entity.getName() }));
    }

    protected void deleteApp(JRepository rep, String name) throws JException
    {
        JApplication app = rep.getApplication(name);

        if (app != null)
        {
            rep.removeApplication(app);
            rep.save();
        }
    }

    protected void deleteConnCache(JRepository rep, String name) throws JException
    {
        JConnectionCache cache = rep.getConnectionCache(name);

        if (cache != null)
        {
            rep.removeConnectionCache(cache);
            rep.save();
        }
    }

    protected void deleteCluster(JRepository rep, String name) throws JException
    {
        JCluster cluster = rep.getCluster(name);

        if (cluster != null)
        {
            rep.deleteCluster(cluster);
            rep.save();
        }
    }

    protected void deleteComponent(JRepository rep, String name) throws JException
    {
        String packName = name.substring(0, name.indexOf('/'));
        String compName = name.substring(name.indexOf('/') + 1);

        JPackage pack = rep.getPackage(packName);
        
        if (pack != null)
        {
            JComponent comp = pack.getComponent(compName);

            if (comp != null)
            {
                pack.removeComponent(comp);
                rep.save();
            }
        }
    }

    protected void deletePackage(JRepository rep, String name) throws JException
    {
        JPackage pack = rep.getPackage(name);

        if (pack != null)
        {
            rep.removePackage(pack);
            rep.save();
        }
    }

    protected void deleteRole(JRepository rep, String name) throws JException
    {
        JRole role = rep.getRole(name);

        if (role != null)
        {
            rep.removeRole(role);
            rep.save();
        }
    }

    protected void deleteServer(JRepository rep, String name) throws JException
    {
        JServer server = rep.getServer(name);

        if (server != null)
        {
            rep.removeServer(server);
            rep.save();
        }
    }

    protected void deleteServlet(JRepository rep, String name) throws JException
    {
        JServlet servlet = rep.getServlet(name);

        if (servlet != null)
        {
            rep.removeServlet(servlet);
            rep.save();
        }
    }

    protected void deleteWebApp(JRepository rep, String name) throws JException
    {
        JWebApplication webapp = rep.getWebApp(name);

        if (webapp != null)
        {
            rep.removeWebApp(webapp);
            rep.save();
        }
    }
}
