// 
// Confidential property of Sybase, Inc.
// (c) Copyright Sybase, Inc. 2000.
// All rights reserved
// 
package com.sybase.jaguar.management.jagtool.commands;

import java.io.*;
import java.text.*;
import java.util.*;
import com.sybase.jaguar.management.jagtool.*;
import com.sybase.jaguar.management.*;
import com.sybase.jaguar.importer.*;
import com.sybase.jaguar.packager.*;
import com.sun.enterprise.deployment.*;

public class ExportCommand extends BaseCommand
{
    public static final String JAGJAR_PROP = "jagjar";
    public static final String DIR_PROP    = "dir";
    public static final String ENTITY_PROP = "entity";

    public static final String[] VALID_ENTITY_TYPES =
    {
        EntityIdentifier.APP_TYPE,
        EntityIdentifier.PACKAGE_TYPE,
        EntityIdentifier.WEBAPP_TYPE
    };
 
    private static JTResourceManager _resources =
    	new JTResourceManager(ExportCommand.class);

    private boolean          _jagjarProp;
    private String           _dirProp;   
    private EntityIdentifier _entity;    

    public String getName()
    {
        return _resources.getString("exportName");
    }

    public String getDescription()
    {
        return _resources.getString("exportDesc");
    }

    public String getUsage()
    {
        return _resources.getString("exportUsage");
    }

    public void setJagJar(boolean jagjarProp) throws Exception
    {
        _jagjarProp = jagjarProp;
    }

    public void setDir(String dirProp) throws Exception
    {
        _dirProp = dirProp;
    }

    public void setEntity(String entityProp) throws Exception
    {
        try
        {
            _entity = new EntityIdentifier(entityProp);
        }
        catch (EntityIdentifierException ex)
        {
            throw new UsageException(Utils.formatInvalidValueMsg(ENTITY_PROP, entityProp), this);
        }
    }

    public void setArgs(String[] args) throws Exception
    {
        for (int i = 0; i < args.length; i++)
        {
            String arg = args[i];

            if (arg.equals("-" + JAGJAR_PROP))
            {
                setJagJar(true);
            }
            else if (arg.equals("-" + DIR_PROP))
            {
                try
                {
                    setDir(args[i+1]);
                    i++;
                }
                catch (ArrayIndexOutOfBoundsException aioobe)
                {
                    throw new UsageException(this);
                }
            }
            else if (arg.startsWith("-"))
            {
                throw new UsageException(Utils.formatUnknownOptionMsg(arg), this);
            }
            else
            {
                setEntity(arg);
                break;
            }
        }
    }

    public void run() throws Exception
    {
        JTContext context = getContext();

        Utils.checkEntityIsSet(this, ENTITY_PROP, _entity);
        Utils.checkEntityType(this, _entity, VALID_ENTITY_TYPES);
        Utils.checkEntityExists(this, context, _entity);

        JRepository repository =
            new JRepository(context.getUserName(),
                            context.getPassword(),
                            context.getHostName(),
                            context.getPort(),
                            "rw");

        String fileName = getFileName();

        if (_jagjarProp == false)
        {
            exportJ2EE(repository, fileName, _entity);
        }
        else
        {
            exportJagJar(repository, fileName, _entity);
        }
    }

    protected void exportJ2EE(JRepository rep, String fileName, EntityIdentifier entity) throws JTException
    {
        JEarPackager packager = new JEarPackager(rep);

        DefaultStatusHandler statusHandler =
            new DefaultStatusHandler(getLogger());

        packager.setStatusDialogInterface(statusHandler);

        String fullName = null;

        try
        {
            File f = new File(fileName);
            fullName = f.getCanonicalPath();

            if (f.exists())
            {
                throw new JTException(_resources.getString("exportFileExistsMsg", 
                    new String[] { fullName }), this);
            }
        }
        catch (IOException ioex)
        {
            throw new JTException(_resources.getString("exportCannotCreateFileMsg", 
                new String[] { fileName }), this, ioex);
        }

        try
        {
            statusHandler.addConsoleMsg(_resources.getString("exportingMsg",
                new String[] { fullName }));

            if (entity.getType().equals(EntityIdentifier.APP_TYPE))
            {
	            packager.exportApplicationEar(entity.getName(), fullName);
            }
            else if (entity.getType().equals(EntityIdentifier.WEBAPP_TYPE))
            {
	            packager.exportWar(entity.getName(), fullName);
            }
            else if (entity.getType().equals(EntityIdentifier.PACKAGE_TYPE))
            {
                packager.exportEJBJar(entity.getName(), fullName);
            }
            else
            {
                Assert.assert(this, false,
                    MessageFormat.format("Unexpected entity type: {0}",
                        new String[] { entity.getType() }));
            }

            statusHandler.addConsoleMsg(_resources.getString("exportSuccessMsg",
                new String[] { entity.getType(), entity.getName(), fullName }));
        }
        catch (Exception ex)
        {
            throw new JTException(_resources.getString("exportFailedMsg"), this, ex);
        }
    }

    protected void exportJagJar(JRepository rep, String fileName, EntityIdentifier entity) throws JTException
    {
        String fullName = null;
        File file = null;

        try
        {
            file = new File(fileName);
            fullName = file.getCanonicalPath();

            if (file.exists())
            {
                throw new JTException(_resources.getString("exportFileExistsMsg", 
                    new String[] { fullName }), this);
            }
        }
        catch (IOException ioex)
        {
            throw new JTException(_resources.getString("exportCannotCreateFileMsg", 
                new String[] { fileName }), this, ioex);
        }

        String dirName = file.getParent();
        JPackager packager = null;

        if (entity.getType().equals(EntityIdentifier.APP_TYPE))
        {
            JApplication app = rep.getApplication(entity.getName());
            packager = new JPackager(app, rep, dirName);
        }
        else if (entity.getType().equals(EntityIdentifier.PACKAGE_TYPE))
        {
            JPackage pack = rep.getPackage(entity.getName());
            packager = new JPackager(pack, rep, dirName);
        }
        else if (entity.getType().equals(EntityIdentifier.WEBAPP_TYPE))
        {
            JWebApplication webapp = rep.getWebApplication(entity.getName());
            packager = new JPackager(webapp, rep, dirName);
        }
        else
        {
            Assert.assert(this, false,
                MessageFormat.format("Unexpected entity type: {0}",
                    new String[] { entity.getType() }));
        }

        if (packager != null && packager.createJarFile())
        {
            try
            {
                packager.MakeJaguarPackageV2();
                println(_resources.getString("exportSuccessMsg",
                    new String[] { entity.getType(), entity.getName(), fullName }));
            }
            catch (Exception ex)
            {
                throw new JTException(_resources.getString("exportFailedMsg"), this, ex);
            }
        }      
        else
        {
            throw new JTException(_resources.getString("exportCannotCreateFileMsg", 
                new String[] { fileName }), this);
        }
    }

    private String getFileName()
    {
        String dirName = _dirProp;

        if (dirName == null)
        {
            dirName = System.getProperty("user.dir");
        }

        String fileName = dirName +
            System.getProperty("file.separator") + 
            _entity.getName();

        if (_jagjarProp == true)
        {
            fileName += ".jar";
        }
        else
        {
            if (_entity.getType().equals(EntityIdentifier.APP_TYPE))
            {
                fileName += ".ear";
            }
            else if (_entity.getType().equals(EntityIdentifier.WEBAPP_TYPE))
            {
                fileName += ".war";
            }
            else if (_entity.getType().equals(EntityIdentifier.PACKAGE_TYPE))
            {
                fileName += ".jar";
            }
        }

        return fileName;
    }
}
