// 
// Confidential property of Sybase, Inc.
// (c) Copyright Sybase, Inc. 2000.
// All rights reserved
// 
package com.sybase.jaguar.management.jagtool.commands;

import java.io.*;
import java.text.*;
import java.util.*;
import com.sybase.jaguar.management.*;
import com.sybase.jaguar.management.jagtool.*;
import com.sybase.jaguar.system.Repository;
import com.sybase.jaguar.system.RepositoryHelper;
import com.sybase.jaguar.util.*;

/**
* Adds an entity to another entity.  Valid combinations
* of source and target entities are:
*
*   app -> server
*   package -> server
*   webapp -> server
*   cache -> server
*   servlet -> server
*   package -> app
*   webapp -> app
*   role -> package
*   role -> component
*   role -> method 
*/
public class InstallCommand extends BaseCommand
{
    public static final String SOURCE_PROP = "source";
    public static final String TARGET_PROP = "target";

    public static final String[] VALID_SERVER_SOURCE_ENTITY_TYPES =
    {
        EntityIdentifier.APP_TYPE,
        EntityIdentifier.CACHE_TYPE,
        EntityIdentifier.PACKAGE_TYPE,
        EntityIdentifier.SERVLET_TYPE,
        EntityIdentifier.WEBAPP_TYPE
    };

    public static final String[] VALID_APP_SOURCE_ENTITY_TYPES =
    {
        EntityIdentifier.PACKAGE_TYPE,
        EntityIdentifier.WEBAPP_TYPE
    };

    public static final String[] VALID_PACKAGE_SOURCE_ENTITY_TYPES =
    {
        EntityIdentifier.ROLE_TYPE
    };

    public static final String[] VALID_COMP_SOURCE_ENTITY_TYPES =
    {
        EntityIdentifier.ROLE_TYPE
    };

    public static final String[] VALID_METHOD_SOURCE_ENTITY_TYPES =
    {
        EntityIdentifier.ROLE_TYPE
    };

    public static final String[] VALID_TARGET_ENTITY_TYPES =
    {
        EntityIdentifier.SERVER_TYPE,
        EntityIdentifier.APP_TYPE,
        EntityIdentifier.PACKAGE_TYPE,
        EntityIdentifier.COMP_TYPE,
        EntityIdentifier.METHOD_TYPE
    };

    private static JTResourceManager _resources =
    	new JTResourceManager(InstallCommand.class);

    private EntityIdentifier _sourceEntity;
    private EntityIdentifier _targetEntity;

    public String getName()
    {
        return _resources.getString("installName");
    }

    public String getDescription()
    {
        return _resources.getString("installDesc");
    }

    public String getUsage()
    {
        return _resources.getString("installUsage");
    }

    public void setSource(String sourceProp) throws Exception
    {
        try
        {
            _sourceEntity = new EntityIdentifier(sourceProp);
        }
        catch (EntityIdentifierException ex)
        {
            throw new UsageException(Utils.formatInvalidValueMsg(SOURCE_PROP, sourceProp), this);
        }
    }

    public void setTarget(String targetProp) throws Exception
    {
        try
        {
            _targetEntity = new EntityIdentifier(targetProp);
        }
        catch (EntityIdentifierException ex)
        {
            throw new UsageException(Utils.formatInvalidValueMsg(TARGET_PROP, targetProp), this);
        }
    }

    public void setArgs(String[] args) throws Exception
    {
        if (args.length != 2)
        {
            throw new UsageException(this);
        }
        else
        {
            setSource(args[0]);
            setTarget(args[1]);
        }
    }
    
    public void run() throws Exception
    {
        JTContext context = getContext();

        Utils.checkEntityIsSet(this, SOURCE_PROP, _sourceEntity);
        Utils.checkEntityIsSet(this, TARGET_PROP, _targetEntity);

        checkEntityTypes(_sourceEntity, _targetEntity);

        Utils.checkEntityExists(this, context, _sourceEntity);
        Utils.checkEntityExists(this, context, _targetEntity);

        JRepository rep =
            new JRepository(context.getUserName(),
                            context.getPassword(),
                            context.getHostName(),
                            context.getPort(),
                            "rw");

        boolean didInstall = false;

        if (_targetEntity.getType().equals(EntityIdentifier.SERVER_TYPE))
        {
            if (_sourceEntity.getType().equals(EntityIdentifier.APP_TYPE))
            {
                didInstall = installAppInServer(rep, _sourceEntity, _targetEntity);
            }
            else if (_sourceEntity.getType().equals(EntityIdentifier.CACHE_TYPE))
            {
                didInstall = installCacheInServer(rep, _sourceEntity, _targetEntity);
            }
            else if (_sourceEntity.getType().equals(EntityIdentifier.PACKAGE_TYPE))
            {
                didInstall = installPackageInServer(rep, _sourceEntity, _targetEntity);
            }
            else if (_sourceEntity.getType().equals(EntityIdentifier.SERVLET_TYPE))
            {
                didInstall = installServletInServer(rep, _sourceEntity, _targetEntity);
            }
            else if (_sourceEntity.getType().equals(EntityIdentifier.WEBAPP_TYPE))
            {
                didInstall = installWebAppInServer(rep, _sourceEntity, _targetEntity);
            }
            else
            {
                Assert.assert(this, false,
                    MessageFormat.format("Unexpected source entity type: {0}",
                        new String[] { _sourceEntity.getType() }));
            }
        }
        else if (_targetEntity.getType().equals(EntityIdentifier.APP_TYPE))
        {
            if (_sourceEntity.getType().equals(EntityIdentifier.PACKAGE_TYPE))
            {
                didInstall = installPackageInApp(rep, _sourceEntity, _targetEntity);
            }
            else if (_sourceEntity.getType().equals(EntityIdentifier.WEBAPP_TYPE))
            {
                didInstall = installWebAppInApp(rep, _sourceEntity, _targetEntity);
            }
            else
            {
                Assert.assert(this, false,
                    MessageFormat.format("Unexpected source entity type: {0}",
                        new String[] { _sourceEntity.getType() }));
            }
        }
        else if (_targetEntity.getType().equals(EntityIdentifier.PACKAGE_TYPE))
        {
            if (_sourceEntity.getType().equals(EntityIdentifier.ROLE_TYPE))
            {
                didInstall = installRoleInPackage(rep, _sourceEntity, _targetEntity);
            }
            else
            {
                Assert.assert(this, false,
                    MessageFormat.format("Unexpected source entity type: {0}",
                        new String[] { _sourceEntity.getType() }));
            }
        }
        else if (_targetEntity.getType().equals(EntityIdentifier.COMP_TYPE))
        {
            if (_sourceEntity.getType().equals(EntityIdentifier.ROLE_TYPE))
            {
                didInstall = installRoleInComponent(rep, _sourceEntity, _targetEntity);
            }
            else
            {
                Assert.assert(this, false,
                    MessageFormat.format("Unexpected source entity type: {0}",
                        new String[] { _sourceEntity.getType() }));
            }
        }
        else if (_targetEntity.getType().equals(EntityIdentifier.METHOD_TYPE))
        {
            if (_sourceEntity.getType().equals(EntityIdentifier.ROLE_TYPE))
            {
                didInstall = installRoleInMethod(rep, _sourceEntity, _targetEntity);
            }
            else
            {
                Assert.assert(this, false,
                    MessageFormat.format("Unexpected source entity type: {0}",
                        new String[] { _sourceEntity.getType() }));
            }
        }
        else
        {
            Assert.assert(this, false,
                MessageFormat.format("Unexpected target entity type: {0}",
                    new String[] { _targetEntity.getType() }));
        }

        if (didInstall)
        {
            println(_resources.getString("installedEntityMsg",
                new String[] { _sourceEntity.getType(), _sourceEntity.getName(),
                               _targetEntity.getType(), _targetEntity.getName() }));
        }
        else
        {
            println(_resources.getString("installAlreadyInstalledMsg",
                new String[] { _sourceEntity.getType(), _sourceEntity.getName(),
                               _targetEntity.getType(), _targetEntity.getName() }));
        }
    }

    protected void checkEntityTypes(EntityIdentifier sourceEntity, EntityIdentifier targetEntity) throws UsageException
    {
        String[] validSourceTypes = null;

        if (targetEntity.getType().equals(EntityIdentifier.SERVER_TYPE))
        {
            validSourceTypes = VALID_SERVER_SOURCE_ENTITY_TYPES;
        }
        else if (targetEntity.getType().equals(EntityIdentifier.APP_TYPE))
        {
            validSourceTypes = VALID_APP_SOURCE_ENTITY_TYPES;
        }
        else if (targetEntity.getType().equals(EntityIdentifier.PACKAGE_TYPE))
        {
            validSourceTypes = VALID_PACKAGE_SOURCE_ENTITY_TYPES;
        }
        else if (targetEntity.getType().equals(EntityIdentifier.COMP_TYPE))
        {
            validSourceTypes = VALID_COMP_SOURCE_ENTITY_TYPES;
        }
        else if (targetEntity.getType().equals(EntityIdentifier.METHOD_TYPE))
        {
            validSourceTypes = VALID_METHOD_SOURCE_ENTITY_TYPES;
        }
        else
        {
            throw new UsageException(_resources.getString("invalidTargetEntityMsg",
                new String[] { targetEntity.getType(), 
                    Utils.arrayToString(VALID_TARGET_ENTITY_TYPES) }), this);
        }

        boolean validSourceType = false;

        String sourceType = sourceEntity.getType();

        for (int i = 0; i < validSourceTypes.length; i++)
        {
            if (sourceType.equals(validSourceTypes[i]))
            {
                validSourceType = true;
                break;
            }
        }

        if (validSourceType == false)
        {
            throw new UsageException(_resources.getString("invalidSourceEntityMsg",
                new String[] { sourceEntity.getType(), targetEntity.getType(),
                    Utils.arrayToString(validSourceTypes) }), this);
        }
    }

    protected boolean installAppInServer(JRepository rep, EntityIdentifier sourceEntity, EntityIdentifier targetEntity) throws JTException
    {
        JApplication app = (JApplication)sourceEntity.getEntity(rep);
        JServer server = (JServer)targetEntity.getEntity(rep);

        if (server.getApplication(sourceEntity.getName()) != null)
        {
            return false;
        }
        else
        {
            try
            {
                server.addApplication(app);
                rep.save();
            }
            catch (JException jex)
            {
                throw new JTException(_resources.getString("installFailedMsg",
                    new String[] { sourceEntity.getType(), sourceEntity.getName(),
                                   targetEntity.getType(), targetEntity.getName() }),
                    this, jex);
            }
        }

        return true;
    }

    protected boolean installCacheInServer(JRepository rep, EntityIdentifier sourceEntity, EntityIdentifier targetEntity) throws JTException
    {
        JConnectionCache cache = (JConnectionCache)sourceEntity.getEntity(rep);
        JServer server = (JServer)targetEntity.getEntity(rep);

        if (server.getConnectionCache(sourceEntity.getName()) != null)
        {
            return false;
        }
        else
        {
            try
            {
                server.addConnectionCache(cache);
                rep.save();
            }
            catch (JException jex)
            {
                throw new JTException(_resources.getString("installFailedMsg",
                    new String[] { sourceEntity.getType(), sourceEntity.getName(),
                                   targetEntity.getType(), targetEntity.getName() }),
                    this, jex);
            }
        }

        return true;
    }

    protected boolean installPackageInServer(JRepository rep, EntityIdentifier sourceEntity, EntityIdentifier targetEntity) throws JTException
    {
        JPackage pack = (JPackage)sourceEntity.getEntity(rep);
        JServer server = (JServer)targetEntity.getEntity(rep);

        if (server.getPackage(sourceEntity.getName()) != null)
        {
            return false;
        }
        else
        {
            JApplication app = pack.getApplication();

            if (app != null)
            {
                throw new JTException(_resources.getString("installAlreadyInstalledMsg2",
                    new String[] { sourceEntity.getType(), sourceEntity.getName(),
                                   EntityIdentifier.APP_TYPE, app.getName() }), this);
            }
            else
            {
                try
                {
                    server.addPackage(pack);
                    rep.save();
                }
                catch (JException jex)
                {
                    throw new JTException(_resources.getString("installFailedMsg",
                        new String[] { sourceEntity.getType(), sourceEntity.getName(),
                                       targetEntity.getType(), targetEntity.getName() }),
                        this, jex);
                }
            }
        }

        return true;
    }

    protected boolean installServletInServer(JRepository rep, EntityIdentifier sourceEntity, EntityIdentifier targetEntity) throws JTException
    {
        JServlet servlet = (JServlet)sourceEntity.getEntity(rep);
        JServer server = (JServer)targetEntity.getEntity(rep);

        if (server.getServlet(sourceEntity.getName()) != null)
        {
            return false;
        }
        else
        {
            try
            {
                server.addServlet(servlet);
                rep.save();
            }
            catch (JException jex)
            {
                throw new JTException(_resources.getString("installFailedMsg",
                    new String[] { sourceEntity.getType(), sourceEntity.getName(),
                                   targetEntity.getType(), targetEntity.getName() }),
                    this, jex);
            }
        }

        return true;
    }

    protected boolean installWebAppInServer(JRepository rep, EntityIdentifier sourceEntity, EntityIdentifier targetEntity) throws JTException
    {
        JWebApplication webapp = (JWebApplication)sourceEntity.getEntity(rep);
        JServer server = (JServer)targetEntity.getEntity(rep);

        if (server.getWebApplication(sourceEntity.getName()) != null)
        {
            return false;
        }
        else
        {
            JApplication app = webapp.getApplication();

            if (app != null)
            {
                throw new JTException(_resources.getString("installAlreadyInstalledMsg2",
                    new String[] { sourceEntity.getType(), sourceEntity.getName(),
                                   EntityIdentifier.APP_TYPE, app.getName() }), this);
            }
            else
            {
                try
                {
                    server.addWebApp(webapp);
                    rep.save();
                }
                catch (JException jex)
                {
                    throw new JTException(_resources.getString("installFailedMsg",
                        new String[] { sourceEntity.getType(), sourceEntity.getName(),
                                       targetEntity.getType(), targetEntity.getName() }),
                        this, jex);
                }
            }
        }

        return true;
    }

    protected boolean installPackageInApp(JRepository rep, EntityIdentifier sourceEntity, EntityIdentifier targetEntity) throws JTException
    {
        JPackage pack = (JPackage)sourceEntity.getEntity(rep);
        JApplication app = (JApplication)targetEntity.getEntity(rep);

        if (app.getPackage(sourceEntity.getName()) != null)
        {
            return false;
        }
        else
        {
            Enumeration enum = pack.getServers();

            if (enum != null && enum.hasMoreElements())
            {
                Vector serverNames = new Vector();

                while (enum.hasMoreElements())
                {
                    JServer server = (JServer)enum.nextElement();
                    serverNames.addElement(server.getName());
                }

                String[] s = new String[serverNames.size()];
                serverNames.copyInto(s);

                throw new JTException(_resources.getString("installAlreadyInstalledInServersMsg",
                    new String[] { _sourceEntity.getType(), _sourceEntity.getName(), Utils.arrayToString(s) }), this);
            }
            else
            {
                try
                {
                    app.addPackage(pack);
                    rep.save();
                }
                catch (JException jex)
                {
                    throw new JTException(_resources.getString("installFailedMsg",
                        new String[] { sourceEntity.getType(), sourceEntity.getName(),
                                       targetEntity.getType(), targetEntity.getName() }),
                        this, jex);
                }
            }
        }

        return true;
    }

    protected boolean installWebAppInApp(JRepository rep, EntityIdentifier sourceEntity, EntityIdentifier targetEntity) throws JTException
    {
        JWebApplication webapp = (JWebApplication)sourceEntity.getEntity(rep);
        JApplication app = (JApplication)targetEntity.getEntity(rep);

        if (app.getWebApp(sourceEntity.getName()) != null)
        {
            return false;
        }
        else
        {
            Enumeration enum = webapp.getServers();

            if (enum != null && enum.hasMoreElements())
            {
                Vector serverNames = new Vector();

                while (enum.hasMoreElements())
                {
                    JServer server = (JServer)enum.nextElement();
                    serverNames.addElement(server.getName());
                }

                String[] s = new String[serverNames.size()];
                serverNames.copyInto(s);

                throw new JTException(_resources.getString("installAlreadyInstalledInServersMsg",
                    new String[] { _sourceEntity.getType(), _sourceEntity.getName(), Utils.arrayToString(s) }), this);
            }
            else
            {
                try
                {
                    app.addWebApp(webapp);
                    rep.save();
                }
                catch (JException jex)
                {
                    throw new JTException(_resources.getString("installFailedMsg",
                        new String[] { sourceEntity.getType(), sourceEntity.getName(),
                                       targetEntity.getType(), targetEntity.getName() }),
                        this, jex);
                }
            }
        }

        return true;
    }

    protected boolean installRoleInPackage(JRepository rep, EntityIdentifier sourceEntity, EntityIdentifier targetEntity) throws JTException
    {
        JRole role = (JRole)sourceEntity.getEntity(rep);
        JPackage pack = (JPackage)targetEntity.getEntity(rep);

        if (pack.hasRoleMembership(sourceEntity.getName()))
        {
            return false;
        }
        else
        {
            try
            {
                pack.addRoleMembership(role);
                rep.save();
            }
            catch (JException jex)
            {
                throw new JTException(_resources.getString("installFailedMsg",
                    new String[] { sourceEntity.getType(), sourceEntity.getName(),
                                   targetEntity.getType(), targetEntity.getName() }),
                    this, jex);
            }
        }

        return true;
    }

    protected boolean installRoleInComponent(JRepository rep, EntityIdentifier sourceEntity, EntityIdentifier targetEntity) throws JTException
    {
        JRole role = (JRole)sourceEntity.getEntity(rep);
        JComponent comp = (JComponent)targetEntity.getEntity(rep);

        if (comp.hasRoleMembership(sourceEntity.getName()))
        {
            return false;
        }
        else
        {
            try
            {
                comp.addRoleMembership(role);
                rep.save();
            }
            catch (JException jex)
            {
                throw new JTException(_resources.getString("installFailedMsg",
                    new String[] { sourceEntity.getType(), sourceEntity.getName(),
                                   targetEntity.getType(), targetEntity.getName() }),
                    this, jex);
            }
        }

        return true;
    }

    protected boolean installRoleInMethod(JRepository rep, EntityIdentifier sourceEntity, EntityIdentifier targetEntity) throws JTException
    {
        JRole role = (JRole)sourceEntity.getEntity(rep);
        JMethod method = (JMethod)targetEntity.getEntity(rep);

        if (method.hasRoleMembership(sourceEntity.getName()))
        {
            return false;
        }
        else
        {
            try
            {
                method.addRoleMembership(role);
                rep.save();
            }
            catch (JException jex)
            {
                throw new JTException(_resources.getString("installFailedMsg",
                    new String[] { sourceEntity.getType(), sourceEntity.getName(),
                                   targetEntity.getType(), targetEntity.getName() }),
                    this, jex);
            }
        }

        return true;
    }
}
