// 
// Confidential property of Sybase, Inc.
// (c) Copyright Sybase, Inc. 2000.
// All rights reserved
// 
package com.sybase.jaguar.management.jagtool.commands;

import java.io.*;
import java.text.*;
import java.util.*;
import com.sybase.jaguar.management.*;
import com.sybase.jaguar.management.jagtool.*;
import com.sybase.jaguar.system.Repository;
import com.sybase.jaguar.system.RepositoryHelper;
import com.sybase.jaguar.util.*;

/**
* Removes an entity from another entity.  Valid combinations
* of source and target entities are:
*
*   app <- server
*   package <- server
*   webapp <- server
*   cache <- server
*   servlet <- server
*   package <- app
*   webapp <- app
*   role <- package
*   role <- component
*   role <- method
*/
public class RemoveCommand extends BaseCommand
{
    public static final String SOURCE_PROP = "source";
    public static final String TARGET_PROP = "target";

    public static final String[] VALID_SERVER_SOURCE_ENTITY_TYPES =
    {
        EntityIdentifier.APP_TYPE,
        EntityIdentifier.CACHE_TYPE,
        EntityIdentifier.PACKAGE_TYPE,
        EntityIdentifier.SERVLET_TYPE,
        EntityIdentifier.WEBAPP_TYPE
    };

    public static final String[] VALID_APP_SOURCE_ENTITY_TYPES =
    {
        EntityIdentifier.PACKAGE_TYPE,
        EntityIdentifier.WEBAPP_TYPE
    };

    public static final String[] VALID_PACKAGE_SOURCE_ENTITY_TYPES =
    {
        EntityIdentifier.ROLE_TYPE
    };

    public static final String[] VALID_COMP_SOURCE_ENTITY_TYPES =
    {
        EntityIdentifier.ROLE_TYPE
    };

    public static final String[] VALID_METHOD_SOURCE_ENTITY_TYPES =
    {
        EntityIdentifier.ROLE_TYPE
    };

    public static final String[] VALID_TARGET_ENTITY_TYPES =
    {
        EntityIdentifier.APP_TYPE,
        EntityIdentifier.SERVER_TYPE,
        EntityIdentifier.PACKAGE_TYPE,
        EntityIdentifier.COMP_TYPE,
        EntityIdentifier.METHOD_TYPE
    };

    private static JTResourceManager _resources =
    	new JTResourceManager(RemoveCommand.class);

    private EntityIdentifier _sourceEntity;
    private EntityIdentifier _targetEntity;

    public String getName()
    {
        return _resources.getString("removeName");
    }

    public String getDescription()
    {
        return _resources.getString("removeDesc");
    }

    public String getUsage()
    {
        return _resources.getString("removeUsage");
    }

    public void setSource(String sourceProp) throws Exception
    {
        try
        {
            _sourceEntity = new EntityIdentifier(sourceProp);
        }
        catch (EntityIdentifierException ex)
        {
            throw new UsageException(Utils.formatInvalidValueMsg(SOURCE_PROP, sourceProp), this);
        }
    }

    public void setTarget(String targetProp) throws Exception
    {
        try
        {
            _targetEntity = new EntityIdentifier(targetProp);
        }
        catch (EntityIdentifierException ex)
        {
            throw new UsageException(Utils.formatInvalidValueMsg(TARGET_PROP, targetProp), this);
        }
    }

    public void setArgs(String[] args) throws Exception
    {
        if (args.length != 2)
        {
            throw new UsageException(this);
        }
        else
        {
            setSource(args[0]);
            setTarget(args[1]);
        }
    }
    
    public void run() throws Exception
    {
        JTContext context = getContext();

        Utils.checkEntityIsSet(this, SOURCE_PROP, _sourceEntity);
        Utils.checkEntityIsSet(this, TARGET_PROP, _targetEntity);

        checkEntityTypes(_sourceEntity, _targetEntity);

        Utils.checkEntityExists(this, context, _sourceEntity);
        Utils.checkEntityExists(this, context, _targetEntity);

        JRepository rep =
            new JRepository(context.getUserName(),
                            context.getPassword(),
                            context.getHostName(),
                            context.getPort(),
                            "rw");

        boolean didRemove = false;

        if (_targetEntity.getType().equals(EntityIdentifier.SERVER_TYPE))
        {
            if (_sourceEntity.getType().equals(EntityIdentifier.APP_TYPE))
            {
                didRemove = removeAppFromServer(rep, _sourceEntity, _targetEntity);
            }
            else if (_sourceEntity.getType().equals(EntityIdentifier.CACHE_TYPE))
            {
                didRemove = removeCacheFromServer(rep, _sourceEntity, _targetEntity);
            }
            else if (_sourceEntity.getType().equals(EntityIdentifier.PACKAGE_TYPE))
            {
                didRemove = removePackageFromServer(rep, _sourceEntity, _targetEntity);
            }
            else if (_sourceEntity.getType().equals(EntityIdentifier.SERVLET_TYPE))
            {
                didRemove = removeServletFromServer(rep, _sourceEntity, _targetEntity);
            }
            else if (_sourceEntity.getType().equals(EntityIdentifier.WEBAPP_TYPE))
            {
                didRemove = removeWebAppFromServer(rep, _sourceEntity, _targetEntity);
            }
            else
            {
                Assert.assert(this, false,
                    MessageFormat.format("Unexpected source entity type: {0}",
                        new String[] { _sourceEntity.getType() }));
            }
        }
        else if (_targetEntity.getType().equals(EntityIdentifier.APP_TYPE))
        {
            if (_sourceEntity.getType().equals(EntityIdentifier.PACKAGE_TYPE))
            {
                didRemove = removePackageFromApp(rep, _sourceEntity, _targetEntity);
            }
            else if (_sourceEntity.getType().equals(EntityIdentifier.WEBAPP_TYPE))
            {
                didRemove = removeWebAppFromApp(rep, _sourceEntity, _targetEntity);
            }
            else
            {
                Assert.assert(this, false,
                    MessageFormat.format("Unexpected source entity type: {0}",
                        new String[] { _sourceEntity.getType() }));
            }
        }
        else if (_targetEntity.getType().equals(EntityIdentifier.PACKAGE_TYPE))
        {
            if (_sourceEntity.getType().equals(EntityIdentifier.ROLE_TYPE))
            {
                didRemove = removeRoleFromPackage(rep, _sourceEntity, _targetEntity);
            }
            else
            {
                Assert.assert(this, false,
                    MessageFormat.format("Unexpected source entity type: {0}",
                        new String[] { _sourceEntity.getType() }));
            }
        }
        else if (_targetEntity.getType().equals(EntityIdentifier.COMP_TYPE))
        {
            if (_sourceEntity.getType().equals(EntityIdentifier.ROLE_TYPE))
            {
                didRemove = removeRoleFromComponent(rep, _sourceEntity, _targetEntity);
            }
            else
            {
                Assert.assert(this, false,
                    MessageFormat.format("Unexpected source entity type: {0}",
                        new String[] { _sourceEntity.getType() }));
            }
        }
        else if (_targetEntity.getType().equals(EntityIdentifier.METHOD_TYPE))
        {
            if (_sourceEntity.getType().equals(EntityIdentifier.ROLE_TYPE))
            {
                didRemove = removeRoleFromMethod(rep, _sourceEntity, _targetEntity);
            }
            else
            {
                Assert.assert(this, false,
                    MessageFormat.format("Unexpected source entity type: {0}",
                        new String[] { _sourceEntity.getType() }));
            }
        }
        else
        {
            Assert.assert(this, false,
                MessageFormat.format("Unexpected target entity type: {0}",
                    new String[] { _targetEntity.getType() }));
        }

        if (didRemove)
        {
            println(_resources.getString("removedEntityMsg",
                new String[] { _sourceEntity.getType(), _sourceEntity.getName(),
                               _targetEntity.getType(), _targetEntity.getName() }));
        }
        else
        {
            println(_resources.getString("removeNotInstalledMsg",
                new String[] { _sourceEntity.getType(), _sourceEntity.getName(),
                               _targetEntity.getType(), _targetEntity.getName() }));

        }
    }

    protected void checkEntityTypes(EntityIdentifier sourceEntity, EntityIdentifier targetEntity) throws UsageException
    {
        String[] validSourceTypes = null;

        if (targetEntity.getType().equals(EntityIdentifier.SERVER_TYPE))
        {
            validSourceTypes = VALID_SERVER_SOURCE_ENTITY_TYPES;
        }
        else if (targetEntity.getType().equals(EntityIdentifier.APP_TYPE))
        {
            validSourceTypes = VALID_APP_SOURCE_ENTITY_TYPES;
        }
        else if (targetEntity.getType().equals(EntityIdentifier.PACKAGE_TYPE))
        {
            validSourceTypes = VALID_PACKAGE_SOURCE_ENTITY_TYPES;
        }
        else if (targetEntity.getType().equals(EntityIdentifier.COMP_TYPE))
        {
            validSourceTypes = VALID_COMP_SOURCE_ENTITY_TYPES;
        }
        else if (targetEntity.getType().equals(EntityIdentifier.METHOD_TYPE))
        {
            validSourceTypes = VALID_METHOD_SOURCE_ENTITY_TYPES;
        }
        else
        {
            throw new UsageException(_resources.getString("invalidTargetEntityMsg",
                new String[] { targetEntity.getType(), 
                    Utils.arrayToString(VALID_TARGET_ENTITY_TYPES) }), this);
        }

        boolean validSourceType = false;

        String sourceType = sourceEntity.getType();

        for (int i = 0; i < validSourceTypes.length; i++)
        {
            if (sourceType.equals(validSourceTypes[i]))
            {
                validSourceType = true;
                break;
            }
        }

        if (validSourceType == false)
        {
            throw new UsageException(_resources.getString("invalidSourceEntityMsg",
                new String[] { sourceEntity.getType(), targetEntity.getType(),
                    Utils.arrayToString(validSourceTypes) }), this);
        }
    }

    protected boolean removeAppFromServer(JRepository rep, EntityIdentifier sourceEntity, EntityIdentifier targetEntity) throws JTException
    {
        JApplication app = (JApplication)sourceEntity.getEntity(rep);
        JServer server = (JServer)targetEntity.getEntity(rep);

        if (server.getApplication(sourceEntity.getName()) == null)
        {
            return false;
        }
        else
        {
            try
            {
                server.removeApplication(app);
                rep.save();
            }
            catch (JException jex)
            {
                throw new JTException(_resources.getString("removeFailedMsg",
                    new String[] { sourceEntity.getType(), sourceEntity.getName(),
                                   targetEntity.getType(), targetEntity.getName() }),
                    this, jex);
            }
        }

        return true;
    }

    protected boolean removeCacheFromServer(JRepository rep, EntityIdentifier sourceEntity, EntityIdentifier targetEntity) throws JTException
    {
        JConnectionCache cache = (JConnectionCache)sourceEntity.getEntity(rep);
        JServer server = (JServer)targetEntity.getEntity(rep);

        if (server.getConnectionCache(sourceEntity.getName()) == null)
        {
            return false;
        }
        else
        {
            try
            {
                server.removeConnectionCache(cache);
                rep.save();
            }
            catch (JException jex)
            {
                throw new JTException(_resources.getString("removeFailedMsg",
                    new String[] { sourceEntity.getType(), sourceEntity.getName(),
                                   targetEntity.getType(), targetEntity.getName() }),
                    this, jex);
            }
        }

        return true;
    }

    protected boolean removePackageFromServer(JRepository rep, EntityIdentifier sourceEntity, EntityIdentifier targetEntity) throws JTException
    {
        JPackage pack = (JPackage)sourceEntity.getEntity(rep);
        JServer server = (JServer)targetEntity.getEntity(rep);

        if (server.getPackage(sourceEntity.getName()) == null)
        {
            return false;
        }
        else
        {
            try
            {
                server.removePackage(pack);
                rep.save();
            }
            catch (JException jex)
            {
                throw new JTException(_resources.getString("removeFailedMsg",
                    new String[] { sourceEntity.getType(), sourceEntity.getName(),
                                   targetEntity.getType(), targetEntity.getName() }),
                    this, jex);
            }
        }

        return true;
    }

    protected boolean removeServletFromServer(JRepository rep, EntityIdentifier sourceEntity, EntityIdentifier targetEntity) throws JTException
    {
        JServlet servlet = (JServlet)sourceEntity.getEntity(rep);
        JServer server = (JServer)targetEntity.getEntity(rep);

        if (server.getServlet(sourceEntity.getName()) == null)
        {
            return false;
        }
        else
        {
            try
            {
                server.removeServlet(servlet);
                rep.save();
            }
            catch (JException jex)
            {
                throw new JTException(_resources.getString("removeFailedMsg",
                    new String[] { sourceEntity.getType(), sourceEntity.getName(),
                                   targetEntity.getType(), targetEntity.getName() }),
                    this, jex);
            }
        }

        return true;
    }

    protected boolean removeWebAppFromServer(JRepository rep, EntityIdentifier sourceEntity, EntityIdentifier targetEntity) throws JTException
    {
        JWebApplication webapp = (JWebApplication)sourceEntity.getEntity(rep);
        JServer server = (JServer)targetEntity.getEntity(rep);

        if (server.getWebApplication(sourceEntity.getName()) == null)
        {
            return false;
        }
        else
        {
            try
            {
                server.removeWebApp(webapp);
                rep.save();
            }
            catch (JException jex)
            {
                throw new JTException(_resources.getString("removeFailedMsg",
                    new String[] { sourceEntity.getType(), sourceEntity.getName(),
                                   targetEntity.getType(), targetEntity.getName() }),
                    this, jex);
            }
        }

        return true;
    }

    protected boolean removePackageFromApp(JRepository rep, EntityIdentifier sourceEntity, EntityIdentifier targetEntity) throws JTException
    {
        JPackage pack = (JPackage)sourceEntity.getEntity(rep);
        JApplication app = (JApplication)targetEntity.getEntity(rep);

        if (app.getPackage(sourceEntity.getName()) == null)
        {
            return false;
        }
        else
        {
            try
            {
                app.removePackage(pack);
                rep.save();
            }
            catch (JException jex)
            {
                throw new JTException(_resources.getString("removeFailedMsg",
                    new String[] { sourceEntity.getType(), sourceEntity.getName(),
                                   targetEntity.getType(), targetEntity.getName() }),
                    this, jex);
            }
        }

        return true;
    }

    protected boolean removeWebAppFromApp(JRepository rep, EntityIdentifier sourceEntity, EntityIdentifier targetEntity) throws JTException
    {
        JWebApplication webapp = (JWebApplication)sourceEntity.getEntity(rep);
        JApplication app = (JApplication)targetEntity.getEntity(rep);

        if (app.getWebApp(sourceEntity.getName()) == null)
        {
            return false;
        }
        else
        {
            try
            {
                app.removeWebApp(webapp);
                rep.save();
            }
            catch (JException jex)
            {
                throw new JTException(_resources.getString("removeFailedMsg",
                    new String[] { sourceEntity.getType(), sourceEntity.getName(),
                                   targetEntity.getType(), targetEntity.getName() }),
                    this, jex);
            }
        }

        return true;
    }

    protected boolean removeRoleFromPackage(JRepository rep, EntityIdentifier sourceEntity, EntityIdentifier targetEntity) throws JTException
    {
        JRole role = (JRole)sourceEntity.getEntity(rep);
        JPackage pack = (JPackage)targetEntity.getEntity(rep);

        if (!pack.hasRoleMembership(sourceEntity.getName()))
        {
            return false;
        }
        else
        {
            try
            {
                pack.removeRoleFromMembership(role);
                rep.save();
            }
            catch (JException jex)
            {
                throw new JTException(_resources.getString("removeFailedMsg",
                    new String[] { sourceEntity.getType(), sourceEntity.getName(),
                                   targetEntity.getType(), targetEntity.getName() }),
                    this, jex);
            }
        }

        return true;
    }

    protected boolean removeRoleFromComponent(JRepository rep, EntityIdentifier sourceEntity, EntityIdentifier targetEntity) throws JTException
    {
        JRole role = (JRole)sourceEntity.getEntity(rep);
        JComponent comp = (JComponent)targetEntity.getEntity(rep);

        if (!comp.hasRoleMembership(sourceEntity.getName()))
        {
            return false;
        }
        else
        {
            try
            {
                comp.removeRoleFromMembership(role);
                rep.save();
            }
            catch (JException jex)
            {
                throw new JTException(_resources.getString("removeFailedMsg",
                    new String[] { sourceEntity.getType(), sourceEntity.getName(),
                                   targetEntity.getType(), targetEntity.getName() }),
                    this, jex);
            }
        }

        return true;
    }

    protected boolean removeRoleFromMethod(JRepository rep, EntityIdentifier sourceEntity, EntityIdentifier targetEntity) throws JTException
    {
        JRole role = (JRole)sourceEntity.getEntity(rep);
        JMethod method = (JMethod)targetEntity.getEntity(rep);

        if (!method.hasRoleMembership(sourceEntity.getName()))
        {
            return false;
        }
        else
        {
            try
            {
                method.removeRoleFromMembership(role);
                rep.save();
            }
            catch (JException jex)
            {
                throw new JTException(_resources.getString("removeFailedMsg",
                    new String[] { sourceEntity.getType(), sourceEntity.getName(),
                                   targetEntity.getType(), targetEntity.getName() }),
                    this, jex);
            }
        }

        return true;
    }
}
