package com.sybase.jaguar.sample.components;

/**
* Imports
*/

import java.io.*;
import java.util.*;
import java.sql.*;

import com.sybase.jaguar.util.*;
import com.sybase.jaguar.sql.*;
import com.sybase.jaguar.server.*;
import com.sybase.jaguar.jcm.*;

/**
* This class is a superclass for all Jaguar components.  This class
* has methods to handle creating and maintaining connections and connection
* caches.  As well, this allows you to run your component in a DEBUG mode.
*
*  
* 
* @author David Lance Wolf
* @author Gerry Moses, Sybase Professional Services
* @version 1.0
*
* copyright 1998 Sybase Inc, ALL RIGHTS RESERVED
* 
*/


public class JaguarComponentFramework extends Object
{
	/**
	* Instance variables
	*	
	*/

	private  String USER 		= 	"dba";
	private  String PASSWORD 	= 	"sql";
	private  String URL 		= 	"jdbc:odbc:Jaguar SVU Sample";
	private  int    isolation 	= 	1;


	public final int WAIT_FOR_AVAILABLE_CONNECTION 	= 	JCMCache.JCM_WAIT;
	public final int FORCE_CONNECTION_OPEN 		= 	JCMCache.JCM_FORCE;
	public final int EXCEPTION_ON_CACHE_FULL 	= 	JCMCache.JCM_NOWAIT;

	protected boolean DEBUG		= 	true;
	public JCMCache cache 		= 	null;
	public Connection connection 	= 	null;
	
	/**
	* Contructs the component and builds a cache reference. 
	*
	* @throws Exception
	*/

	public JaguarComponentFramework() throws Exception
   	{
		setCacheContext();
		if(DEBUG) printToErrorLog(this.toString() + " was constructed");
	}


	/**
	* Contructs the component and builds a cache reference. This constructor allows you to override
	* the initial instance members.
	*
	* @parameter String Username for database
	* @parameter String Password for database
	* @parameter String URL for database
	* @throws Exception
	*/

	public JaguarComponentFramework(String ps_username, 
				String ps_password, String ps_URL) throws Exception
   	{
		USER = ps_username;
		PASSWORD = ps_password;
		URL = ps_URL;
		if (DEBUG)
			printToErrorLog(
				"JaguarComponentFramework:parameters:" + 
				USER + ":" +
				PASSWORD + ":" +
				URL);
		setCacheContext();
		if(DEBUG) printToErrorLog(this.toString() + " was constructed with parameters");
	}


	/**
	*
	* Contructs the component and builds a cache reference.  This constructor
	* takes the parameters as set in an INI file.
	*
	* @parameter File file.  The INI file to be read.
	* @throws FileNotFoundException
	* @throws Exception
	*
	*/

	public JaguarComponentFramework(File file) throws FileNotFoundException, Exception
	{
		Properties props = new Properties();
		FileInputStream in = new FileInputStream(file);
		props.load(in);
	 	USER = props.getProperty("user");
		PASSWORD = props.getProperty("password");
		URL = props.getProperty("url");
		if (DEBUG)
			printToErrorLog(
				"ImpactJaguarComponent:parameters:" + 
				USER + ":" +
				PASSWORD + ":" +
				URL);
		setCacheContext();
		if(DEBUG) printToErrorLog(this.toString() + " was constructed from INI file with parameters");
	}

	public JaguarComponentFramework(String name) throws Exception
	{
		setCacheContext(name);
		if(DEBUG) printToErrorLog(this.toString() + " was constructed with named cache " + name);
	}

	/**
	*  Prints a message to the Jaguar errorlog with a timestamp.
	*
	*  @parameter String Message being written.
	*
	*/

	protected void printToErrorLog(String message)
	{
		try
		{
			Jaguar.writeLog(true, ":framework: " + message);
		}
		catch(JException je)
		{
			System.out.println("PANIC!! Unable to log error to errorlog, dumping stack!");
			System.out.println("Error was: " + message);
			je.printStackTrace();
		}
		catch(Exception e)
		{
			System.out.println("PANIC!! Unable to log error to errorlog, dumping stack!");
			System.out.println("Error was: " + message);
			e.printStackTrace();
		}
	}

	/**
	* Builds a cache context.
	*
	* @throws Exception
	*/

	protected void setCacheContext() throws Exception
	{
		try
		{
			this.cache = JCM.getCache(USER,PASSWORD,URL);
			if(DEBUG) printToErrorLog("Cache Built for " + this.toString());
		}
		catch(JException je)
		{
			printToErrorLog("JException thrown in building cache context of " + this.toString());
			printToErrorLog("Exception was: " + je.toString());
			je.printStackTrace();
			throw je;	
		}
		catch(Exception e)
		{
			printToErrorLog("Unexpected Exception thrown in cache context of " + this.toString());
			printToErrorLog("Exception was: " + e.toString());
			e.printStackTrace();
			throw e;
		}

		if(cache == null)
		{
			printToErrorLog("Cache was null for " + this.toString());
			throw new JException("Cache was null after construction");
		}
	}

	protected void setCacheContext(String name) throws Exception
	{
		try
		{
			this.cache = JCM.getCacheByName(name);
			if(DEBUG) printToErrorLog("Cache Built for " + this.toString());
		}
		catch(JException je)
		{
			printToErrorLog("JException thrown in building cache context of " + this.toString());
			printToErrorLog("Exception was: " + je.toString());
			je.printStackTrace();
			throw je;	
		}
		catch(Exception e)
		{
			printToErrorLog("Unexpected Exception thrown in cache context of " + this.toString());
			printToErrorLog("Exception was: " + e.toString());
			e.printStackTrace();
			throw e;
		}

		if(cache == null)
		{
			printToErrorLog("Cache was null for " + this.toString());
			throw new JException("Cache was null after construction");
		}
	}		

	/**
	* Drops the cache context.  Can be used to clean up memory.
	*
	*/

	protected void destroyCacheContext()
	{
		cache = null;
		if(DEBUG) printToErrorLog("Cache reference set to null");
	}

	/**
	* Creates a database connection and initializes the instance connection.
	*
	* @throws JException
	* @throws SQLException
	* @throws JConnectionNotFoundException
	*
	*/

	protected void setConnection(int type)throws JException, SQLException, JConnectionNotFoundException
	{
		if(connection != null || cache == null)
		{
			if(DEBUG)
			{
				printToErrorLog("Cache or Connection not ready");
				printToErrorLog("Cache = " + cache.toString());
				printToErrorLog("Connection = " + connection.toString());
			}
			throw new JException("Cache or Connection not ready");

		}

		connection = cache.getConnection(type);
		if(DEBUG) printToErrorLog("Connection built for " + this.toString());
	}

	/**
	* Releases the instance connection back into the cache.  Resets the connection
	* settings to default in case they were reset.
	*
	*/

	protected void dropConnection() throws SQLException
	{
		setConnectionInfo();
		cache.releaseConnection(connection);
		connection = null;
		if(DEBUG) printToErrorLog("Connection dropped for " + this.toString());
	}

	/**
	* Destroys a connection without ever releasing it into the cache for reuse.
	*
	* @throws SQLException
	*/

	protected void destroyConnection() throws SQLException
	{
		cache.dropConnection(connection);
		connection = null;
		if(DEBUG) printToErrorLog("Connection destroyed for " + this.toString());
	}
	
	/**
	* Resets the connection objects parameters. If you have other specific connection parameters that must be
	* set, you can set them in this method.
	*
	* @throws SQLException
	*/

	protected void setConnectionInfo() throws SQLException
	{
		try {
				
			if ( connection.getTransactionIsolation() != connection.TRANSACTION_NONE )
			{
				connection.setTransactionIsolation(isolation);
			}
			
			if(DEBUG) printToErrorLog("Connection parameters reset for " + this.toString());
		}
		
		catch ( SQLException sqle) 
		{
			printToErrorLog ("Unable to reset connection info " + sqle.getMessage());
		}

	}


	/**
	* Overriding the superclass finalize to print to the errorlog when this class is
	* garbage collected. This is nice if the runtime monitoring shows you leaking components.
	*
	*/

	protected void finalize()
	{
		if(DEBUG) printToErrorLog(this.toString() + " was garbage collected");
	}

	/**
	* Sets the debug state of the component.  If the debug is set to true, debug
	* statements will go in the Jaguar errorlog. This is a public method, so that if wanted, the
	* DEBUG state could be set by a client stub.
	*
	* @paramater boolean Debug.
	*/

	protected void setDebug(boolean debug)
	{
		DEBUG = debug;
		printToErrorLog("Debug state set to " + DEBUG);
	}

	/**
	* Sets the userid of the connection cache in Jaguar.
	*
	* @parameter String user.  The userid.
	*
	*/

	protected void setUser(String user)
	{
		this.USER = user;
	}

	/**
	* Sets the password of the connection cache in Jaguar.
	*
	* @parameter String password.  The password.
	*
	*/

	protected void setPassword(String password)
	{
		this.PASSWORD = password;
	}

	/**
	* Sets the SERVER URL of the connection cache in Jaguar.
	*
	* @parameter String server URL of the server.
	*
	*/

	protected void setServer(String server)
	{
		this.URL = server;
	}

	/**
	* Sets the isolation level of transactions for this connection.
	*
	* @throws JException
	*
	*/

	protected void setIsolationLevel(int i) throws JException
	{
		if(i < 0 || i > 3) throw new JException("Invalid isolation level " + i );
		else this.isolation = i;
	}

}
