package com.sybase.jaguar.sample.svu;

import com.sybase.jaguar.sample.components.*;
import com.sybase.jaguar.beans.enterprise.*;
import java.sql.*;
import org.omg.CORBA.*;
import com.sybase.CORBA.jdbc11.*;

/**
* This is a CORBA/EJB component using IDL datatypes that executes login
* transactions against the SVU Demo Application.  This component is EJB stateless
* and is instance pooled. At the completion of each method call, this component will
* be deactivated and its instance made available to other stubs.  This presents a very
* scalable component.  Care must be taken not to store any stateful instance members, as their
* state cannot be maintained across method invocations.   
*
* Business Case:
*
*	The application needs to be able to confirm that this is a valid student, as well
* 	as gather that users Name and Social Security Information
*
* Solution:
*
*	We have created an EJB stateless component. 
*
* @author David Lance Wolf, Jaguar Product Team
* @version 1.0
* 
* COPYRIGHT 1998 Sybase Inc
* ALL RIGHTS RESERVED
*
*/

public class SVUEnrollmentImpl extends JaguarComponentFramework implements ServerBean
{

	/** 
	* Instamce members
	*
	*/

	InstanceContext state = null;
	SVUMetaData metadata = null;


	/**
	* Contructs the component making the call to the superclasses constructor which builds a cache reference.
	* This component is going to make intercomponent calls, and is using a CORBA stub which
	* is built in the contructor.  Note that you MUST use stubs for intercomponent calls in order
	* for Jaguar to	
	*
	* 1) Handle the transaction across components
	* 2) Use the InstanceContext and SharedObjects
	*
	* @throws Exception
	*
	*/

	public SVUEnrollmentImpl() throws Exception
	{
		super();
		java.util.Properties props = new java.util.Properties();
		props.put("org.omg.CORBA.ORBClass", "com.sybase.CORBA.ORB");
		ORB orb = ORB.init((String[])null, props);		
 		metadata = SVUMetaDataHelper.narrow(orb.string_to_object("SVU/SVUMetaData"));
	}

	/**
	* Activates the component and builds a connection. In an instance pooled, early deactivated
	* component, activate() is called before each method invocation as the already instantiated
	* component enters the Activate state.
	*
	* @param InstanceContext state
	* @param String notUsed
	*
	* @throws EnterpriseBeanException
	*
	*/

	public void activate(InstanceContext ctx, String notUsed) throws EnterpriseBeanException
	{
		state = ctx;
		if(connection == null)
		{
			try
			{
				setConnection(WAIT_FOR_AVAILABLE_CONNECTION);
				return;
			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new EnterpriseBeanException(e.getMessage());
			}
		}
	}

	/** 
	* Returns a list of majors by making an intercomponent call to the SVUMetaData component.  This
	* is a good example of seperating the data access layer from the business rules layer in component
	* programming.  We return the ResultSet using the TabularResults::ResultSet IDL datatype.
	*
	* @returns TabularResults.ResultSet
	* @throws DatabaseError
	*
	*/

	public TabularResults.ResultSet getMajors() throws DatabaseError
	{
		try
		{
			TabularResults.ResultSet trs = metadata.getMajors();
			state.completeWork();
			return trs;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new DatabaseError(e.getMessage());
		}
	}
	
	/** 
	* Returns a list of semesters by making an intercomponent call to the SVUMetaData component.  This
	* is a good example of seperating the data access layer from the business rules layer in component
	* programming.  We return the ResultSet using the TabularResults::ResultSet IDL datatype.
	*
	* @returns TabularResults.ResultSet
	* @throws DatabaseError
	*
	*/

	public TabularResults.ResultSet getSemesters() throws DatabaseError
	{
		try
		{
			TabularResults.ResultSet trs = metadata.getSemesters();
			state.completeWork();
			return trs;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new DatabaseError(e.getMessage());
		}
	}

	/** 
	* Returns a list of courses taken for a student in that major and semester.
	* We return the ResultSet using the TabularResults::ResultSet IDL datatype.
	*
	* @param String Social Security Number in the form XXX-XX-XXXX
	* @param String semester
	* @param String major
	* @returns TabularResults.ResultSet
	* @throws DatabaseError
	*
	*/

	public TabularResults.ResultSet getCoursesTaken(String ssn, String semester, String major) throws DatabaseError
	{
		try
		{
			String sql = "SELECT a.course_id, a.course_name, a.instructor_id, a.units ";
			sql += "FROM all_courses a,  offered_courses o, enrollments e ";
			sql += "WHERE o.course_id = a.course_id ";
			sql += "AND e.offered_course_id = o.offered_course_id ";
			sql += "AND a.dept_id = ? ";
			sql += "AND o.semester_id = ? ";
			sql += "AND e.student_id = ?";
			PreparedStatement pstmt = connection.prepareStatement(sql);
			pstmt.setString(1, major);
			pstmt.setString(2, semester);
			pstmt.setString(3, ssn);
			ResultSet rs = pstmt.executeQuery();
			TabularResults.ResultSet trs = IDL.getResultSet(rs);
			pstmt.close();
			rs = null;
			state.completeWork();
			return trs;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new DatabaseError(e.getMessage());
		}
	}


	/** 
	* Returns a list of courses offered for in that major and semester.
	* We return the ResultSet using the TabularResults::ResultSet IDL datatype.
	*
	* @param String Social Security Number in the form XXX-XX-XXXX
	* @param String semester
	* @param String major
	* @returns TabularResults.ResultSet
	* @throws DatabaseError
	*
	*/

	public TabularResults.ResultSet getCoursesOffered(String semester, String major) throws DatabaseError
	{
		try
		{
			String sql = "SELECT a.course_id, a.course_name, a.instructor_id, a.units ";
			sql += "FROM all_courses a,  offered_courses o ";
			sql += "WHERE o.course_id = a.course_id ";
			sql += "AND a.dept_id = ? ";
			sql += "AND o.semester_id = ? ";
			PreparedStatement pstmt = connection.prepareStatement(sql);
			pstmt.setString(1, major);
			pstmt.setString(2, semester);
			ResultSet rs = pstmt.executeQuery();
			TabularResults.ResultSet trs = IDL.getResultSet(rs);
			pstmt.close();
			rs = null;
			state.completeWork();
			return trs;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new DatabaseError(e.getMessage());
		}
	}

	/** 
	* Returns a list of courses remaining for a student in that major and semester.
	* We return the ResultSet using the TabularResults::ResultSet IDL datatype.
	*
	* @param String Social Security Number in the form XXX-XX-XXXX
	* @param String semester
	* @param String major
	* @returns TabularResults.ResultSet
	* @throws DatabaseError
	*
	*/

	public TabularResults.ResultSet getCoursesRemaining(String ssn, String semester, String major) throws DatabaseError
	{
		try
		{
			String sql = "SELECT a.course_id, a.course_name, a.instructor_id, a.units ";
			sql += "FROM all_courses a,  offered_courses o ";
			sql += "WHERE o.course_id = a.course_id ";
			sql += "AND a.dept_id = ? ";
			sql += "AND o.semester_id = ? ";
			sql += "and o.offered_course_id NOT IN (SELECT offered_course_id FROM enrollments WHERE student_id = ? ) ";
			PreparedStatement pstmt = connection.prepareStatement(sql);
			pstmt.setString(1, major);
			pstmt.setString(2, semester);
			pstmt.setString(3, ssn);
			ResultSet rs = pstmt.executeQuery();
			TabularResults.ResultSet trs = IDL.getResultSet(rs);
			pstmt.close();
			rs = null;
			state.completeWork();
			return trs;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new DatabaseError(e.getMessage());
		}
	}

	/**
	* Registers a student for a class.  If we catch any Exceptions, we will doom the transaction.
	*
	* @param String Social Security Number in the form XXX-XX-XXXX
	* @param String course
	* @throws DatabaseError
	*
	*/

	public void addClass(String ssn, String course) throws DatabaseError
	{
		try
		{
			String sql = "INSERT INTO enrollments (offered_course_id, student_id) ";
			sql += "VALUES(?,?) ";
			PreparedStatement pstmt = connection.prepareStatement(sql);
			pstmt.setString(1, course);
			pstmt.setString(2, ssn);
			pstmt.executeUpdate();
			pstmt.close();
			pstmt = null;
			state.completeWork();
			return;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			state.rollbackWork();
			throw new DatabaseError(e.getMessage());
		}
	}

	/**
	* Informs the Jaguar Server that this component wishes to be instance
	* pooled.  
	*
	* @returns boolean Indicating if this component can be reused
	*
	*/

	public boolean canReuse()
	{
		return true;
	}

	/**
	* Deactivates the component. Any Exceptions thrown in deactivate() will cause the 
	* transaction to roll back.
	*
	* @throws EnterpriseBeanException
	*
	*/

	public void deactivate() throws EnterpriseBeanException
	{
		try
		{
			dropConnection();
			return;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new EnterpriseBeanException(e.getMessage());
		}
	}


	/**
	* Destroys the component.  This is a good place to clean up memory.
	*
	*/

	public void destroy()
	{
		destroyCacheContext();
	}

}
