package com.sybase.jaguar.sample.svu;

import com.sybase.CORBA.jdbc11.*;
import com.sybase.jaguar.beans.enterprise.*;
import java.sql.*;
import com.sybase.jaguar.sample.components.*;

/**
* This is a CORBA/EJB component using IDL datatypes that executes financial
* transactions against the SVU Demo Application.  This component is EJB stateful
* and is not instance pooled. We have also emulated the disallowCommit() model
* by using a boolean flag called isCommitable.  
*
* Business Case:
*
*	Students should be able to transfer money out of their accounts to
*	pay for classes, as well as transfer money in to replenish their accounts.
*	The balance on a session may fall below $0, but must be above $0 before
*	submitting their transactions in batch.
*
* Solution:
*
*	We have created an EJB stateful component which emulates the disallowCommit()
*	model by setting a boolean to false whenever a method completes with
* 	the balance below $0.
*
* @author David Lance Wolf, Jaguar Product Team
* @version 1.0
* 
* COPYRIGHT 1998 Sybase Inc
* ALL RIGHTS RESERVED
*
*/
 
public class SVUFinanceImpl extends JaguarComponentFramework implements ServerBean
{

	/** 
	* Instamce members
	*
	*/

	private InstanceContext state;
	private String id;
	private float fee;
	private boolean isCommitable;

	/**
	* Contructs the component setting the isCommitable flag to true, and the fee
	* to $0.  The call to the superclasses constructor builds a cache reference
	*
	* @throws Exception
	*
	*/

	public SVUFinanceImpl() throws Exception
	{
		super();
		isCommitable = true;
		fee = 0;
	}

	/**
	* Informs the Jaguar Server that this component does not wish to be instance
	* pooled.  This is common for stateful components.
	*
	* @returns boolean Indicating if this component can be reused
	*
	*/

	public boolean canReuse()
	{
		return false;
	}

	/**
	* Activates the component and builds a connection.
	*
	* @param InstanceContext state
	* @param String notUsed
	*
	* @throws EnterpriseBeanException
	*
	*/

	public void activate(InstanceContext ctx, String notUsed) throws EnterpriseBeanException
	{
		state = ctx;
		try
		{
			setConnection(WAIT_FOR_AVAILABLE_CONNECTION);
			return;
		}
		catch(Exception e)
		{
			throw new EnterpriseBeanException(e.getMessage());
		}
	}

	/**
	* Sets the instance member id which is a String in the form XXX-XX-XXXX.
	*
	* @param String Social Security Number
	*
	*/

	public void setId(String value)
	{
		id = value;
	}

	/**
	* Returns the current fee for this instance. This is an example of the ability
	* to pass values as return types.
	*
	* @returns float fee
	*
	*/

	public float getFee()
	{
		return fee;
	}

	/**
	* Returns the currnet balance in the account.
	*
	* @returns float balance
	* @throws DatabaseError
	*
	*/
 
	public float getBalance() throws DatabaseError
	{
		try
		{
			String sql = "SELECT balance FROM account WHERE student_id = ?";
			PreparedStatement pstmt = connection.prepareStatement(sql);
			pstmt.setString(1, id);
			ResultSet rs = pstmt.executeQuery();
			rs.next();
			float balance = rs.getFloat(1);
			pstmt.close();
			state.continueWork();
			return balance;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new DatabaseError(e.getMessage());
		}
	}

	/**
	* Sets the balance in the account.  This method is private and not defined in
	* the SVU::SVUFinance IDL.  It is therefore not exposed.
	*
	* @param float balance
	* @throws DatabaseError
	*
	*/
 
	private void setBalance(float balance) throws DatabaseError
	{
		try
		{
			String sql = "UPDATE account ";
			sql += "SET balance = ? ";
			sql += "WHERE student_id = ? ";
			PreparedStatement pstmt = connection.prepareStatement(sql);
			pstmt.setFloat(1, balance);
			pstmt.setString(2, id);
			pstmt.executeUpdate();
			pstmt.close();
			pstmt = null;
			state.continueWork();
			return;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new DatabaseError(e.getMessage());
		}
	}

	/**
	* Returns the financial history for this user. The values are returned using
	* the IDL TabularResults::ResultSet object.  
	*
	* @returns TabularResults.ResultSet
	* @throws DatabaseError
	*
	*/

	public TabularResults.ResultSet getFinancialHistory() throws DatabaseError
	{
		try
		{
			String sql = "SELECT type, transactiondate, amount, remarks ";
			sql += "FROM fintrac ";
			sql += "WHERE student_id = ?";
			PreparedStatement pstmt = connection.prepareStatement(sql);
			pstmt.setString(1, id);
			ResultSet rs = pstmt.executeQuery();
			TabularResults.ResultSet trs = IDL.getResultSet(rs);
			pstmt.close();
			pstmt = null;
			state.continueWork();
			return trs;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new DatabaseError(e.getMessage());
		}
	}
	
	/**
	* Adds a new financial transaction entry into the tracking tables.
	*
	* @param String type of transaction
	* @param float amount of transaction
	* @param String remarks
	* @throws DatabaseError
	*
	*/

	public void setFinancialData(String type, float amount, String remarks)
		throws DatabaseError
	{
		float balance = 0;
		try
		{
			balance = getBalance();
		}
		catch(DatabaseError e)
		{
			e.printStackTrace();
			throw e;
		}
		if(type.toUpperCase().equals("TRANSFER"))
		{
			balance -= amount;
			if(balance < 0)
			{
				isCommitable = false;
			}
		}
		else
		{
			balance += amount;
			if(balance > 0)
			{
				isCommitable = true;
			}
		}
		fee += 5;
		try
		{
			setBalance(balance);
			String sql = "INSERT INTO fintrac (student_id, type, transactiondate, amount, remarks) ";
			sql += "VALUES(?,?,?,?,?) ";
			Date date = new Date(new java.util.Date().getTime());
			PreparedStatement pstmt = connection.prepareStatement(sql);
			pstmt.setString(1, id);
			pstmt.setString(2, type);
			pstmt.setDate(3, date);
			pstmt.setFloat(4, amount);
			pstmt.setString(5, remarks);
			pstmt.executeUpdate();
			pstmt.close();
			state.continueWork();
			return;
		}
		catch(Exception e)
		{
			state.continueWork();
			isCommitable = false;
			throw new DatabaseError(e.getMessage());
		}
	}

	/**
	* Attempts to save the current transaction.  If the isCommitable flag is false,
	* then the balance must be below $0 as of the last transaction, and we should
	* rollback.  This is an example of coding a disallowCommit() algorithm which
	* does not exist in the Java API.
	*
	* @returns boolean flag
	*/

	public boolean save()
	{
		boolean flag = false;
		if(!isCommitable)
		{
			state.rollbackWork();
			flag = false;
		}
		else
		{
			state.completeWork();
			flag = true;
		}
		return flag;
	}

	/**
	* Deactivates the component.  Since this component returns false in canResuse()
	* this component will then be immediately destroyed.  We make one last check of
	* the isCommitable flag, in case the client disconnected without calling save()
	* Any Exceptions thrown in deactivate() will cause the transaction to roll back.
	*
	* @throws EnterpriseBeanException
	*
	*/

	public void deactivate() throws EnterpriseBeanException
	{
		if(!isCommitable)
		{
			throw new EnterpriseBeanException("Transaction being rolled back");
		}
		try
		{
			dropConnection();
			return;
		}
		catch(SQLException sqle)
		{
			throw new EnterpriseBeanException(sqle.getMessage());
		}
	}

	/**
	* Destroys the component.  This is a good place to clean up memory.
	*
	*/

	public void destroy()
	{
		destroyCacheContext();
	}

}
