package com.sybase.jaguar.sample.svu;

import com.sybase.CORBA.jdbc11.*;
import com.sybase.jaguar.beans.enterprise.*;
import java.sql.*;
import com.sybase.jaguar.sample.components.*;

/**
* This is a CORBA/EJB component using IDL datatypes that executes login
* transactions against the SVU Demo Application.  This component is EJB stateless
* and is instance pooled. At the completion of each method call, this component will
* be deactivated and its instance made available to other stubs.  This presents a very
* scalable component.  Care must be taken not to store any stateful instance members, as their
* state cannot be maintained across method invocations.   
*
* Business Case:
*
*	The application needs to be able to confirm that this is a valid student, as well
* 	as gather that users Name and Social Security Information
*
* Solution:
*
*	We have created an EJB stateless component. 
*
* @author David Lance Wolf, Jaguar Product Team
* @version 1.0
* 
* COPYRIGHT 1998 Sybase Inc
* ALL RIGHTS RESERVED
*
*/

public class SVULoginImpl extends JaguarComponentFramework implements ServerBean
{

	/** 
	* Instamce members
	*
	*/

	private InstanceContext state;

	/**
	* Contructs the component making the call to the superclasses constructor which builds a cache reference
	*
	* @throws Exception
	*
	*/

	public SVULoginImpl() throws Exception
	{
		super();
	}

	/**
	* Activates the component and builds a connection. In an instance pooled, early deactivated
	* component, activate() is called before each method invocation as the already instantiated
	* component enters the Activate state.
	*
	* @param InstanceContext state
	* @param String notUsed
	*
	* @throws EnterpriseBeanException
	*
	*/

	public void activate(InstanceContext ctx, String notUsed) throws EnterpriseBeanException
	{
		state = ctx;
		try
		{
			if(connection == null)
			{
				setConnection(WAIT_FOR_AVAILABLE_CONNECTION);
				return;
			}
		}
		catch(Exception e)
		{
			throw new EnterpriseBeanException(e.getMessage());
		}
	}

	/**
	* Determines if the given Social Security Number is a known student.  We return a boolean value.
	* The returning of values is new to the Jaguar 2.0 release.
	*
	* @param ssn Social Security Number in the form XXX-XX-XXXX
	* @returns boolean
	* @throws DatabaseError
	*
	*/
 
	public boolean isLogin(String ssn) throws DatabaseError
	{
		try
		{
			boolean flag = false;
			String sql = "SELECT count(*) ";
			sql += "FROM students ";
			sql += "WHERE student_id = ?";
			PreparedStatement pstmt = connection.prepareStatement(sql);
			pstmt.setString(1, ssn);
			ResultSet rs = pstmt.executeQuery();
			rs.next();
			if(rs.getInt(1) == 1) flag = true;
			pstmt.close();
			state.completeWork();
			return flag;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new DatabaseError(e.getMessage());
		}
	}

	/**
	* Adds a new student to the system.  If we catch any Exceptions while attempting to insert
	* the row, we will doom the transaction.  
	*
	* @param String Social Security Number in the form XXX-XX-XXXX
	* @param String firstname
	* @param String lastname
	* @param String address
	* @param String city
	* @param String state
	* @param String zip
	* @param String major
	* @returns boolean
	* @throws DatabaseError
	*
	*/

	public boolean addLogin(String ssn, String firstname, String lastname, String address, String city, String stateCode, String zip, String major) throws DatabaseError
	{
		boolean flag = false;
		String sql = "INSERT INTO students ";
		sql += "(student_id, lastname, firstname, address, city, state, zip, major) ";
		sql += "VALUES( ?,?,?,?,?,?,?,?)";
		try
		{
			PreparedStatement pstmt = connection.prepareStatement(sql);
			pstmt.setString(1, ssn);
			pstmt.setString(2, lastname);
			pstmt.setString(3, firstname);
			pstmt.setString(4, address);
			pstmt.setString(5, city);
			pstmt.setString(6, stateCode);
			pstmt.setString(7, zip);
			pstmt.setString(8, major);
			int i = pstmt.executeUpdate();
			pstmt.close();
			pstmt = null;
			flag = true;
			state.completeWork();
		}
		catch(Exception e)
		{
			state.rollbackWork();
			throw new DatabaseError(e.getMessage());
		}
		return flag;
	}

	/**
	* Returns the metadata about a specific user.  We are returning a ResultSet as a
	* TabularResults::ResultSet IDL datatype.  Note the casting of a ResultSet to
	* a TabularResults.ResultSet using the IDL static method.
	*
	* @param String Social Security Number in the form XXX-XX-XXXX
	* @returns TabularResults.ResultSet
	* @throws Database Error
	*
	*/

	public TabularResults.ResultSet getUserInfo(String ssn) throws DatabaseError
	{
		try
		{
			String sql = "SELECT student_id, firstname, lastname, major ";
			sql +=  "FROM students ";
			sql +=  "WHERE student_id = ?";
			PreparedStatement pstmt = connection.prepareStatement(sql);
			pstmt.setString(1, ssn);
			ResultSet rs = pstmt.executeQuery();
			TabularResults.ResultSet trs = IDL.getResultSet(rs);
			pstmt.close();
			pstmt = null;
			state.completeWork();
			return trs;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new DatabaseError(e.getMessage());
		}
	}

	/**
	* Deactivates the component. Any Exceptions thrown in deactivate() will cause the 
	* transaction to roll back.
	*
	* @throws EnterpriseBeanException
	*
	*/

	public void deactivate() throws EnterpriseBeanException
	{
		state = null;
		try
		{
			if(connection != null)
			{
				dropConnection();
				return;
			}
		}
		catch(Exception e)
		{
			throw new EnterpriseBeanException(e.getMessage());
		}
	}

	/**
	* Destroys the component.  This is a good place to clean up memory.
	*
	*/

	public void destroy()
	{
		destroyCacheContext();
	}

	/**
	* Informs the Jaguar Server that this component wishes to be instance
	* pooled.  
	*
	* @returns boolean Indicating if this component can be reused
	*
	*/

	public boolean canReuse()
	{
		return true;
	}

}
