package com.sybase.jaguar.sample.svu;

import com.sybase.CORBA.jdbc11.*;
import com.sybase.jaguar.beans.enterprise.*;
import com.sybase.jaguar.server.JContext;
import java.sql.*;
import com.sybase.jaguar.sample.components.*;

/**
* This is a CORBA/EJB component using IDL datatypes that executes metadata queries 
* transactions against the SVU Demo Application.  This component is EJB stateless
* and is instance pooled. At the completion of each method call, this component will
* be deactivated and its instance made available to other stubs.  This presents a very
* scalable component.  Care must be taken not to store any stateful instance members, as their
* state cannot be maintained across method invocations.   
*
* This is a good example of seperating the data access logic from the business logic.  This
* component is used in intercomponent calls from the SVUBookstore and SVUEnrollment components.
*
*
* Solution:
*
*	We have created an EJB stateless component. 
*
* @author David Lance Wolf, Jaguar Product Team
* @version 1.0
* 
* COPYRIGHT 1998 Sybase Inc
* ALL RIGHTS RESERVED
*
*/

public class SVUMetaDataImpl extends JaguarComponentFramework implements ServerBean
{


	/** 
	* Instamce members
	*
	*/

	InstanceContext state;

	/**
	* Contructs the component making the call to the superclasses constructor which builds a cache reference
	*
	* @throws Exception
	*
	*/

	public SVUMetaDataImpl() throws Exception
	{
		super();
	}

	/**
	* Activates the component and builds a connection. In an instance pooled, early deactivated
	* component, activate() is called before each method invocation as the already instantiated
	* component enters the Activate state.
	*
	* @param InstanceContext state
	* @param String notUsed
	*
	* @throws EnterpriseBeanException
	*
	*/

	public void activate(InstanceContext ctx, String notUsed) throws EnterpriseBeanException
	{
		state = ctx;
		try
		{
			if(connection == null)
			{
				setConnection(WAIT_FOR_AVAILABLE_CONNECTION);
				return;
			}
		}
		catch(Exception e)
		{
			throw new EnterpriseBeanException(e.getMessage());
		}
	}

	/**
	* Returns a list of the current Majors. We are returning the ResultSet using the
	* TabularResults::ResultSet IDL datatype.
	*
	* @returns TabulaResults.ResultSet
	* @throws DatabaseError
	*	
	*/

	public TabularResults.ResultSet getMajors() throws DatabaseError
	{
		try
		{
			String sql = "SELECT dept_id, dept_name, description FROM departments";
			Statement stmt = connection.createStatement();
			ResultSet rs = stmt.executeQuery(sql);
			TabularResults.ResultSet trs = IDL.getResultSet(rs);
			stmt.close();
			state.completeWork();
			return trs;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new DatabaseError(e.getMessage());
		}
	}

	/**
	* Returns a list of the courses. We are returning the ResultSet using the
	* TabularResults::ResultSet IDL datatype.
	*
	* @returns TabulaResults.ResultSet
	* @throws DatabaseError
	*	
	*/

	public TabularResults.ResultSet getAllCourses() throws DatabaseError
	{
		try
		{
			String s = "SELECT course_id, dept_id, course_number, course_name, description, units, instructor_id FROM all_courses";
			Statement stmt = connection.createStatement();
			ResultSet rs = stmt.executeQuery(s);
			TabularResults.ResultSet trs = IDL.getResultSet(rs);
			stmt.close();
			state.completeWork();
			return trs;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new DatabaseError(e.getMessage());
		}
	}

	/**
	* Returns a list of the courses by major. We are returning the ResultSet using the
	* TabularResults::ResultSet IDL datatype.
	*
	* @param String major
	* @returns TabulaResults.ResultSet
	* @throws DatabaseError
	*	
	*/

	public TabularResults.ResultSet getCoursesByMajor(String major) throws DatabaseError
	{
		try
		{
			String sql = "SELECT course_id, dept_id, course_number, course_name, description, units, instructor_id ";
			sql +=  "FROM all_courses ";
			sql +=  "WHERE dept_id = ?";
			PreparedStatement pstmt = connection.prepareStatement(sql);
			pstmt.setString(1, major);
			ResultSet rs = pstmt.executeQuery();
			TabularResults.ResultSet trs = IDL.getResultSet(rs);
			pstmt.close();
			pstmt = null;
			rs = null;
			state.completeWork();
			return trs;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new DatabaseError(e.getMessage());
		}
	}

	/**
	* Returns a list of the instructors. We are returning the ResultSet using the
	* TabularResults::ResultSet IDL datatype.
	*
	* @returns TabulaResults.ResultSet
	* @throws DatabaseError
	*	
	*/

	public TabularResults.ResultSet getInstructors() throws DatabaseError
	{
		try
		{
			String s = "SELECT instructor_id, lastname, firstname, resume FROM instructors";
			Statement stmt = connection.createStatement();
			ResultSet rs = stmt.executeQuery(s);
			TabularResults.ResultSet trs = IDL.getResultSet(rs);
			stmt.close();
			rs = null;
			state.completeWork();
			return trs;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new DatabaseError(e.getMessage());
		}
	}

	/**
	* Returns a list of the semsters. We are returning the ResultSet using the
	* TabularResults::ResultSet IDL datatype.
	*
	* @returns TabulaResults.ResultSet
	* @throws DatabaseError
	*	
	*/

	public TabularResults.ResultSet getSemesters() throws DatabaseError
	{
		try
		{
			String s = "SELECT semester_id, semester_description FROM semesters";
			Statement stmt = connection.createStatement();
			ResultSet rs = stmt.executeQuery(s);
			TabularResults.ResultSet trs = IDL.getResultSet(rs);
			stmt.close();
			rs = null;
			state.completeWork();
			return trs;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new DatabaseError(e.getMessage());
		}
	}

	/**
	* Deactivates the component. Any Exceptions thrown in deactivate() will cause the 
	* transaction to roll back.
	*
	* @throws EnterpriseBeanException
	*
	*/

	public void deactivate() throws EnterpriseBeanException
	{
		state = null;
		try
		{
			if(connection != null)
			{
				dropConnection();
				return;
			}
		}
		catch(Exception e)
		{
			throw new EnterpriseBeanException(e.getMessage());
		}
	}

	/**
	* Informs the Jaguar Server that this component wishes to be instance
	* pooled.  
	*
	* @returns boolean Indicating if this component can be reused
	*
	*/

	public boolean canReuse()
	{
		return true;
	}

	/**
	* Destroys the component.  This is a good place to clean up memory.
	*
	*/

	public void destroy()
	{
		destroyCacheContext();
	}

}
