/*
 * $Id: AccessControlEntry.java,v 1.3 1999/10/30 08:08:35 tcfujii Exp $
 *
 * Copyright 1998, 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package javax.ejb.deployment;

import java.security.Identity;
import java.lang.reflect.Method;
import java.io.*;

/**
 * The class AccessControlEntry associates a list of security identities
 * with an enterprise Bean's method. The specified identities are permitted 
 * to invoke the enterprise Bean's method.
 *
 * <p> The Method that is associated with an AccessControlEntry must be
 * a Method of the enterprise Bean class and the method must be
 * one of the following: a business method, an ejbCreate(...) method,
 * a finder method, or the ejbDestroy method. 
 *
 * <p> If the Method used in an AccessControlEntry is null, then the
 * AccessControlEntry is considered to be associated with the entire Bean. 
 * A Bean-level AccessControlEntry provides the default value for the
 * methods that do not have a method-level AccessControlEntry.
 */
public class AccessControlEntry implements java.io.Serializable {
    /**
     * Constructor.
     */
    public AccessControlEntry()
    {
	super();
    }

    /**
     * Construct an AccessControlEntry for the specified enterprise Bean's
     * method. If method is null, the entry is considered to be the default
     * AccessControlEntry for the enterprise Bean.
     *
     * @param method An enterprise Bean's method, or null if this is
     *    the default AccessControlEntry for the enterprise Bean.
     */
    public AccessControlEntry(Method method)
    {
	super();
	this.method = method;
    }

    /**
     * Construct an AccessControlEntry for the specified enterprise Bean's
     * method. If method is null, the entry is considered to be the default
     * AccessControlEntry for the enterprise Bean.
     *
     * @param method An enterprise Bean's method, or null if this is
     *    the default AccessControlEntry for the enterprise Bean.
     *
     * @param identities An array of security Identities that are permitted
     *    to invoke this method.
     */
    public AccessControlEntry(Method method, Identity[] identities)
    {
	super();
	this.method = method;
	allowedIdentities = identities;
    }

    /**
     * Get the method to which this AccessControlEntry applies.
     *
     * @return An enterprise Bean's method to which this AccessControlEntry 
     *    applies. If the return value is null, this is the default 
     *    AccessControlEntry for the enterprise Bean.
     */    
    public Method getMethod()
    {
	return method;
    }

    /**
     * Set the method to which this AccessControlEntry applies.
     *
     * @param value An enterprise Bean's method, or null if this is
     *    the default AccessControlEntry for the enterprise Bean.
     */    
    public void setMethod(Method value)
    {
	method = value;
    }

    /**
     * Get the array of Identities that are permitted to invoke this method.
     *
     * @return An array of security Identities that are permitted
     *    to invoke this method.
     */
    public Identity[] getAllowedIdentities()
    {
	return allowedIdentities;
    }

    /**
     * Set the array of Identities that are permitted to invoke this method.
     *
     * @param values An array of security Identities that are permitted
     *    to invoke this method.
     */
    public void setAllowedIdentities(Identity[] values)
    {
	allowedIdentities = values;
    }

    /**
     * Get the Identity at the specified index from the array of
     * Identities that are permitted to invoke this method.
     *
     * @param index The index in the array.
     *
     * @return The Identity at the specified index.
     */
    public Identity getAllowedIdentities(int index)
    {
	return allowedIdentities[index];
    }

    /**
     * Set the Identity at the specified index in the array of
     * Identities that are permitted to invoke this method.
     *
     * @param index The index in the array.
     *
     * @param identity The Identity to be set at the specified index.
     */
    public void setAllowedIdentities(int index, Identity value)
    {
	/* Reallocate the array if necessary. */
	if (allowedIdentities == null ||
	        index >= allowedIdentities.length) {
	    Identity[] newAllowedIdentities = new Identity[index + 1];
	    if (allowedIdentities != null) {
		for (int i = 0; i < allowedIdentities.length; i++)
		    newAllowedIdentities[i] = allowedIdentities[i];
	    }
	    allowedIdentities = newAllowedIdentities;
	}
	allowedIdentities[index] = value;
    }

    /*
     * We have to override the writeObject and readObject serialization
     * methods because instances of java.lang.reflect.Method are not 
     * serializable. During serialization, we replace an instance of 
     * java.lang.reflect.Method with an instance of SerializableMethod.
     */

    transient private Method method = null;
    private Identity[] allowedIdentities = null;
    
    /* 
     * The field 'serializableMethod' is a serializable representation of
     * the field 'method'.
     */
    private SerializableMethod serializableMethod = null;

    private void writeObject(ObjectOutputStream out)
	throws IOException
    {
	/*
	 * Create the serializable representation of the field 'method'.
	 */
	serializableMethod = new SerializableMethod(method);

	out.defaultWriteObject();
    }

    private void readObject(ObjectInputStream in)
	throws IOException, ClassNotFoundException, NoSuchMethodException
    {
	in.defaultReadObject();

	/*
	 * Construct the field 'method' from its serializable representation.
	 */
	method = serializableMethod.getMethod();
    }
}
