/*
 * $Id: ControlDescriptor.java,v 1.5 1999/11/13 22:17:04 harishp Exp $
 *
 * Copyright 1998, 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package javax.ejb.deployment;

import java.security.Identity;
import java.lang.reflect.Method;
import java.io.*;

/**
 * The ControlDescriptor defines the transaction and security attributes 
 * to be associated with the runtime execution of an enterprise Bean method.
 *
 * <p> If the Method used in an ControlDescriptor is null, then the
 * ControlDescriptor is considered to be associated with the entire Bean. 
 * A Bean-level ControlDescriptor provides the default value for the
 * methods that do not have a method-level ControlDescriptor.
 *
 * <p> The methods of the ControlDescriptor class conform to the JavaBeans 
 * property design pattern.
 */
public class ControlDescriptor implements java.io.Serializable {
    /**
     * The enterprise Bean does not support a global transaction. The container
     * must not invoke the enterprise Bean's method in the scope of 
     * a global transaction.
     */
    public static final int TX_NOT_SUPPORTED = 0;

    /**
     * The enterprise Bean manages transaction boundaries itself using the
     * javax.jts.CurrentTransaction interface. 
     */
    public static final int TX_BEAN_MANAGED = 1;

    /**
     * The enterprise Bean requires that the method be executed in 
     * a global transaction.
     *
     * <p> The container is responsible for managing transaction boundaries for
     * the enterprise Bean as follow.
     *
     * <p> If the caller is associated with a transaction, the execution of 
     * the enterprise Bean method will be associated with the caller's 
     * transaction.
     *
     * <p> If the caller is not associated with a transaction, the container
     * starts a new global transaction, executes the enterprise Bean's method 
     * in the scope of the transaction, and commits the transaction when the 
     * enterprise Bean's method has completed.
     */
    public static final int TX_REQUIRED = 2;

    /**
     * The enterprise Bean supports the execution of a method in a global
     * transaction scope.
     *
     * The container is responsible for managing transaction boundaries for
     * the enterprise Bean as follow.
     *
     * <p> If the caller is associated with a transaction, the execution of 
     * the enterprise Bean method will be associated with the caller's 
     * transaction.
     *
     * <p> If the caller is not associated with a transaction, the container
     * executes the enterprise Bean's method without a transaction.
     */
    public static final int TX_SUPPORTS = 3;

    /**
     * <p> The enterprise Bean requires that a method is executed in a new
     * global transaction scope.
     *
     * <p> The container is responsible for managing transaction boundaries 
     * for the enterprise Bean as follow.
     *
     * <p> The container starts a new transaction, executes the enterprise
     * Bean's method in the scope of the new transaction, and commits the 
     * new transaction when the enterprise Bean's method has completed.
     *
     * <p> If the caller is associated with a transaction, the association
     * of the current thread with the caller's transaction is suspended during
     * the execution of the enterprise Bean's method, and resumed when the
     * enterprise Bean's method has completed.
     */
    public static final int TX_REQUIRES_NEW = 4;

    /**
     * <p> The enterprise Bean requires that the client invocation includes
     * a global transaction scope.
     *
     * The container is responsible for managing transaction boundaries for
     * the enterprise Bean as follow.
     *
     * <p> If the caller is associated with a transaction, the execution of
     * the enterprise Bean method will be associated with the caller's 
     * transaction.
     *
     * <p> If the caller is not associated with a transaction, the container
     * throws the javax.jts.TransactionRequiredException to the caller.
     */
    public static final int TX_MANDATORY = 5;


    /*
     * Transaction isolation levels.
     */

    //
    // TRANSACTION_NONE is not applicable to EJB, therefore we do not
    // define it as a legal value.
    //
    ///**
    // * Transactions are not supported. 
    // */
    //public static final int TRANSACTION_NONE = 0;
    //

    /**
     * Isolation degree equivalent to the JDBC 
     * TRANSACTION_READ_UNCOMMITTED level.
     */
    public static final int TRANSACTION_READ_UNCOMMITTED = 1;

   /**
     * Isolation degree equivalent to the JDBC 
     * TRANSACTION_READ_COMMITTED level.
     */
    public static final int TRANSACTION_READ_COMMITTED = 2;

    /**
     * Isolation degree equivalent to the JDBC 
     * TRANSACTION_REPEATABLE_READ level.
     */
    public static final int TRANSACTION_REPEATABLE_READ = 4;

    /**
     * Isolation degree equivalent to the JDBC 
     * TRANSACTION_SERIALIZABLE level.
     */
    public static final int TRANSACTION_SERIALIZABLE = 8;

    /*
     * Security attributes.
     */

    /**
     * Run the enterprise Bean method with the client's security identity.
     */
    public static final int CLIENT_IDENTITY = 0;

    /**
     * Run the enterprise Bean method with the security identity of a 
     * specified user account.
     */
    public static final int SPECIFIED_IDENTITY = 1;

    /**
     * Run the enterprise Bean method with the Identity of a "privileged
     * account". The container maps the abstract notion of a "privileged
     * account" to a suitable privileged account on the underlying platform,
     * such as the database administrator, or the operating system 
     * administrator account.
     */
    public static final int SYSTEM_IDENTITY = 2;

    /**
     * Construct a Bean-level ControlDescriptor.
     */
    public ControlDescriptor() {
	super();
    }

    /**
     * Construct a ControlDescriptor for a specified Method.
     *
     * @param method The Method associated with the ControlDescriptor. The
     *    Method must be a method of the enterprise Bean class and the Method must be
     *    one of the following: a business method, an ejbCreate(...) method,
     *    an finder method, or the ejbDestroy method. If method is 
     *    null, the ControlDescriptor will be a Bean-level one.
     */
    public ControlDescriptor(Method method) {
	super();
	this.method = method;
    }

    /**
     * Obtain the Method associated with the with this ControlDescriptor.
     *
     * @return A Method associated with the ControlDescriptor. If the return
     *    value is null, the ControlDescriptor is a Bean-level one.
     */
    public Method getMethod()
    {
	return method;
    }

    /**
     * Set the method to which this ControlDescriptor applies.
     *
     * @param value An enterprise Bean's method, or null if this is
     *    the default ControlDescriptor for the enterprise Bean.
     */    
    public void setMethod(Method value)
    {
	method = value;
    }

    /**
     * Test if the ControlDescriptor is a method-level ControlDescriptor 
     * descriptor.
     *
     * @return True if the ControlDescriptor is associated with an individual
     *    method, false if it is a Bean-level ControlDescriptor.
     */
    boolean isMethodLevel()
    {
	return method != null;
    }
    
    /**
     * Get the value of the transaction attribute. The transaction attribute 
     * tells the container how to manage transaction scopes before and after
     * the execution of the enterprise Bean method.
     *
     * @return The value of the transaction attribute. It must be one of
     *    TX_NOT_SUPPORTED, TX_BEAN_MANAGED, TX_REQUIRED, TX_SUPPORTS,
     *    TX_REQUIRES_NEW and TX_MANDATORY.
     */
    public int getTransactionAttribute() 
    {
	return transactionAttribute;
    }

    /**
     * Set the value of the transaction attribute. The transaction attribute
     * tells the container how to manage transaction scopes before and after
     * the execution of the enterprise Bean method.
     *
     * @param value The value of the transaction attribute. It must be one of
     *    TX_NOT_SUPPORTED, TX_BEAN_MANAGED, TX_REQUIRED, TX_SUPPORTS,
     *    TX_REQUIRES_NEW and TX_MANDATORY.
     */
    public void setTransactionAttribute(int value)
    { 
	switch (value) {
	case TX_NOT_SUPPORTED:
	case TX_BEAN_MANAGED:
	case TX_REQUIRED:
	case TX_REQUIRES_NEW:
	case TX_MANDATORY:
	case TX_SUPPORTS:
	    transactionAttribute = value;
	    break;
	default:
	    throw new IllegalArgumentException(
			"Illegal value of transactionAttribute");
	}
    }

    /**
     * Get the transaction isolation level.
     *
     * @return Transaction isolation level. The value is one of 
     *    TRANSACTION_READ_UNCOMMITTED, TRANSACTION_READ_COMMITTED,
     *    TRANSACTION_REPEATABLE_READ, and TRANSACTION_SERIALIZABLE.
     * 
     */
    public int getIsolationLevel() 
    { 
	return isolationLevel;
    }

    /**
     * Set the transaction isolation level.
     *
     * @param value Transaction isolation level. The value must be one of 
     *    TRANSACTION_READ_UNCOMMITTED, TRANSACTION_READ_COMMITTED,
     *    TRANSACTION_REPEATABLE_READ, and TRANSACTION_SERIALIZABLE. 
     */
    public void setIsolationLevel(int value)
    { 
	switch (value) {
	case TRANSACTION_READ_UNCOMMITTED:
	case TRANSACTION_READ_COMMITTED:
	case TRANSACTION_REPEATABLE_READ:
	case TRANSACTION_SERIALIZABLE:
	    isolationLevel = value;
	    break;
	default:
	    throw new IllegalArgumentException(
			"Illegal value of isolationLevel");
	}
    }

    /**
     * Get the value of the runAsMode security attribute. The runAsMode
     * attribute tells the container the security identity to associate
     * with the execution of the enterprise Bean method.
     *
     * @return The value of the runAsMode attribute. The value is one of
     *    CLIENT_IDENTITY, SPECIFIED_IDENTITY, and SYSTEM_IDENTITY.
     */
    public int getRunAsMode() 
    {
	return runAsMode;
    }

    /**
     * Set the value of the runAsMode security attribute. The runAsMode
     * attribute tells the container the security identity to associate
     * with the execution of the enterprise Bean method.
     *
     * @param value The value of the runAsMode attribute. The value must be 
     *    one of CLIENT_IDENTITY, SPECIFIED_IDENTITY, and SYSTEM_IDENTITY.
     */
    public void setRunAsMode(int value)
    {
	switch (value) {
	case CLIENT_IDENTITY:
	case SPECIFIED_IDENTITY:
	case SYSTEM_IDENTITY:
	    runAsMode = value;
	    break;
	default:
	    throw new IllegalArgumentException("Illegal value of runAsMode");
	}
    }

    /**
     * Get the value of the runAsIdentity security attribute. The runAsIdentity
     * attribute tells the container the security identity to associate
     * with the execution of the enterprise Bean method.
     *
     * <p> The value of the runAsIdentity is used only if the value of 
     * the runAsMode is SPECIFIED_IDENTITY; it is ignored otherwise.
     *
     * @return The Identity to associate with the execution of the enterprise
     *    Bean method.
     */
    public Identity getRunAsIdentity() 
    {
	return runAsIdentity;
    }

    /**
     * Set the value of the runAsIdentity security attribute. The runAsIdentity
     * attribute tells the container the security identity to associate
     * with the execution of the enterprise Bean method.
     *
     * <p> The value of the runAsIdentity is used only if the value of 
     * the runAsMode is SPECIFIED_IDENTITY; it is ignored otherwise.
     *
     * @param value The Identity to associate with the execution of the 
     *    enterprise Bean method.
     */
    public void setRunAsIdentity(Identity value)
    {
	runAsIdentity = value;
    }

    /*
     * Initialize the control descriptor with defaults.
     */
    private int transactionAttribute = TX_SUPPORTS;
    private int isolationLevel = TRANSACTION_READ_COMMITTED;
    private int runAsMode = SPECIFIED_IDENTITY;
    private Identity runAsIdentity = null;

    transient private Method method = null; /* method is not serializable */
    
    /* 
     * The field 'serializableMethod' is a serializable representation of
     * the field 'method'.
     */
    private SerializableMethod serializableMethod = null;

    /*
     * We override the writeObject and readObject methods to replace the
     * non-serializable 'method' field by the serializable 'serializableMethod'
     * field.
     */
    private void writeObject(ObjectOutputStream out)
	throws IOException
    {
	/*
	 * Create the serializable representation of the field 'method'.
	 */
	serializableMethod = new SerializableMethod(method);

	out.defaultWriteObject();
    }

    private void readObject(ObjectInputStream in)
	throws IOException, ClassNotFoundException, NoSuchMethodException
    {
	in.defaultReadObject();

	/*
	 * Construct the field 'method' from its serializable representation.
	 */
	method = serializableMethod.getMethod();
    }
}
