/*
 * $Id: DeploymentDescriptor.java,v 1.5 1999/11/13 22:17:05 harishp Exp $
 *
 * Copyright 1998, 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package javax.ejb.deployment;

import java.io.*;
import java.util.Properties;
import javax.naming.Name;
import java.security.Identity;

/**
 * The DeploymentDescriptor class is the common base class for the 
 * SessionDescriptor and EntityDescriptor deployment descriptor classes.
 *
 * <p> The methods of the class conform to the JavaBeans property design 
 * pattern.
 *
 * @see EntityDescriptor
 * @see SessionDescriptor
 */
public abstract class DeploymentDescriptor implements java.io.Serializable {

    /**
     * Create an instance of DeploymentDescriptor.
     */
    public DeploymentDescriptor() {
	super();
    }

    /**
     * Get the enterprise Bean's full class name.
     *
     * @return The enterprise Bean's class name.
     */
    public String getEnterpriseBeanClassName() 
    { 
	return enterpriseBeanClassName;
    }

    /**
     * Set the enterprise Bean's full class name.
     *
     * @param value The enterprise Bean's class name.
     */
    public void setEnterpriseBeanClassName(String value)
    { 
	enterpriseBeanClassName = value;
    }

    /**
     * Get the full name of the enterprise Bean's home interface.
     *
     * @return The name of the enterprise Bean's home interface.
     */
    public String getHomeInterfaceClassName()
    { 
	return homeInterfaceClassName;
    }

    /**
     * Set the full name of the enterprise Bean's home interface.
     *
     * @param value The name of the enterprise Bean's home interface.
     */
    public void setHomeInterfaceClassName(String value)
    { 
	homeInterfaceClassName = value;
    }

    /**
     * Get the full name of the enterprise Bean's remote interface.
     *
     * @return The name of the enterprise Bean's remote interface.
     */
    public String getRemoteInterfaceClassName()
    { 
	return remoteInterfaceClassName;
    }

    /**
     * Set the full name of the enterprise Bean's remote interface.
     *
     * @param value The name of the enterprise Bean's remote interface.
     */
    public void setRemoteInterfaceClassName(String value)
    { 
	remoteInterfaceClassName = value;
    }

    /**
     * Get the name to associate with the enterprise Bean in the JNDI name
     * space. The container will bind the enterprise Bean's home interface
     * with a JNDI name that includes this name as its trailing part. This
     * means that the container can prefix the name returned by getBeanName()
     * with an arbitrary JNDI path.
     *
     * <p> For example, if getBeanHomeName() returns "bank/Account", the 
     * container can bind the Bean's home interface in the JNDI name space
     * with the name "mis/ejb-components/bank/Account".
     *
     * @return A JNDI name for this enterprise Bean.
     */
    public Name getBeanHomeName() 
    { 
	return beanHomeName;
    }

    /**
     * Set the name to associate with the enterprise Bean in the JNDI name
     * space. The container will bind the enterprise Bean's home interface
     * with a JNDI name that includes this name as its trailing part. This
     * means that the container can prefix the name returned by getBeanName()
     * with an arbitrary JNDI path.
     *
     * <p> Note that using the type java.naming.Name makes the format of
     * the name independent of the syntax used by the actual naming system.
     *
     * @param value A JNDI name for this enterprise Bean.
     */
    public void setBeanHomeName(Name value) 
    {
	beanHomeName = value;
    }

    /**
     * Get enterprise Bean's environment properties.
     *
     * @return Enterprise Bean's environment properties.
     */
    public Properties getEnvironmentProperties() 
    {
	return environment;
    }

    /**
     * Set enterprise Bean's environment properties.
     *
     * @param value Enterprise Bean's environment properties.
     */
    public void setEnvironmentProperties(Properties value) 
    {
	environment = value;
    }

    /**
     * Get the AccessControlEntry objects for the enterprise Bean. An
     * AccessControlEntry object associates an enterprise Bean's method
     * with a list of security Identities that are allowed to invoke the
     * method.
     *
     * @return An array of AccessControlEntry objects.
     */
    public AccessControlEntry[] getAccessControlEntries()
    {
	return accessControlEntries;
    }

    /**
     * Set the AccessControlEntry objects for the enterprise Bean. An
     * AccessControlEntry object associates an enterprise Bean's method
     * with a list of security Identities that are allowed to invoke the
     * method.
     *
     * @param values An array of AccessControlEntry objects.
     */
    public void setAccessControlEntries(AccessControlEntry[] values)
    {
	accessControlEntries = values;
    }

    /**
     * Get the AccessControlEntry object at a specified index. An
     * AccessControlEntry object associates an enterprise Bean's method
     * with a list of security Identities that are allowed to invoke the
     * method.
     *
     * @param index An index in the array of Identities.
     *
     * @return The AccessControlEntry at the index.
     */
    public AccessControlEntry getAccessControlEntries(int index)
    {
	return accessControlEntries[index];
    }

    /**
     * Set the AccessControlEntry object at a specified index. An
     * AccessControlEntry object associates an enterprise Bean's method
     * with a list of security Identities that are allowed to invoke the
     * method.
     *
     * @param index An index in the array of Identities.
     *
     * @param value The AccessControlEntry to set at the index.
     */
    public void setAccessControlEntries(int index, AccessControlEntry value)
    {
	/* Reallocate the array if necessary. */
	if (accessControlEntries == null ||
	        index >= accessControlEntries.length) {
	    AccessControlEntry[] newAccessControlEntries = 
		new AccessControlEntry[index + 1];
	    if (accessControlEntries != null) {
		for (int i = 0; i < accessControlEntries.length; i++)
		    newAccessControlEntries[i] = accessControlEntries[i];
	    }
	    accessControlEntries = newAccessControlEntries;
	}
	accessControlEntries[index] = value;
    }

    /**
     * Get the array of the enterprise Bean's control descriptors. 
     *
     * @return An array of enterprise Bean's control descriptors.
     */
    public ControlDescriptor[] getControlDescriptors() 
    { 
	return controlDescriptors;
    }

    /**
     * Set the array of the enterprise Bean's control descriptors.
     *
     * @param value An array of the enterprise Bean's control descriptors.
     */
    public void setControlDescriptors(ControlDescriptor[] value) 
    {
	controlDescriptors = value;
    }

    /**
     * Get the control descriptor at the specified index.
     *
     * @param index The index of the control descriptor.
     *
     * @return The control descriptor at the specified index.
     */
    public ControlDescriptor getControlDescriptors(int index)
    { 
	return controlDescriptors[index];
    }

    /**
     * Set the control descriptor at the specified index.
     *
     * @param index The index of the control descriptor.
     *
     * @param value The control descriptor to be set at the specified index.
     */
    public void setControlDescriptors(int index, ControlDescriptor value) 
    {
	/* Reallocate the array if necessary. */
	if (controlDescriptors == null ||
	        index >= controlDescriptors.length) {
	    ControlDescriptor[] newControlDescriptors = 
		new ControlDescriptor[index + 1];
	    if (controlDescriptors != null) {
		for (int i = 0; i < controlDescriptors.length; i++)
		    newControlDescriptors[i] = controlDescriptors[i];
	    }
	    controlDescriptors = newControlDescriptors;
	}
	controlDescriptors[index] = value;
    }

    /**
     * Test if the enterprise Bean is re-entrant. Only entity Beans can
     * be defined as re-entrant, and it is an error for a session Bean
     * deployment descriptor to return true.
     *
     * @return True if the Bean is reentrant, false otherwise.
     */
    public boolean isReentrant() 
    { 
	return reentrant;
    }

    /**
     * This method returns the same result as isReentrant(). It is included
     * for compatibility with the JavaBeans design-pattern.
     */
    public boolean getReentrant() 
    { 
	return isReentrant(); 
    }

    /**
     * Specify whether the enterprise Bean is re-entrant. Only entity Beans can
     * be defined as re-entrant, and it is an error for a session Bean
     * deployment descriptor to attempt to specify that the Bean is re-entrant.
     *
     * @param value true if the Bean is reentrant, false otherwise.
     */
    public void setReentrant(boolean value)
    {
	if ( value == true && this instanceof SessionDescriptor) 
	    throw new IllegalArgumentException(
		      "Session bean cannot be re-entrant");

	reentrant = value;
    }

    private String enterpriseBeanClassName = "";
    private String remoteInterfaceClassName = "";
    private String homeInterfaceClassName = "";
    private ControlDescriptor[] controlDescriptors = null;
    private AccessControlEntry[] accessControlEntries = null;
    private Properties environment = new Properties();
    private boolean reentrant = false;
    private Name beanHomeName = null;

    /*
     * A version number will help in evolution of the deployment descriptor
     * format.
     */
    protected int versionNumber = 1;
}
