/*
 * $Id: EntityDescriptor.java,v 1.3 1999/10/30 08:08:39 tcfujii Exp $
 *
 * Copyright 1998, 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package javax.ejb.deployment;

import java.lang.reflect.Field;
import java.io.*;

/**
 * The EntityDescriptor class defines the deployment descriptor for
 * an entity enterprise Bean.
 *
 * <p> A serialized instance of the EntityDescriptor class is used as the
 * standard format for passing the entity enterprise Bean's declarative 
 * deployment attributes in the ejb-jar file. 
 *
 * <p> The Bean provider tools use the setter functions to initialize an 
 * instance of the deployment descriptor. The Bean provider tools then
 * serialize the instance into the ejb-jar file.
 *
 * <p> The getter functions are used by the container tools at deployment
 * time. The tools deserialize the instance from the ejb-jar file, and use
 * the getter functions to obtain information about the enterprise Bean.
 *
 * <p> Note that the Enterprise JavaBeans architecture does not prescribe
 * whether the actual deployment descriptor class is used by the container
 * at runtime. Therefore, the container is allowed to import the information 
 * from the deployment descriptor at deployment time, and store the information
 * in a container-specific format. The container is however required to enforce
 * the declarative attributes at runtime, as specified by the Enterprise
 * JavaBeans specification.
 *
 * <p> After an enterprise Bean has been installed into a container, the
 * container tools can be then used to view and change the values of  
 * the deployment attributes. As changing the values of the deployment
 * descriptor attributes may alter the semantics of a deployed application, 
 * the container may restrict changes to certain attributes.
 *
 * <p> The methods of the EntityDescriptor class conform to the JavaBeans 
 * property design pattern.
 */
public class EntityDescriptor extends DeploymentDescriptor {
    /**
     * Create an instance of EntityDescriptor.
     */
    public EntityDescriptor() {
	super();
    }

    /**
     * Get the full class name of the enterprise Bean's primary key.
     *
     * @return The primary key class name.
     */
    public String getPrimaryKeyClassName() 
    { 
	return primaryKeyClassName;
    }

    /**
     * Set the full class name of the enterprise Bean's primary key.
     *
     * @return The primary key class name.
     */
    public void setPrimaryKeyClassName(String value) 
    { 
	primaryKeyClassName = value;
    }

    /**
     * Get the array of the container-managed fields.
     *
     * @return The array of the container-managed fields.
     */
    public Field[] getContainerManagedFields()  
    { 
	return containerManagedFields;
    }

    /**
     * Set the array of the names of the container-managed fields.
     *
     * @param value The array of the names of the container-managed fields.
     */    
    public void setContainerManagedFields(Field[] values)
    {
	containerManagedFields = values;
    }

    /**
     * Get the name of field at the given index in the array of 
     * container-managed fields.
     *
     * @param index The index in the array.
     *
     * @return The container-managed field at the specified index.
     */
    public Field getContainerManagedFields(int index) 
    { 
	return containerManagedFields[index];
    }

    /**
     * Set the field at the given index in the array of container-managed 
     * fields.
     *
     * @param index The index in the array.
     *
     * @param value The container-managed field to be set at the index.
     */
    public void setContainerManagedFields(int index, Field value) 
    {
	/* Reallocate the array if necessary. */
	if (containerManagedFields == null ||
	        index >= containerManagedFields.length) {
	    Field[] newContainerManagedFields = new Field[index + 1];
	    if (containerManagedFields != null) {
		for (int i = 0; i < containerManagedFields.length; i++)
		    newContainerManagedFields[i] = containerManagedFields[i];
	    }
	    containerManagedFields = newContainerManagedFields;
	}
	containerManagedFields[index] = value;
    }

    private String primaryKeyClassName = "";

    /*
     * We have to override the writeObject and readObject serialization
     * methods because instances of java.lang.reflect.Field are not 
     * serializable. During serialization, we replace the instances of 
     * java.lang.reflect.Field with instances of SerializableField.
     */
    transient private Field[] containerManagedFields = null;

    /* 
     * The field 'serializableFields' is a serializable representation of
     * the field 'containerManagedFields'.
     */
    private SerializableField[] serializableFields = null;

    private void writeObject(ObjectOutputStream out)
	throws IOException
    {
	/*
	 * Create the serializable representation of the field 
	 * 'containerManagedFields'.
	 */
	if (containerManagedFields == null) {
	    serializableFields = null;
	} else {
	    int count = containerManagedFields.length;

	    serializableFields = new SerializableField[count];
	    for (int i = 0; i < count; i++) {
		serializableFields[i] = 
		    new SerializableField(containerManagedFields[i]);
	    }
	}

	out.defaultWriteObject();
    }

    private void readObject(ObjectInputStream in)
	throws IOException, ClassNotFoundException, NoSuchFieldException
    {
	in.defaultReadObject();

	/*
	 * Construct the field 'containerManagedFields' from its 
	 * serializable representation.
	 */
	if (serializableFields == null) {
	    containerManagedFields = null;
	} else {
	    int count = serializableFields.length;

	    containerManagedFields = new Field[count];
	    for (int i = 0; i < count; i++) {
		    containerManagedFields[i] = 
			serializableFields[i].getField();
	    }
	}
    }
}
