/*
 * $Id: SessionDescriptor.java,v 1.5 1999/11/13 22:17:07 harishp Exp $
 *
 * Copyright 1998, 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package javax.ejb.deployment;

/**
 * The SessionDescriptor class defines the deployment descriptor for
 * a session enterprise Bean.
 *
 * <p> A serialized instance of the SessionDescriptor class is used as the
 * standard format for passing the session enterprise Bean's declarative 
 * deployment attributes in the ejb-jar file. 
 *
 * <p> The Bean provider tools use the setter functions to initialize an 
 * instance of the deployment descriptor. The Bean provider tools then
 * serialize the instance into the ejb-jar file.
 *
 * <p> The getter functions are used by the container tools at deployment
 * time. The tools deserialize the instance from the ejb-jar file, and use
 * the getter functions to obtain information about the enterprise Bean.
 *
 * <p> Note that the Enterprise JavaBeans architecture does not prescribe
 * whether the actual deployment descriptor class is used by the container
 * at runtime. Therefore, the container is allowed to import the information 
 * from the deployment descriptor at deployment time, and store the information
 * in a container-specific format. The container is however required to enforce
 * the declarative attributes at runtime, as specified by the Enterprise
 * JavaBeans specification.
 *
 * <p> After an enterprise Bean has been installed into a container, the
 * container tools can be used to view and change the values of  
 * the deployment attributes. As changing the values of the deployment
 * descriptor attributes may alter the semantics of a deployed application, 
 * the container may restrict changes to certain attributes.
 *
 * <p> The methods of the SessionDescriptor class conform to the JavaBeans 
 * property design pattern.
 */
public class SessionDescriptor extends DeploymentDescriptor {
    /**
     * Create an instance of SessionDescriptor.
     */
    public SessionDescriptor() {
	super();
    }

    /**
     * Get the session timeout value in seconds. A zero value means that
     * the container should use a container-specific default value.
     *
     * @return The timeout value in seconds.
     */
    public int getSessionTimeout() 
    {
	return sessionTimeout;
    }

    /**
     * Set the session timeout value in seconds. A zero value means that
     * the container should use a container-specific default value.
     *
     * @param value The timeout value in seconds.
     */
    public void setSessionTimeout(int value) {
	sessionTimeout = value;
    }

    /**
     * The session Bean is stateless. An instance of a stateless Bean can 
     * be reused for multiple session EJB objects.
     */
    public static final int STATELESS_SESSION = 0;

    /**
     * The session Bean is stateful. An instance of a stateful session Bean
     * remains associated with a session EJB object for its lifetime.
     */
    public static final int STATEFUL_SESSION = 1;

    /**
     * Get the session Bean's state management type.
     *
     * @return The session Bean's state management type. Its value must
     *    be either STATEFUL_SESSION or STATELESS_SESSION.
     */
    public int getStateManagementType()
    {
	return stateManagementType;
    }

    /**
     * Set the session Bean's state management type;.
     *
     * @return The session Bean's state management type. Its value must
     *    be either STATEFUL_SESSION or STATELESS_SESSION.
     */
    public void setStateManagementType(int value) 
    {
	if (value == STATEFUL_SESSION || value == STATELESS_SESSION)
	    stateManagementType = value;
	else
	    throw new IllegalArgumentException(
		        "Illegal value of session stateManagementType");
    }

    /*
     * This is the state that gets serialized into ejb-jar file.
     */
    private int sessionTimeout = 0;
    private int stateManagementType = STATEFUL_SESSION;
}

