//
//  Copyright (c) 1999, 2001
//  Sybase, Inc
//  Emeryville, CA 94608
//  All Rights Reserved
//
package jcs.util;

import java.io.*;
import java.sql.*;
import java.util.*;

/**  
 *  
 *  A class with miscellaneous static methods to execute SQL statements and scripts.
 *  <pre>
 *  connectTo(server)  -   Return a connection to the given server.
 *                         Specify an empty <code>server</code> when you invoke
 *                         this method in the server.
 *
 *  statement(sql, server) - Get a connection and execute the given SQL statement.
 *
 *  query(sql, server) - Get a connection, execute the given SQL query, and return
 *       the ResultSet.
 *  querySingleString(sql, server) - Get a connection, execute the given query,
 *       which must return a single column/row that is a string, and return that string.  
 *
 *  querySingleInt(sql, server) - Get a connection, execute the given query,
 *       which must return a single column/row that is an int, and return that int.  
 *  </pre>
 */  

public class ExecSql {

   /**  
    *  The ExecSql class contains only static methods, and cannot be instantiated.
    */     
   private ExecSql() {}  

   /**  
    *  Connects to the given SQL server. 
    *  <p>This method is intended for routines 
    *  that will run in either a client or server environment.
    *  <ul>
    *  <li>A <code>server</code> parameter that is null, empty 
    *  or blank, or is <code>jdbc:default:connection</code> indicates 
    *  a server-side call, 
    *  using the native JDBC driver.
    *  <li>A <code>server</code> parameter that is not empty and not
    *  <code>jdbc:default:connection</code> should be a valid url  
    *  for a client-side connect, 
    *  using the jConnect JDBC driver.  This parameter must include the 
    *  server-name, port-number, user-name, and (optional) password.
    *  <p>sFor example:
    *  </pre>
    *  <pre>
    *  antibes:4000?user=jsmith&password=wxyz
    *  antibes:4000?user=sa      
    *  </pre>
    *  In the second example, the password is omitted and is empty by default.
    *  </ul>
    *  @param  server  a string that identifies the server.  See above.
    *  @exception Exception  Thrown by <code>java.io, java.sql, or java.util</code>
    *  @return  a java.sql.Connection object for the server.
    *
    */  

   public static Connection connectTo(String server) throws Exception {

      /**  
       *   The JDBC driver for client execution
       */
      String clientDriver = "com.sybase.jdbc.SybDriver";
      /**  
       *   The JDBC driver for server execution
       */
      String serverDriver = "sybase.asejdbc.ASEDriver";
      /**
       *   The URL for client execution
       */  
      String clientURL = "jdbc:sybase:Tds:";
      /**
       *   The URL for server execution
       */  
      String serverURL = "jdbc:sybase:asejdbc";

      String url = "";
      // Load the client or server JDBC driver

      if (StringUtil.isNothing(server)
              ||server.equals("jdbc:default:connection")) {   
         //  A <code>server</code> parameter that is empty
         //  or 'defaut' indicates a server-side call.
         //  Load the native JDBC driver.
         Class.forName(serverDriver);
         url = serverURL;
      } else {
         //  A <code>server</code> parameter that is not empty
         //  or 'default' indicates a client-side call.
         //  Load the jConnect JDBC driver.
         Class.forName(clientDriver);
         //  Append the server-name (etc) to the clientURL.
         url = clientURL+server;
      }

      // Get a connection
      Connection con = DriverManager.getConnection(url);
      return con;
   }
 

   /**  
    *   Connects to the given server, and executes the given SQL statement.
    *   This is a convenience method for non-parameterized statements.  
    *   <p>Example Java code:
    *   <pre>
    *   ExecSql.statement("delete from emps", "");
    *   String stmt = 
    *          "set quoted_identifier on  "
    *         + "insert into emps('emp id', name) " +
    *         + "values('12345', 'John Doe')";
    *   ExecSql.statement(stmt, "some-server:1000?user=sa");
    *   </pre>
    *   @param sql  a string with the SQL statement to be executed.  
    *               This statement must have no question-mark parameters, 
    *               and must return no result set.
    *   @param server  a string that identifies the server in which to execute 
    *                  the statement.
    *   @exception Exception  Thrown by <code>java.io, java.sql, or java.util</code>
    *   @see #connectTo
    */  

   public static void statement (String sql, String server)  
           throws Exception {

      Connection con = connectTo(server);
      Statement stmt = con.createStatement();
      int res = stmt.executeUpdate(sql);
   }

   /**  
    *   Connects to the given server, and executes the given SQL query. 
    *   This is a convenience method for non-parametrized queries.
    *   <p>Example Java code:
    *   <pre>
    *   java.sql.ResultSet rs 
    *            = ExecSql.query("select * from systypes", "");
    *   </pre>
    *   @param sql  a string with the SQL statement to be executed.  
    *               This statement must have  no question-mark parameters, 
    *                and must return a result set.
    *   @param server  a string that identifies the server in which to execute 
    *                  the statement.
    *   @exception Exception  Thrown by <code>java.io, java.sql, or java.util</code>
    *                 
    *   @see #connectTo
    */  

   public static ResultSet query (String query, String server)  
           throws Exception {

      Connection con = connectTo(server);
      Statement stmt = con.createStatement();
      ResultSet rs = stmt.executeQuery(query);
      return rs;
   }

   /**  
    *   Connects to the given server, executes the given 
    *   single-valued SQL query, and returns that single (String) value. 
    *   <p>This is a convenience method for non-parametrized queries
    *   that return a single row with a single string column.
    *   <p>Example Java code:
    *   <pre>
    *   String s = ExecSql.querySingleString("select max(name) from systypes",  
    *                                         "antibes:4000?user=sa");
    *   </pre>
    *   @param sql  a string with the SQL statement to be executed.  
    *               This statement must have  no question-mark parameters, 
    *               and must return a result set with a single row and
    *               a single column that is a character string or String.
    *   @param server  a string that identifies the server in which to execute 
    *                  the statement.
    *   @exception Exception  Thrown by <code>java.io, java.sql, or java.util</code>
    *                 
    *   @see #connectTo
    */  

   public static String querySingleString (String query, String server)  
           throws Exception {

      ResultSet rs = ExecSql.query(query, server);
      if (rs.next()) {
         String workString = rs.getString(1);  
         if (rs.next()) {
            throw new Exception("Select returned more than one row");
         } else {
            return workString;
         }          
      } else { 
         throw new Exception("Select returned no rows"); 
      }
   }

   /**  
    *   Connects to the given server, executes the given 
    *   single-valued SQL query, and returns that single (int) value. 
    *   <p>This is a convenience method for non-parametrized queries
    *   that return a single row with a single  column.
    *   <p>Example Java code:
    *   <pre>
    *   String s = query("select count(*) from systypes",  
    *                                         "antibes:4000?user=sa");
    *   </pre>
    *   @param sql  a string with the SQL statement to be executed.  
    *               This statement must have  no question-mark parameters, 
    *               and must return a result set with a single row and
    *               a single column that is an integer.
    *   @param server  a string that identifies the server in which to execute 
    *                  the statement.
    *   @exception Exception  Thrown by <code>java.io, java.sql, or java.util</code>
    *                 
    *   @see #connectTo
    */  

   public static int querySingleInt (String query, String server)  
           throws Exception {

      ResultSet rs = ExecSql.query(query, server);
      if (rs.next()) {
         int work = rs.getInt(1);  
         if (rs.next()) {
            throw new Exception("Select returned more than one row");
         } else {
            return work;
         }          
      } else { 
         throw new Exception("Select returned no rows"); 
      }
   }
} 
 






