//
//  Copyright (c) 1999, 2001
//  Sybase, Inc
//  Emeryville, CA 94608
//  All Rights Reserved
//
package jcs.util;

import java.sql.*;
import java.io.*;
import java.util.*;

/**  
 *  A class with miscellaneous methods for a client environment:
 *  <pre>
 *  main:        a client-side command-line interpreter to invoke 
 *               the other methods
 *  putString:   copies a file to a Java String in the SQL server
 *  getString:   copies a string from the SQL server to a file
 *  putStream:   streams a file to a Java String or TEXT column 
 *               in the SQL server
 *  getStream:   streams a string or TEXT column from the SQL server 
 *               to a file
 *  file2String: copies a client file to a String variable
 *  string2File: copies a String variable to a client file
 *  </pre>
 *  These are convenience methods that illustrate file I/O techniques.
 */  

public class FileUtil {

   /**  
    *  The FileUtil class contains only static methods, and cannot be instantiated.
    */     
   private FileUtil() {}  

   /**  
    *  Invokes the <code>putString, getString, putStream, and getStream</code>
    *  methods from a client command line.
    *  <p>Usage:
    *  <pre>
    *  java jcs.util.FileUtil -S server -A [putString | putStream]
    *                   -I input-file-name   -Q sql-insert-statement 
    *  java jcs.util.FileUtil -S server -A [getString | getStream] 
    *                   -O output-file-name  -Q sql-query 
    *  </pre>
    *  <p>Assume a file named "emps.xml", and the following table:
    *  <pre>
    *  create table stringtable 
    *     (id varchar(50), textcol text, stringcol java.lang.String)  
    *  </pre>
    *  <p>Example command lines:
    *  <pre> 
    *  java jcs.util.FileUtil -A putString  -I emps.xml         \
    *     -S "antibes:4000?user=sa"                             \
    *     -Q "insert into stringtable (id, stringcol)           \ 
    *                values ('example-1', ?)"
    *
    *  java jcs.util.FileUtil  -A getString -O emps.getString   \
    *     -S "antibes:4000?user=sa"                             \
    *     -Q "select stringcol                                  \
    *         from stringtable where id='example-1' "   
    *
    *  java jcs.util.FileUtil  -A putStream -I emps.xml         \
    *     -S "antibes:4000?user=sa"                                \ 
    *     -Q "insert into stringtable (id, textcol)                \
    *                  values ('example-2', ?)"  
    *
    *  java jcs.util.FileUtil   -A getStream -O emps.getStream        \
    *     -S "antibes:4000?user=sa"                                      \
    *     -Q "select textcol from stringtable where id='example-2' " 
    *  </pre>
    *  @param args   the String[] containing command line arguments  
    *  @see #putString
    *  @see #getString
    *  @see #putStream
    *  @see #getStream
    */  

   public static void main(String args[]) {
   
      //  argTags and argVals are the argument tag letters 
      //  and the default argument values.
      //  

      String[] argTags = {"-A", "-S", "-I", "-O", "-Q"};
      String[] argVals = {"",    "",  "",   "",   ""  };

      try {  
         //  Call doCommandArgs to merge the user-supplied arguments
         //  with the defaults.

         StringUtil.doCommandArgs(args, argTags, argVals);

         String action = argVals[0].trim().toLowerCase();  
         String server = argVals[1];  
         String inFile = argVals[2];
         String outFile = argVals[3];
         String query = argVals[4];    
         String sql = "";

         if (action.equals("putstring")) {
            FileUtil.putString(inFile, query, server);
         } else if (action.equals("getstring")) {
            FileUtil.getString(outFile, query, server);
         } else if (action.equals("putstream")) {
            FileUtil.putStream(inFile, query, server);
         } else if (action.equals("getstream")) {
            FileUtil.getStream(outFile, query, server);
         } else {
            throw new IllegalArgumentException();             
         }

      } catch (IllegalArgumentException iae) {
         System.out.println("\nInvalid arguments: " + iae.getMessage() + "\n" 
            + "Usage:  java jcs.util.FileUtil -S server -A [putstring | putstream]\n"
            +"                 -I input-file-name   -Q sql-insert-statement \n"
            +" or:     java jcs.util.FileUtil -S server -A [getstring | getstream]\n "
            +"                 -O output-file-name  -Q sql-query ");
      } catch (Exception e) {
         System.out.println("\nException:");
         e.printStackTrace();
      }  
   }


   /**  
    *  Copies a client operating system file to a Java String 
    *  in the SQL server.  
    *  <p>This method can be invoked directly, or from the <code>main</code>
    *  method command line.
    *  <p>Assume a text file named "emps.xml", and the following table:
    *  <pre>
    *     create table stringtable 
    *         (id varchar(50), textcol text, stringcol java.lang.String)  
    *  </pre>
    *  <p>Example Java code:
    *  <pre>
    *  String server = "antibes:4000?user=sa";
    *  String putStringVar = " insert into stringtable (id, stringcol) "
    *             +"     values ('some-id', ?)";
    *  jcs.util.FileUtil.putString("emps.xml", putStringVar, server);
    *  </pre>
    *  @param  inFile      a string that is the name of the file from which to copy
    *  @param  insertStmt  a string containing an SQL statement that has 
    *                      a single question-mark parameter.
    *                      The datatype of that parameter must be 
    *                      <code>String</code>.
    *  @param server  a string that identifies the server in which to execute 
    *                 the statement.
    *  @throws Exception    Thrown by <code>java.sql</code> methods
    *  @see ExecSql#connectTo 
    */  
  
   public static void putString(String inFile, String insertStmt, 
                                           String server) throws Exception {    
      String workString= file2String(inFile);
      Connection con = ExecSql.connectTo(server);

      PreparedStatement pstmt = con.prepareStatement(insertStmt);
      pstmt.setObject(1, workString);
      int res = pstmt.executeUpdate();
    }


   /**  
    *  Copies a Java String from the SQL server to a client operating system file.
    *  <p>This method can be invoked directly, or from the <code>main</code>
    *  method command line.
    *  <p>Assume the following table:
    *  <pre>
    *  create table stringtable 
    *     (id varchar(50), textcol text, stringcol java.lang.String)  
    *  </pre>
    *  <p>Example Java code:
    *  <pre>
    *  String server = "antibes:4000?user=sa";
    *  String getStringVar 
    *     = "select stringcol from stringtable  where id='some-id' ";
    *  jcs.util.FileUtil.getString2("emps.getString", getStringVar, server);
    *  </pre>
    *  @param outFile  a string that is the name of the file to store into
    *  @param query    a string that contains an SQL query that returns a result set 
    *                  with one row and one column.  The datatype of that column 
    *                  must be <code>String</code>.
    *  @param server   a string that identifies the server in which to execute 
    *                  the statement
    *  @throws Exception    Thrown by <code>java.sql</code> methods
    *  @see ExecSql#connectTo 
    */  
  
   public static void getString(String outFile, String query, 
                                         String server) throws Exception {    

      ResultSet rs = ExecSql.query(query, server);
      if (rs.next()) {
         String workString = (String)rs.getObject(1);  
         if (workString==null) {
            throw new Exception("Select returned null");
         } 
         string2File(outFile, workString); 
         if (rs.next()) {
            throw new Exception("Select returned more than one row");
         }          
      } else { 
         throw new Exception("Select returned no rows"); 
      }
   }

   /**  
    *  Streams a client operating system file to a TEXT or String column in the
    *  SQL server.
    *  <p>This method can be invoked directly, or from the <code>main</code>
    *  method command line.
    *  <p>Assume a file named "emps.xml", and the following table:
    *  <pre>
    *     create table stringtable 
    *         (id varchar(50), textcol text, stringcol java.lang.String)  
    *  </pre>
    *  <p>Example Java code:
    *  <pre> 
    *  String server = "antibes:4000?user=sa";
    *  String putStreamVar 
    *       = "insert into stringtable(id, textcol) "
    *         +"values ('putStreamId', ?)";
    *  jcs.util.FileUtil.putStream("emps.xml", putStreamVar, server);
    *  </pre>
    *  @param  inFile      a string that is the name of the file from which to copy
    *  @param  insertStmt  a string that contains an SQL statement that has 
    *                      a single question-mark parameter.
    *                      The datatype of that parameter must be 
    *                      either TEXT or java.lang.String.
    *  @param server  identifies the server in which to execute the statement
    *  @throws Exception    Thrown by <code>java.sql</code> methods
    *  @see ExecSql#connectTo
    */  

   public static void putStream(String inFile, String insertStmt, String server) 
                                   throws Exception {

      int insertLength;
      Connection con = ExecSql.connectTo(server);
      PreparedStatement pstmt = con.prepareStatement(insertStmt);
      FileInputStream fis = new FileInputStream(inFile);
      pstmt.setAsciiStream(1, fis, -1);
      int res = pstmt.executeUpdate();
   }

   /**  
    *  Streams a TEXT or String column in the SQL server to 
    *  a client operating system file .
    *  <p>This method can be invoked directly, or from the <code>main</code>
    *  method command line.
    *  <p>Assume a file named "emps.xml", and the following table:
    *  <pre>
    *     create table stringtable 
    *         (id varchar(50), textcol text, stringcol java.lang.String)  
    *  </pre>
    *  <p>Example Java code:
    *  <pre>
    *   String getStreamVar 
    *       = "select textcol from stringtable where id='putStreamId'";
    *   jcs.util.FileUtil.getStream("emps.getStream", getStreamVar, server);       
    *  </pre>
    *  @param outFile  a string that is the name of the file to store into
    *  @param query    a string that contains an SQL query that returns a result set 
    *                  with one row and one column.  The datatype of that 
    *                  column must be either <code>TEXT</code> 
    *                  or <code>java.lang.String</code>.
    *  @param server  a string that identifies the server in which to execute 
    *                 the statement
    *  @throws Exception    Thrown by <code>java.sql</code> methods
    *  @see ExecSql#connectTo
    */  

   public static void getStream(String outFile, String query, String server) 
                                                             throws Exception {

      ResultSet rs = ExecSql.query(query, server);      
      String setTextSize = "set textsize 200000";
      if (rs.next()) {
         InputStream is = rs.getAsciiStream(1);               
         FileOutputStream fos = new FileOutputStream(outFile);
         byte[] buf = new byte[100];
         int len;
         if (is==null) {
            throw new Exception("Select returned null stream");
         }
         while((len = is.read(buf)) !=-1) {
            fos.write(buf, 0, len);
                 
         }
         if (rs.next()) {
            throw new Exception("Select returned more than one row");
         }          
      } else { 
         throw new Exception("Select returned no rows"); 
      }  
   }
      
    
   /**  
    *  
    *  Copies a client operating system file to a Java String 
    *  in a Client variable.   
    *  <p>Example Java code:
    *  <pre>
    *  String stringFromFile=jcs.util.FileUtil.file2String("my-file.txt");
    *  </pre>
    *  @param fileName  a string that is the name of the file from which to copy
    *  @result          a string containing the contents of the file
    *  @throws Exception    Thrown by <code>java.io</code> methods
    *  @see ExecSql#connectTo
    */  

   public static String file2String(String fileName)  throws Exception  {

      // From inFile to  stringBuffer
             
      FileInputStream inFile = new FileInputStream(fileName);                      
      StringWriter sWrite = new StringWriter();
      byte[] bytes = new byte[128]; 
      int count=0;

      while ((count = inFile.read(bytes)) != -1) {
         sWrite.write(new String(bytes, 0, count));
      }
      inFile.close();
      return sWrite.toString();
   }

   /**  
    *  
    *  Copies a Java String in a Client variable to a client operating system file.
    *  <p>Example Java code:
    *  <pre>
    *  jcs.util.FileUtil.file2String("my-file.txt", string-variable);
    *  </pre>
    *  @param  fileName  a string that is the name of the file to copy into
    *  @param  source    the string to copy into the file
    *  @throws Exception    Thrown by <code>java.io</code> methods
    *  @see ExecSql#connectTo
    */  

   public static void string2File(String fileName, String source) 
                                                   throws Exception {

      // From  source to file fileName
      OutputStream outFile = new FileOutputStream(fileName);
      StringReader sRead= new StringReader(source);
      char[] chars = new char[128];
      int count=0;
      while ((count = sRead.read(chars, 0, 128)) != -1) {
         outFile.write(new String(chars, 0, count).getBytes(), 0, count);
      }          
      outFile.close();            
   }
}