//
//  Copyright (c) 1999, 2001
//  Sybase, Inc
//  Emeryville, CA 94608
//  All Rights Reserved
//
package jcs.xml;
import jcs.util.StringUtil;
import java.io.*;
import java.util.*;

import javax.xml.parsers.DocumentBuilder; 
import javax.xml.parsers.DocumentBuilderFactory;  
import javax.xml.parsers.FactoryConfigurationError;  
import javax.xml.parsers.ParserConfigurationException;  


import org.apache.xml.serialize.*;

import org.xml.sax.SAXException;  
import org.xml.sax.SAXParseException;  
import org.w3c.dom.*;


/**  
 *  
 *  A uninstantiable class with miscellaneous utility methods 
 *  to access a DOM tree from parsed XML.
 */  

public class DomTree {

   /**  
    *  The DomTree class contains only static methods, 
    *  and cannot be instantiated.
    */     
   private DomTree() {}  

   /**  
    *  Returns the value of an attribute for a given XML node 
    *  and attributeName. 
    *  For example, if the <code>node</code> is positioned 
    *  on <code>&lt;column null=true&gt;</code>, 
    *  a call of <code>getAttribute(node, "null")</code> returns 
    *  the string "true".
    *  @param   node  a Node that is positioned on a node of a DOM tree
    *  @param   attributename  a string that is the name of an attribute
    *                          of the <code>node</code>
    *  @return  a String that is the value of the given attribute 
    *  of the node, or null
    */  
   public static String getAttribute(Node node, String attributeName) {

      if (node == null) return "";
      NamedNodeMap nnm = node.getAttributes();
      if (nnm == null || nnm.getLength() == 0) {
         return "";
      } else {
         Node wantedNode = nnm.getNamedItem(attributeName);
         if (wantedNode==null) {
            return "";
         } else {
            return wantedNode.getNodeValue();
         }       
      }
   }

   /**  
    *  Sets the value of an attribute for a given XML node 
    *  and attributeName. 
    *  For example, if the <code>node</code> is positioned 
    *  on <code>&lt;column null=true&gt;</code>, 
    *  a call of <code>setAttribute(node, "null" "false")</code> sets the
    *  the "null" attribute to  "true" (or inserts a new "null" attribute).

    *  @param   node  a Node that is positioned on a node of a DOM tree
    *  @param   attributename  a string that is the name of an attribute
    *                          of the <code>node</code>
    *  @param   attributeValue  a string that is the value of the attribute
    */  

   public static void setAttribute(Element node, String attributeName, 
                             String attributeValue) {
      node.setAttribute(attributeName, attributeValue);
   }

   /**  
    *  
    * Returns a subnode of a given XML node, given the sub-node name.
    * <p>For example 
    * <ul>
    * <li>If the <code>node</code> is positioned 
    * on a <code>&lt;ResultSet&gt;</code> node,
    * and that node has a child node <code>&lt;ResultSetMetaData&gt;</code>,
    * then the call <code>findNamedNode(node, "ResultSetMetadata", 0)</code> 
    * returns the (first or only) <code>&lt;ResultSetMetaData&gt;</code> node, 
    * or null.
    * <li.If the <code>node</code> is positioned on a  
    * <code>&lt;ResultSetData&gt;</code> node that has multiple child nodes
    * <code>&lt;Row</code>, then the call 
    * <br><code>findNamedNode(node, "ResultSetMetadata", 3)</code> 
    * <br>will return the 3rd <code>&lt;Row</code>,
    * if there is one, and otherwise will return <code>null</code>.
    * </ul> 
    * @param  node  a Node that is positioned on a node of a DOM tree
    * @param  nodeName   a string that is the name of a child of the node
    * @param  nodeNumber   a string that is the number of a child of the node
    * @return  the given node, or null
    */  


   public static Node findNode(Node node, String nodeName, int nodeNumber) {
      if (node==null || !node.hasChildNodes()) {
         return null;
      } else {
         int currentNodeNumber = -1;
         NodeList nl = node.getChildNodes();
         int size = nl.getLength();
         for (int i = 0; i<size; i++) {
             String gNN = nl.item(i).getNodeName();

             if (gNN!=null && gNN.equals(nodeName) 
                  && ++currentNodeNumber == nodeNumber) {                
                return nl.item(i);
             }
         } 
         return null;
      }     
   }


   /**  
    *  A convenience shorthand for <code>findNode(N, NN, 0)</code>
    * @param  node  a Node that is positioned on a node of a DOM tree
    * @param  nodeName   a string that is the name of a child of the node
    * @param  nodeNumber   a string that is the number of a child of the node
    * @return  the given node, or null
    */  

   public static Node findNode(Node node, String nodeName) {
      return findNode(node, nodeName, 0);
   }

   /**  
    *  Returns the text of a given XML node.
    *  For example, if "node" is positioned on 
    *  <code>&lt;Column&gt;123&lt;/column&gt;</code>,
    *  then <code>getText(node)</code> returns "123".
    *  @param node  a Node that is positioned on a node of a DOM tree
    *  @return  a string that is the text of the given node, or null
    */  
   public static String getText (Node node) {
      Node textNode = getTextNode(node);
      if (textNode==null) return "";
      if (textNode.getNodeName().equals("#text")) {
         return textNode.getNodeValue();
      } else if (textNode.getNodeName().equals("#cdata-section")) {
         return textNode.getNodeValue();
      }    
      return "";
  
   }

   /**  
    *  Sets the text of a given XML node.  If the given node is
    *  null or doesn't exist, then no action and returns null.
    *  For example, if <code>node</code> is positioned on 
    *  <code>&lt;Column&gt;123&lt;/column&gt;</code>,
    *  and <code>doc</code> is the Document containing <code>node</code>,
    *  then <code>setText(doc, node, "987")</code> sets the value to 987
    *  and returns the updated node.
    *  @param node  a Node that is positioned on a node of a DOM tree
    *  @return  the updated node, or null if no update was performed.
    */  
   public static Node  setText (Document doc, Node targetNode, 
                                             String newValue) {

      if (targetNode==null) return null;
      Node targetTextNode = getTextNode(targetNode);
      targetNode.replaceChild(doc.createTextNode(newValue), 
                     targetTextNode);
      return targetNode;
  
   }

   /**  
    * Returns the child node containing text, for a given XML node.
    * For example if "node" is positioned on 
    * <code>&lt;Column&gt;123&lt;/column&gt;</code>,
    * then <code>getText(node)</code> will return the node with "123".
    * Use this method when you want to update the text node.
    * @param node  a Node that is positioned on a node of a DOM tree
    * @return  the <code>#text</code> or <code>#cdatasection</code> node
    *          that is a child of the node
    */  
   public static Node getTextNode (Node node) {
      if (node==null) return null;
      NodeList nl = node.getChildNodes();
      int size = nl.getLength();
      for (int i=0; i<size; i++)  {
          Node nli = nl.item(i);
          String nodeName = nli.getNodeName();
          if (nodeName.equals("#text") 
                 || nodeName.equals("#cdata-section") ) {
             return nli;
          }
      }
      return null;
  
   }


   /**  
    *  Returns the index (if any) of the next Node in Nodelist <code>nl</code> 
    *  at or after the <code>from</code> index, whose node name is 
    *  in the <code>given</code> string.
    *  Otherwise, returns -1. 
    *  This is mainly handy for skipping over <code>#text</code> nodes 
    *  with whitespace.
    *  @param nl      a NodeList that is the child nodes of a DOM tree node
    *  @param from    an int that is the index of the first child node in the
    *                 NodeList to consider.  For example, if you use this
    *                 method to step through the child nodes, then the
    *                 <code>from</code> value will be one more 
    *                 than the index of the last child node that you processed.
    *  @param given   a string that is the name of the child node 
    *                 that is sought
    *  @return  an int that is the index of the node with the given name in
    *           the NodeList, or is -1 if there are no (more) such nodes
    */  
   public static int nextGivenChild(NodeList nl, int from, String given )  {
      for (int i = from;  i<nl.getLength(); i++) {
         if (given.indexOf(nl.item(i).getNodeName())!=-1) {
            return i;
         }
      }
      return -1;
   }

   /**  
    *  
    *  Returns the index (if any) of the next Node in Nodelist <code>nl</code>, 
    *  at or after the <code>from</code> index, that is not whitespace.
    *  This is mainly handy for skipping over <code>#text</code> nodes 
    *  with whitespace.
    *  @param nl      a NodeList that is the child nodes of a DOM tree node
    *  @param from    an int that is the index of the first child node in the
    *                 NodeList to consider.  For example, if you use this
    *                 method to step through the child nodes, then the
    *                 <code>from</code> value will be one more than 
    *                 the index of the last child node that you processed.
    *  
    */  
   public static int nextNonWhiteSpace(NodeList nl, int from)  {
      for (int i = from;  i<nl.getLength(); i++) {
         if (!StringUtil.isNothing(nl.item(i).getNodeValue())) {
            return i;
         }
      }
      return -1;
   }


   /**  
    *  Checks that the name of the root node of an XML document 
    *  is a given name.
    *  @param Document  a parsed XML document
    *  @param rootName a string that is the expected name of the root node
    *                  of the document
    *  @return an Document that is the parsed object
    *  @throws Exception  Any exceptions arising in the XML parse, 
    *                        or a root name that is not <code>rootName</code>
    */  

   public static Document checker(Document doc, String rootName) 
                 throws Exception {
      // Get the root Element.
      Element root = doc.getDocumentElement();

      if (root==null) return null;
      if (root.getNodeType()==Node.TEXT_NODE 
         && StringUtil.isNothing(root.getNodeValue()))  {
         return null ;
      }
      String gNN = root.getNodeName();
      if (!gNN.equals(rootName)) {
         throw  new Exception ("Unexpected ("+gNN+")");
      }
      return doc;
   }
         

   /**  
    *  Checks that the root node of an XML document has a given name,
    *  and returns that root node. This method is convenient when you only 
    *  want to read the parsed document, i.e. you don't want to modify it.
    *  @param  doc  a parsed XML document
    *  @param rootName a string that is the expected name of the root node
    *                  of the document
    *  @return an Document that is the parsed object.
    *  @throws Exception  Any exceptions arising in the XML parse, 
    *                        or a root name that is not <code>rootName</code>
    */  
   public static Node topNode(Document doc, String rootName) 
                            throws Exception {
      //  Create an instance of com.sybase.xml.ServerParse

      Document doc2 = checker(doc, rootName);
      Element root = doc2.getDocumentElement();
      return root;
   }

   /**  
    *  Append a node with a given name as a child of a given parent node.
    *  For example, if <code>doc</code> contains a <code>node</code that
    *  is a <code>&lt;ResultSetData&gt;</code> node, then the call
    *  <code>appendChild(doc, node,  "Row")</code> will append a new
    *  <code>&lt;Row&gt;</code> node and return it.
    *  @param  doc        a parsed XML document
    *  @param parent      a node of doc that will be the parent of the new node
    *  @param childName   a string that will be the name of the new child node
    *  @return            the new child node
    */  

   public static Element appendChild(Document doc, Node parent, 
                        String childName )  {
      Element childNode = (Element)doc.createElement(childName);      
      parent.appendChild( (childNode));
      return childNode;
   }

   /**  
    *  Append a node with a given name and text as a child of a given parent node.
    *  For example, if <code>doc</code> contains a <code>node</code that
    *  is a <code>&lt;Row&gt;</code> node, then the call
    *  <code>appendChild(doc, node,  "Column", "Apples", false)</code> will append
    *  a new <code>&lt;Column&gt;Apples&lt;/Column&gt;</code> node and return it.
    *  @param  doc        a parsed XML document
    *  @param parent      a node of doc that will be the parent of the new node
    *  @param childName   a string that will be the name of the new child node
    *  @param childText   the text of the new child node
    *  @param cData       a boolean <b>true</b> iff the text should be a CDATA section
    *  @return            the new child node
    */  

   public static Element appendChild(Document doc, Node parent, 
                  String childName, String childText, boolean cData )  {
      Element childNode = (Element)doc.createElement(childName);
      parent.appendChild(childNode );
      if (cData) {
         childNode.appendChild(doc.createCDATASection(childText));
      } else {
         childNode.appendChild(doc.createTextNode(childText));
      }
      return childNode;
   }

   /**  
    *  Return a new empty <code>Document</code> object.
    *  @param  validate  a boolean indication of whether (true) or not (false)
    *                    the new <code>Document</code> should validate.
    *  @return            the new <code>Document</code> object.
    */  

   public static Document newDocument () throws Exception {

       // Get Document Builder Factory
       DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
       // Set validation, and turn off namespaces
       factory.setValidating(false);
       factory.setNamespaceAware(false);
       DocumentBuilder builder = factory.newDocumentBuilder();
       return builder.newDocument();
   }

   /**  
    *  Return a String XML document from a <code>Document</code>.
    *  @param  doc   a <code>Document</code>
    *  @return       a String containing the text of the doc.
    */
   public static String doc2String (Document doc) throws Exception {

      try{

        ByteArrayOutputStream outStream = new ByteArrayOutputStream();
        OutputFormat outForm = new OutputFormat(doc, "UTF-8", true);
        XMLSerializer ser = new XMLSerializer(outStream, outForm);
        ser.asDOMSerializer().serialize(doc);
        return outStream.toString();
 
      } catch (Exception e) {
         throw new Exception( "** Document output error" 
                + e.getMessage());


      }

   }

}




