//
//  Copyright (c) 1999, 2001
//  Sybase, Inc
//  Emeryville, CA 94608
//  All Rights Reserved
//
package jcs.xml.resultset;

import jcs.util.*;
import jcs.xml.resultset.*;
import jcs.xml.*;

import java.io.*;
import java.sql.*;
import java.util.*;

/**  

 *  A client command line interpreter to invoke
 *  the <code>ResultSetXml</code> class.  
 *  <p>This command line interpreter invokes the <code>ResultSetXml</code> 
 *  constructor that evaluates an SQL query and translates the result set 
 *  to an XML document. 
 *  <p>The  <code>ResultSetXml</code> class can be invoked in either a
 *  client or server environment.  This class <code>DoSql2Xml</code> 
 *  invokes the class constructor in a client environment, using a client 
 *  file for the results.
 */  


public class DoSql2Xml {

   /**  
    *  The DoSql2Xml class contains only static methods, and cannot be instantiated.
    */    
   private DoSql2Xml() {}  

   /**  
    *  Invokes the <code>query2Xml</code> method from a client 
    *  command line.
    *  <p>Usage:
    *  <pre>
    *  java jcs.resultset.DoSql2Xml -Q query [-C cdataColumns] 
    *       [-N columnNameOption]  -O output-file-name  -S server 
    *  </pre>
    *  <p>Example command line:
    *  <pre> 
    *  java jcsexec.DoSql2Xml -Q "select d.name, e.name, d.dno       \
    *       from emps e, dept d  where e.dno = d.dno order by d.name, e.name"  \
    *       -C  "000" -N "yes" -S "antibes:4000?user=sa"  -O emp-dept.xml
    *  </pre> 
    *  <p>This call will evaluate the query (<b>-Q</b>), generate an XML result
    *  set with the query's metadata and data, and store the XML document in
    *  file <code>emp-dept.xml</code>.
    *  <p>The parameters tags are as follows.  For a detailed description
    *  of the <code>query</code>, <code>cdataColumns</code>, 
    *  <code>columnNamesOption</code>, and <code>server</code>  parameters, 
    *  see the <code>jcs.resultset.resultSetXml</code> constructor,
    *  which this command-line routine invokes.  For a description of the
    *  <code>server</code> parameter, see the method 
    *  <code>jcs.util.ExecSql.connectTo</code>.
    *  <ul>
    *  <li><b>-Q</b>:  a string that is an SQL query returning a result set
    *  <li><b>-C</b>:  the <code>cdataColumns</code> parameter 
    *  <li><b>-N</b>:  the <code>columnNamesOption</code> parameter 
    *  <li><b>-O</b>:  the name of the client file for the output XML document 
    *  <li><b>-S</b>:  the <code>server</code> parameter
    *  </ul>
    *  @param args  a string array with the command line arguments    
    */  

   public static void main(String args[]) {
   
      //  argTags and argVals are the argument tag letters 
      //  and the default argument values.
      //  

      String[] argTags = {"-S", "-O", "-Q", "-C", "-N"};
      String[] argVals = {"",   "",   "",   "",   "yes"};

      try {  
         //  Call doCommandArgs to merge the user-supplied arguments
         //  with the defaults.

         StringUtil.doCommandArgs(args, argTags, argVals);

         //  Copy the resulting supplied/defaulted args to named variables.
         String system = argVals[0];  
         String outFile = argVals[1];
         String query = argVals[2];    
         String cdataColumns = argVals[3];
         String columnNameOption = argVals[4].toLowerCase();

         String workString = "";    
             
         if ( outFile == "" || system == "" || query == "") {
            throw new IllegalArgumentException("the 'clientcall' action "
                + "requires the -O, -S, and -Q parameters. ");
         }
            
         // Construct a ResultSet object, using the constructor that evaluates
         // a query and converts it to an XML document.
         ResultSetXml rsx = new ResultSetXml(query, cdataColumns, columnNameOption, system);

         // Get the XML document from that ResultSetXml object.
         workString = rsx.toString();

               
         // Check for empty result.
         if (workString==null) {
            System.out.println("\nInfo:  Query returned no row"); 
            return;
         }

         // Copy the returned XML document to the designated output file.
         FileUtil.string2File(outFile, workString);

         System.out.println("\nInfo:  " 
                   + "Retrieved XML result set into '"+outFile+"'");

      } catch (IllegalArgumentException iae) {
         System.out.println("\nInvalid arguments: " + iae.getMessage() + "\n" 
            + "Usage:  java jcsexec.DoSql2Xml -S server:port -O output-file-name \n"
            + "          -Q query  [-C cdata-columns] [-N column-names] \n");

      } catch (Exception e) {
         System.out.println("\nError:  Unexpected exception:");
         e.printStackTrace();
      }  
   }

} 
 






