//
//  Copyright (c) 1999, 2001
//  Sybase, Inc
//  Emeryville, CA 94608
//  All Rights Reserved
//
package jcs.xml.resultset;
import jcs.util.*;
import jcs.xml.*;

//import com.sun.xml.tree.ElementNode;
//import com.sun.xml.tree.XmlDocument;

import org.w3c.dom.*;

import java.io.*;
import java.sql.*;
import java.util.Vector;

/**
 *  A class whose static method <code>sql2Xml</code> converts the SQL 
 *  result set yielded by a given SQL query into an XML 
 *  <code>&lt;ResultSet&gt;</code> document.  
 */

public class  RX implements java.io.Serializable  {

   /**  
    *  The RX class contains only static methods, and cannot be instantiated.
    */     
   private RX() {}  

   /**  
    *  Returns an XML representation of the SQL result set returned 
    *  by the argument query.
    *  <p>This method is the body of the <code>ResultSetXml</code> 
    *  constructor.  It is placed in a separate class to make it easier 
    *  to examine and maintain.  It can also be useful as a standalone 
    *  routine.
    *  <p>Example Java code:
    *  <pre> 
    *  String  rsx 
    *     = RX.sql2Xml("select d.name, e.name, d.dno "
    *            +"from emps e, dept d "
    *            + " where e.dno = d.dno order by d.name, e.name",
    *           "000", "yes", "antibes:4000?user=sa"); 
    *  </pre> 
    *  <p>This call evaluates the given query and returns a String containing
    *  an XML representation of the result set.
    *  <p>Comparison of this method <code>RX.sql2Xml</code> and the constructor
    *  for  <code>ResultSetXml</code>.
    *  <ul>
    *  <li>Method <code>RX.sql2Xml(...)</code> returns the XML result set as
    *  a String.  Use this method when you simply want the result set document.
    *  <li>Constructor <code>ResultSetXml(...)</code> invokes  
    *  <code>RX.sql2Xml</code>, and returns a <code>ResultSetXml</code>
    *  object containing the result set string.  
    *  Use this when you want to store the result set document as a typed object,
    *  and then use the other methods of <code>ResultSetXml</code> to access and
    *  update the components of the result set.
    *  </ul>
    *  <p>The <code>cdataColumns</code> parameter is a string whose i-th character
    *  is <code>0</code> or <code>1</code> indicating whether (or not) 
    *  the i-th column should be a <code>CDATA</code> section.  For example, 
    *  the following value of the <code>cdataColumns</code> parameter would 
    *  indicate that columns 2, 5, and 7 should be <code>CDATA</code> sections,
    *  and columns 1, 3, 4, 6, 8, and 9 are normal text:
    *  <pre>
    *      "010010100"
    *  </pre>
    *  <p>The <code>columnNameOption</code> parameter indicates whether the
    *  output XML document should include column name attributes in the 
    *  individual column tags.  The column names are not needed or used by the
    *  <code>XS.resultSet2Script</code> method.  The column names increase
    *  the size of the output XML document, but improve its readability and
    *  may be useful for some applications.
    *  @param query  a string that is the SQL query whose result set is to be
    *                returned as an XML document
    *  @param cdataColumns  a string indicating which columns of the result set 
    *                        should be specified as <code>CDATA</code> sections 
    *                        in the output XML document.  See above.
    *  @param columnNameOption   a string indicating whether (<code>yes</code>)
    *                            or not (<code>no</code>) the <code>
    *                            &lt;Column&gt;</code> tags in the output XML
    *                            document should have column name attributes.
    *                            See above.
    *  @param  server  a string that identifies the SQL server in which to execute
    *                  the query.  See the {@link ExecSql#connectTo connectTo} method 
    *                  for a description of this parameter.
    *  @return a string with the output XML document that contains the XML
    *          representation of the result set  
    *  @throws ClassNotFoundException  Thrown when the XML parser class 
    *                                     cannot be loaded.
    *  @throws SQLException  Thrown for any exceptions raised during execution
    *                           of the SQL statement
    *  @throws Exception  Thrown for invalid parameters and other conditions
    */  

   public static String sql2Xml(String query,
               String cdataColumns, String columnNameOption, String server) 
         throws ClassNotFoundException, SQLException, Exception {  



      //  Check parameters. 
      String columnNameOptionVar 
        = StringUtil.validateArg(columnNameOption, "yes,no",
             "Invalid 'columnNameOption' ('yes' or 'no'): ");

      //  Create the output XML document, "doc".
      Document doc = jcs.xml.DomTree.newDocument();

      //  Create a "<ResultSet>" element and attach it to the doc.
      Element root = DomTree.appendChild(doc, doc, "ResultSet");

      //  Evaluate the SQL query to get the result set rs.
      ResultSet rs  = ExecSql.query(query, server );

      //  'columnNames' will hold the column names.  
      //  We get the column names from doResultSetMetaData, 
      //  and use them in doResultSetData.
      Vector columnNames = new Vector();
      StringBuffer charColumns = new StringBuffer(100);
      int columnCount = doResultSetMetaData(rs, doc, charColumns, columnNames);

      //Append a dummy item as element-0, so that the i-th character will be for the i-th column.
      charColumns.append("x"); 

      doResultSetData(rs, doc, cdataColumns, charColumns.toString(), columnNameOptionVar, 
                                   columnNames, columnCount );

      //  The output XML document is now complete, in "doc".

      return jcs.xml.DomTree.doc2String(doc);

   }

   /**  
    *  Processes the result set's metadata and appends a 
    *  <code>&lt;ResultSetMetadata&gt;</code>
    *  node to the <code>ResultSet</code>.
    *  @param rs   the ResultSet of the query
    *  @param root  the root node of the output XML document. This method could
    *               obtain the root node from the <code>doc</code> parameter,
    *               but it's simpler for the <code>sql2Xml</code> method
    *               to remember the root when appending it.
    *  @param doc  the Document that will be the output. This is needed in addition
    *              to the root node, for use in creating new nodes.
    *  @param columnNames  a Vector in which the method will store the column names.
    *                      This vector is then passed to the
    *                      <code>doresultSet</code> method, so that it can insert
    *                      column names in the <code>&lt;Column&gt;</code> tags.

    *  @throws Exception  To be added.
    */  

   private static int doResultSetMetaData(ResultSet rs, Document doc, 
                StringBuffer charColumns, Vector columnNames) throws Exception {

      //  Re-position on the root
      Element root = (Element)DomTree.findNode(doc, "ResultSet");

      //  Create a "<ResultSetMetaData>" element and append it to the root.
      //  We will fill-in this element below.

      Element resultSetMetaData 
           = DomTree.appendChild(doc, root, "ResultSetMetaData");

      //  Get the result set meta data
      ResultSetMetaData rsmd = rs.getMetaData();

      //  Get the column count, 
      //  and add a columncount attribute to the <ResultSetMetaData> element.
      int columnCount =  rsmd.getColumnCount();
      resultSetMetaData.setAttribute("getColumnCount", 
                                      String.valueOf(columnCount));

      //  Set the 0-th element of columnNames to a dummy "".
      //  This is for convenience, so that the i-th element of columnNames
      //  will be the i-th column name.
      columnNames.addElement("");

      //  Process the metadata for each of the columns.
      for(int i=1; i<=columnCount; i++) {

         //  Create a <ColumnMetaData> element 
         //  and append it to the <ResultSetMetaData> element 
         Element columnMetaData 
            = DomTree.appendChild(doc, resultSetMetaData, "ColumnMetaData");

         //  Now (for the rest of this for-loop) fill-in the <columnMetadata> element

         // method getCatalogName not currently used 
         // columnMetaData.setAttribute("getCatalogName", rsmd.getCatalogName(i));

         columnMetaData.setAttribute("getColumnDisplaySize", 
                                   String.valueOf(rsmd.getColumnDisplaySize(i)));

         columnMetaData.setAttribute("getColumnLabel", rsmd.getColumnLabel(i));

         //Save the column name for an optional attribute of <column>
         //Note:  the setAttribute method implicitly entityizes the argument.
         columnNames.addElement(rsmd.getColumnName(i));  
         columnMetaData.setAttribute("getColumnName", rsmd.getColumnName(i));    

         String colType = String.valueOf(rsmd.getColumnType(i));
         columnMetaData.setAttribute("getColumnType", colType);
         String colTypeSQL =  XR.lookUpTypeCode(colType);
         if (XR.isQuotedType(colTypeSQL)){
	      charColumns.append("1");
         } else {
	      charColumns.append("0");
         }
	 

         // method getColumnTypeName not currently used
         // columnMetaData.setAttribute("getColumnTypeName", 
         //                                  rsmd.getColumnTypeName(i));

         columnMetaData.setAttribute("getPrecision", 
                       String.valueOf(rsmd.getPrecision(i)));

         columnMetaData.setAttribute("getScale", String.valueOf(rsmd.getScale(i)));

         // method getSchemaName not currently used
         // columnMetaData.setAttribute("getSchemaName", rsmd.getSchemaName(i));

         // method getTableName not currently used
         // columnMetaData.setAttribute("getTableName", rsmd.getTableName(i));

         columnMetaData.setAttribute("isAutoIncrement", 
                          String.valueOf(rsmd.isAutoIncrement(i)));

         // method isCaseSensitive not currently used
         // columnMetaData.setAttribute("isCaseSensitive", 
         //                     String.valueOf(rsmd.isCaseSensitive(i)));

         columnMetaData.setAttribute("isCurrency", String.valueOf(rsmd.isCurrency(i)));

         columnMetaData.setAttribute("isDefinitelyWritable", 
                                String.valueOf(rsmd.isDefinitelyWritable(i)));

         columnMetaData.setAttribute("isNullable", 
             String.valueOf((rsmd.isNullable(i)==ResultSetMetaData.columnNoNulls)
                               ?false:true));

         // method isSearchable not currently used
         // columnMetaData.setAttribute("isSearchable", 
         //                 String.valueOf(rsmd.isSearchable(i)));

         columnMetaData.setAttribute("isSigned", String.valueOf(rsmd.isSigned(i)));

         // method isWritable not currently used
         // columnMetaData.setAttribute("isWritable", 
         //                     String.valueOf(rsmd.isWritable(i)));

      }  //  End of metadata for-loop
      return columnCount;
   }


   /**  
    *  
    *  Processes the data rows of the result set, 
    *  and appends a <code>&lt;ResultSetData&gt;</code> node to the 
    *  <code>ResultSet</code>.
    *  @param rs   the ResultSet of the query
    *  @param doc  the Document that will be the output
    *  @param cdataColumns       a string whose i-th character indicates
    *                            whether (<code>1</code>) or not (<code>0</code>)
    *                            the i-th column should be CDATA.
    *                            <code>cdataColumns</code> can also be <code>all</code>
    *                            or <code>none</code>.
    *  @param columnNameOption   a string indicating whether (<code>yes</code>)
    *                            or not (<code>no</code>) the <code>
    *                            &lt;Column&gt;</code> tags in the output XML
    *                            document should have column name attributes.
    *  @param columnNames  a Vector containing the column names.  This vector was built 
    *                      by the <code>doResultSetMetaData</code> method.
    *                      This method will use it to place column names in the 
    *                      <code>&lt;Column&gt;</code> tags.
    *  @param  columnCount   an int with the number of columns in the result set.
    *                        This is a convenience for the method.
    *  @throws Exception  To be added.

    */  

   private static void doResultSetData(ResultSet rs, Document doc, 
         String cdataColumns, String charColumns, String columnNameOption, Vector columnNames, 
           int columnCount )   throws Exception {

      // Work variables
      String xString = "";
      int xint;
      boolean xboolean;


      //  Create a "<ResultSetData>" element and append it to the root.
      //  We will fill-in this element below.
      Element resultSetData
          = (Element)doc.createElement("ResultSetData");
      Element root = (Element)DomTree.findNode(doc, "ResultSet");
      root.appendChild(resultSetData);

      Element row=null;
      Element column=null;
      while (rs.next()) {
         //  Create a <Row> element and append it to the <ResultSetData> element.
         row = DomTree.appendChild(doc, resultSetData, "Row");

         //   Process the column data for this row.      
	 for(int i=1; i<=columnCount; i++) {

            //  Create a <Column> element and append it to the <Row> element. 
            xString = rs.getString(i);
            xString = (xString==null)?"":xString;              
            if (charColumns.substring(i, i)=="1") {
               xString = jcs.util.StringUtil.entityize(xString);
            }

            // Check the "cdataColumns" parameter to see if column i
            // should be treated as a CDATA section.
            if (cdataColumns == null || cdataColumns.equals("none")) {
               xboolean = false;
            } else if (cdataColumns.equals("all")) {
               xboolean = true;
            } else if (cdataColumns.length()>=i 
                       && cdataColumns.substring(i-1, i).equals("1")) {
               xboolean = true;
            } else {
               xboolean = false;
            }

            //  Append a "<Column>" with the given value.  We'll fill-in
            //  the "null" attribute after the column is appended.
            //  The 'xboolean' parameter indicates whether the column
            //  is a CDATA section.
            column = DomTree.appendChild(doc, row, "Column", xString, xboolean);

            boolean wasNull = rs.wasNull();
            if (wasNull)  {
               column.setAttribute("null", "true");
            } 
            // Note:  the "setAttribute" method implicitly entityizes the argument.
            if (columnNameOption.equals("yes")) {
               column.setAttribute("name", (String)columnNames.elementAt(i));
            }     
         }  // End of for-loop for columns in a row.
      } //  End of while-loop for rows in the result set
   }
}
