//  Copyright (c) 1999, 2001
//  Sybase, Inc
//  Emeryville, CA 94608
//  All Rights Reserved
//
package jcs.xml.resultset;

import jcs.util.*;
import jcs.xml.*;

import org.w3c.dom.*;
import org.xml.sax.SAXException;

import java.io.*;
import java.sql.*;
import java.util.Vector;

/**  
 *  A class to contain an XML document that represents an SQL result set.
 */  

public class ResultSetXml extends JXml implements java.io.Serializable {

   /**  
    *  Returns the ResultSet DTD.   This method is called by the <code>parse</code>
    *  method of the superclass JXml, to obtain the DTD for a given ID.
    *  The <code>publicId</code> and <code>systemId</code> parameters
    *  are the PUBLIC and SYSTEM values from an external entity declaration
    *  in the document being parsed.
    *  If the SYSTEM value is "ResultSet.dtd", then this method returns the
    *  ResultSet DTD.   Otherwise, it raises an exception, which indicates
    *  that a document other than a Result Set XML document is being parsed
    *  as a ResultSet XML document.
    *  @param publicId    a String with the PUBLIC value from an external entity
    *                     declaration.
    *  @param systemId    a String with the SYSTEM value from an external entity
    *                     declaration.
    *  @return   a string containing the ResultSet DTD.
    *  @throws  SAXException   Thrown when the <code>systemId</code> 
    *                             is not "ResultSet.dtd".
    */  

   public String resolveExternal(String publicId, String systemId)  throws SAXException{
      if ( (systemId!=null && systemId.toLowerCase().indexOf(DTD_NAME)!= -1)
		||  (publicId!=null && publicId.toLowerCase().indexOf(DTD_NAME)!= -1)) {
          return DTD;
      }  else {
         throw new SAXException ("Invalid external reference for ResultSetXml\n"
           +"\t publicID is (" + publicId + ") systemID is (" + systemId
           +")\n\t expected name containing ("+ DTD_NAME +")" );
      }
   }

   /**  
    *   Constructs a <code>ResultSetXml</code> instance from 
    *   an existing XML document.
    *   @param  doc  a string that is an XML document.  The document must
    *                be a valid SQL result set, using the JavaSqlResultSet DTD.
    *   @throws Exception  To be added
    */  

   public ResultSetXml(String xmlText) throws Exception {
      super(xmlText);
      this.parse(true);
   }

   
   /**  
    *  Constructs a <code>ResultSetXml</code> instance from the 
    *  SQL result set returned  by the argument query.
    *  <p>Example Java code:
    *  <pre> 
    *  ResultSetXml rsx 
    *     = new ResultSetXml("select d.name, e.name, d.dno "
    *            +"from emps e, dept d "
    *            + " where e.dno = d.dno order by d.name, e.name",
    *           "000", "yes", "antibes:4000?user=sa"); 
    *  </pre> 
    *  @param query  a string that is the SQL query whose result set is to be
    *                returned as an XML document.  The query must have no
    *                question mark parameters, and must return a single result set.
    *  @param cdataColumns  a string indicating which columns of the result set 
    *                        should be specified as <code>CDATA</code> sections 
    *                        in the output XML document.  See above.
    *  @param columnNameOption   a string indicating whether (<code>yes</code>)
    *                            or not (<code>no</code>) the <code>
    *                            &lt;Column&gt;</code> tags in the output XML
    *                            document should have column name attributes.
    *                            See above.
    *  @param server  a string that identifies the server in which to execute
    *                 the query. See the {@link ExecSql#connectTo connectTo} 
    *                 method for a description of this parameter.
    *  @throws ClassNotFoundException  Thrown when the XML parser class 
    *                                     cannot be loaded
    *  @throws SQLException  Thrown for any exceptions raised during execution
    *                           of the SQL statement
    *  @throws Exception  Thrown for invalid parameters and other conditions
    */  

   public ResultSetXml(String query,
               String cdataColumns, String columnNameOption, String server) 
         throws ClassNotFoundException, SQLException, Exception {  
      super( RX.sql2Xml(query, cdataColumns, columnNameOption, server));

   }

   /**
    *  Generates an SQL script from the <code>ResultSetXml</code> document.
    *  The SQL script will contain an SQL <code>create</code> statement and 
    *  a sequence of SQL <code>insert</code> statements that 
    *  create and populate a table with the data of the result set.
    *  <p>The <code>goOption</code> parameter indicates whether you want
    *  the output script to include <code>go</code> commands after
    *  the <code>create</code> and <code>insert</code> statements.
    *  This depends on how you plan to execute the script:  ISQL requires
    *  the <code>go</code> commands, and JDBC doesn't allow them. 
    *  @param resultTableName   a string that is the SQL table name to
    *                           specify in the output <code>create</code> and 
    *                           <code>insert</code>  statements
    *  @param columnPrefix  a string to use as a prefix when generating
    *                       column names in the output script.  Such generated
    *                       names are needed when the result set has columns
    *                       with no names or with duplicate names.
    *  @param goOption    a string indicating whether the output script should
    *               include <code>go</code> commands. This is indicated by
    *               <code>yes</code> or <code>no</code>.  See above.
    */
   public String toSqlScript(String resultTableName, String columnPrefix, String goOption) 
				throws Exception {
      return XR.toSqlScript(this.parse(), resultTableName, columnPrefix, goOption);
   }

   /**  
    *  Returns the String value of row 'rowNumber'  column 'columnNumber'. 
    *  @param rowNumber  an int that is the number of a row of the result set
    *  @param columnNumber  an int that is the number of a column of the result set
    *  @return   a string that is the value of the given column of the given row,
    *            or is null if that element is null or doesn't exist.
    *  @throws   Exception    Thrown if the document is invalid.
    */
   public String getColumn(int rowNumber, int columnNumber) throws Exception {
      return this.getColumn(rowNumber, columnNumber, null);
   }

   /**  
    *  Return the String value of row 'rowNumber'  column 'columnName'. 
    *  @param rowNumber  an int that is the number of a row of the result set
    *  @param columnName  a string that is the name of a column of the result set
    *  @return   a string that is the value of the given column of the given row,
    *            or is null if that element is null or doesn't exist.
    *  @throws     To be added

    */
   public String getColumn(int rowNumber, String columnName) throws Exception {
      return this.getColumn(rowNumber, -1, columnName);
   }

   /** 
    *  Returns the String value of row number i and the column identified 
    *  by either the columnNumber or columnName. This method is 
    *  the common code for the public <code>getColumn</code> methods.
    *  @param rowNumber  an int that is the number of a row of the result set
    *  @param columnNumber  an int that is the number of a column of the result set
    *  @param columnName  a string that is the name of a column of the result set
    *  @return   a string that is the value of the given column of the given row,
    *            or is null if that element is null or doesn't exist.
    *  @throws     To be added
    */
   private String getColumn(int rowNumber, int columnNumber, String columnName) 
			throws Exception {

      Document doc = this.parse();
      Node resultSet = DomTree.topNode(doc, "ResultSet");
      // Position on the "ResultSetData" node of the XML result set document.
      Node subjectRow = getRowNode(rowNumber, resultSet);
      if (subjectRow==null) {
         return null;
      }
      Node subjectColumn = null;
      if (columnNumber == -1) {
         subjectColumn = getColumnNode(subjectRow, -1, columnName);        
      } else {
         subjectColumn = getColumnNode(subjectRow, columnNumber, null);
      }
      if (subjectColumn==null) {
         return null;
      }
      return xmlColumn(subjectColumn);          

   }

   /** 
    *  Updates the value of row number i column columnName, 
    *  and returns the updated ResultSetXml document. 
    *  @param rowNumber  an int that is the number of a row of the result set
    *  @param columnName  a string that is the name of a column of the result set
    *  @param  newValue  a string that is the new value of the given column
    *  @return   a string that is the value of the updated document
    *  @throws     Exception if the specified row and column don't exist.
    */
   public void setColumn(int rowNumber, String columnName, String newValue) 
			throws Exception {
      this.setColumn(rowNumber, -1, columnName, newValue);
   }

   /** 
    *  Update the value of row number i column number j
    *  and return the updated ResultSetXml document. 
    *  @param rowNumber  an int that is the number of a row of the result set
    *  @param columnNumber  a string that is the number of a column of the result set
    *  @param  newValue  a string that is the new value of the given column
    *  @return   a string that is the value of the updated document
    *  @throws   Exception if the specified row and column don't exist.
    */
   public void setColumn(int rowNumber, int columnNumber, 
                                 String newValue) throws Exception {
      this.setColumn(rowNumber, columnNumber, null, newValue);
   }

   /** 
    *  Updates either column number j or column named 'columnName'.
    *  This method is the common code for the public <code>setColumn</code>
    *  methods. 
    *  @param rowNumber  an int that is the number of a row of the result set
    *  @param columnNumber  a string that is the number of a column of the result set
    *  @param columnName  a string that is the name of a column of the result set
    *  @param  newValue  a string that is the new value of the given column
    *  @return   a string that is the value of the updated document
    *  @throws   Exception if the specified row and column don't exist.
    */ 
   private void setColumn(int rowNumber, int columnNumber, String columnName, String newValue) 
		throws Exception {

      //  Parse the xmlText and step to the root
      Document doc = DomTree.checker(this.parse(), "ResultSet");
      Node resultSet = doc.getDocumentElement();

      //  Find the Node for the row rowNumber.
      Node subjectRow = getRowNode(rowNumber, resultSet);
      Node subjectColumn = null;
      if (columnNumber== -1) {
         //  Find the Node for column named 'columnName' in the i-th row
         subjectColumn = getColumnNode(subjectRow, -1, columnName);        
      } else {
         //  Find the Node for column number 'columnNumber' in the i-th row
         subjectColumn = getColumnNode(subjectRow, columnNumber, null);              
      }   
      //      
      Node textNode = DomTree.getTextNode(subjectColumn);
      if (textNode==null) {
         throw new Exception("ResultSet exception - Specified row and column not found");
      } else if (textNode.getNodeName().equals("#text")) {
          subjectColumn.replaceChild(doc.createTextNode(newValue), textNode);
      }  else if (textNode.getNodeName().equals("#cdata-section")) {
          subjectColumn.replaceChild(doc.createCDATASection(newValue), textNode);
      }
      this.xmlText = jcs.xml.DomTree.doc2String(doc);
   }

   /**  
    *  Returns the Node for the i-th row of the result set, or null
    *  if that row doesn't exist.
    *  @param i  the index of a row of the result set represented 
    *            by the <code>this.toString</code>
    *  @param resultSet    the DOM node of a <code>ResultSet</code>
    *  @return  a Node that is the specified row   
    */  

   private static Node getRowNode(int i, Node resultSet) {
      //  Position on the ResultSetData node
      Node resultSetData = DomTree.findNode(resultSet, "ResultSetData");
      if (resultSetData==null || !resultSetData.hasChildNodes()) {
         return null;
      }
      NodeList nl = resultSetData.getChildNodes();
      // Get the child nodes of the ResultSetData node.      
      int rowNumber=0;
      int nextNode = DomTree.nextGivenChild(nl, 0, "Row");
      while (nextNode !=-1) {
         if (rowNumber == i) {
            return nl.item(nextNode);
         } else {
            nextNode=DomTree.nextGivenChild(nl, nextNode+1, "Row");
         }  
         rowNumber++;
      }
      return null;
   }


   /**  
    *  Returns the Node for a given column of a given row.
    *  @param  subjectRow  a Node that is the row from which to locate the column
    *  @param  columnNumber  the number of a column of the row
    *  @param  columnName    a string that is the name of a column
    *  @return a Node that is the specified column of the specified row,
    *          or null of the column or row doesn't exist.
    */  

   private static Node getColumnNode(Node subjectRow, int columnNumber, 
                                       String columnName) {
      if (subjectRow==null) {
         return null;
      }
      NodeList nl = subjectRow.getChildNodes();
      String thisColumnName = "";
      int thisColumnNumber =0;
      int nextNode = DomTree.nextGivenChild(nl, 0, "Column");
      while (nextNode !=-1) {
         thisColumnName = DomTree.getAttribute(nl.item(nextNode), "name");
         if (thisColumnName.equals(columnName) || columnNumber == thisColumnNumber) {
            return nl.item(nextNode);
         } else {
            nextNode=DomTree.nextGivenChild(nl, nextNode+1, "Column");
         }  
         thisColumnNumber++;
      }         
      return null;
   }

   /**  
    *  
    *  Returns the string value of a column, or null
    *  @param  Column  a DOM node that is a column of the result set
    *  @return   a string that is the value of that column, or is null
    */  
   private static String xmlColumn(Node Column)  {  
      String columnNull = DomTree.getAttribute(Column, "null");
      String columnValue = DomTree.getText(Column);
      if (columnNull.equals("true")) { 
         return null;
      } else {
         return columnValue;      
      }
   }
   

   /**  
    *  Constructs a ResultSetVector from a ResultSetXml document.
    *  @param  xml  a string containing an XML document that is an SQL
    *               result set.  It must conform to the JavaSqlResultSet DTD.
    *  @return a ResultSetVector containing a Vector representation of
    *          the SQL result set.
    *  @throws Exception   To be added
    */  

   public ResultSetVector toResultSetVector () throws Exception {

      Node root = DomTree.topNode(this.parse(), "ResultSet");

      // Position on the "ResultSetData" node of the XML result set document.
      Node resultSetData = DomTree.findNode(root, "ResultSetData");

      // Process the ResultSetData.
      // The 'xmlResultSetData' method will return a vector of row vectors.
      return new ResultSetVector(xmlResultSetData(resultSetData));
   }

   /**  
    *  Processes the XML <code>ResultSetData</code> node and returns 
    *  a Vector of row Vectors.
    *  @param  resultSetData  a DOM Node that is the <code>ResultSetData</code>
    *                         node of a JavaSqlResultSet XML document
    *  @return  a Vector containing the String representations of the 
    *           rows and columns of the SQL result set
    *  @throws  Exception    To be added
    */  

   private static Vector xmlResultSetData (Node resultSetData) 
             throws Exception {

      // Get the child nodes of the ResultSetData node.      
      NodeList nl = resultSetData.getChildNodes();
      Vector rowsVector = new Vector(10,10);

      // Call the 'doRow' method for the child nodes that are Row nodes.
      // I.e. skip any "#TEXT" nodes.

      int nextNode = DomTree.nextGivenChild(nl, 0, "Row");
      while (nextNode !=-1) {
         Vector rowVector = xmlRow(nl.item(nextNode));
         rowsVector.addElement(rowVector);
         nextNode=DomTree.nextGivenChild(nl, nextNode+1, "Row");
      }
      return rowsVector;
   }

   /**  
    *  Processes a Row node and returns a Vector of column values.
    *  @param  Row   a DOM Node that is a <code>Row</code>
    *                node of a JavaSqlResultSet XML document
    *  @return  a Vector containing the String representations of the row
    */  

   private static Vector xmlRow (Node Row) {

      //  Get the child nodes of the Row node.

      NodeList nl = Row.getChildNodes();
      int size = nl.getLength();
      int columnNumber=0;

      //  Step through the child nodes of the Row, 
      //  and call 'doColumn' for each child that is a Column node.
      //  I.e. skip any #TEXT nodes.
      //  Use the variable "columnNumber" for the true column number,
      //  since the variable 'i' will include #TEXT nodes.
      //  The column number is needed when looking up the datatype of a column.
      Vector rowVector = new Vector();

      int nextNode = DomTree.nextGivenChild(nl, 0, "Column");
      while (nextNode !=-1) {
         columnNumber++; 
         rowVector.addElement(xmlColumn(nl.item(nextNode)));            
         nextNode=DomTree.nextGivenChild(nl, nextNode+1, "Column");
      }
      return rowVector;

   }
   

   /**  
    *  Returns the universal quantification of a given comparison
    *  to a given column of the <code>ResultSetXml</code>.
    *  <p>Example Java code:
    *
    *  <pre>rsx.allString(i, "<=", "xxx")</pre>
    *
    *  returns true iff for all elements ci of column i of rsx, 
    *  <pre>ci <= "xxx"</pre>
    *
    *  The <code>allString</code> method performs only String comparisons.
    *  @param columnNumber   the number of a column of the result set
    *  @param  compOp   a string that is one of "=,==,<,>,<>,!=, >=,<=".
    *  @param  comparand  a string that will be the second operand of the compares.
    *  @return  a boolean indicating whether the universal quantification is true
    *  @throws Exception    To be added 
    */  


   /**  
    *  Returns the universal quantification of a given comparison
    *  to a given column of the <code>ResultSetVector</code>.
    *  <p>Example Java code:
    *
    *  <pre>rsv.allString(i, "<=", "xxx")</pre>
    *
    *  returns true iff for all elements ci of column i of rsv, 
    *  <pre>ci <= "xxx"</pre>
    *
    *  The <code>allString</code> method performs only String comparisons.
    *  @param columnNumber   the number of a column of the result set
    *  @param  compOp   a string that is one of "=,==,<,>,<>,!=, >=,<=".
    *  @param  comparand  a string that will be the second operand of the compares.
    *  @return  a boolean indicating whether the universal quantification is true
    *  @throws Exception    Thrown if the document is invalid.
    */  
   
   public Boolean allString(int columnNumber, String compOp, String comparand)  
                           throws Exception {

      ResultSetVector rsv = this.toResultSetVector();
      return rsv.allString(columnNumber, compOp, comparand);
   }


   /**  
    *  Returns the existential quantification of a given comparison
    *  to a given column of the <code>ResultSetXml</code>.
    *  <p>Example Java code:
    *
    *  <pre>rsx.someString(i, "<=", "xxx")</pre>
    *
    *  returns true iff for some element ci of column i of rsx, 
    *  <pre>ci <= "xxx"</pre>
    *
    *  The <code>someString</code> method performs only String comparisons.
    *  @param columnNumber   the number of a column of the result set
    *  @param  compOp   a string that is one of "=,==,<,>,<>,!=, >=,<=".
    *  @param  comparand  a string that will be the second operand of the compares.
    *  @return  a boolean indicating whether the existential quantification is true
    *  @throws Exception    Thrown if the document is invalid.
    */  
   
   public Boolean someString(int columnNumber, String compOp, String comparand)  
                           throws Exception {

      ResultSetVector rsv = this.toResultSetVector();
      return rsv.someString(columnNumber, compOp, comparand);
   }

   /**  
    *  The static final name of the ResultSet DTD.  
    *  This is the name that is used in the SYSTEM clause 
    *  of external entity references to the ResultSet DTD.  
    *  For example:
    *  <pre>
    *  &lt;!DOCTYPE ResultSet SYSTEM "ResultSet.dtd"&gt;
    *  </pre>
    */  
   public static final String DTD_NAME = "resultset.dtd";
   /**  
    *  A line-feed, for pretty print formatting of the DTD.
    */  
   private static final String LINE_FEED = "\n";
   /**  
    *  A double line-feed, for pretty print formatting of the DTD.
    */  
   private static final String LINE_FEED2 = "\n\n";
   /**  
    *  Indentation levels, for pretty print formatting of the DTD.
    */  
   private static final String[ ] INDENT 
                             = {"\n", "\n   ", "\n      ", "\n         "} ;

   /**  
    *  The static final text of the DTD for SQL ResultSet documents.
    */  

   public static final String DTD =  
       "<!ELEMENT ResultSet (ResultSetMetaData , ResultSetData)> " + LINE_FEED
      + "<!-- " 
      + INDENT[1] + "A result set is a description of the Columns (ResultSetMetaData) "
      + INDENT[1] + "followed by 0 or more Rows (ResultSetData). " 
      + INDENT[1] + "Each Row contains the same number of Columns as were specified "
                  + "in the ResultSetMetaData."
      + INDENT[1] + "-->" + LINE_FEED2
      + "<!ELEMENT ResultSetMetaData (ColumnMetaData)+> " + LINE_FEED 
      + INDENT[1] + "<!-- "
      + INDENT[1] + "ResultSetMetaData describes each column of the result set. "
      + INDENT[1] + "There must be at least 1 column. " 
      + INDENT[1] + "The number of columns may be provided as the getColumnCount "
                  + "attribute (an integer), or can be calculated "
      + INDENT[1] + "by iterating over the ColumnMetaData elements." 
      + INDENT[1] + "-->" + LINE_FEED2

      + "<!ATTLIST ResultSetMetaData getColumnCount CDATA #IMPLIED> " + LINE_FEED 
      + INDENT[1] + "<!-- "
      + INDENT[1] + "Each column in the result set must have getColumnType  "
                   + "(a constant from java.sql.Types). "
      + INDENT[1] + "Other attributes are optional. " 
      + INDENT[1] + "-->" + LINE_FEED2
      + "<!ELEMENT ColumnMetaData  EMPTY> " + LINE_FEED
      + "<!ATTLIST ColumnMetaData " 
      + INDENT[1] + "getCatalogName CDATA #IMPLIED " 
      + INDENT[1] + "getColumnDisplaySize CDATA #IMPLIED " 
      + INDENT[1] + "getColumnLabel CDATA #IMPLIED " 
      + INDENT[1] + "getColumnName CDATA #IMPLIED" 
      + INDENT[1] + "getColumnType CDATA #REQUIRED " 
      + INDENT[1] + "getColumnTypeName CDATA #IMPLIED " 
      + INDENT[1] + "getPrecision CDATA #IMPLIED " 
      + INDENT[1] + "getScale CDATA #IMPLIED " 
      + INDENT[1] + "getSchemaName CDATA #IMPLIED "
      + INDENT[1] + "getTablename CDATA #IMPLIED " 
      + INDENT[1] + "isAutoIncrement (true|false) #IMPLIED " 
      + INDENT[1] + "isCaseSensitive (true|false) #IMPLIED " 
      + INDENT[1] + "isCurrency (true|false) #IMPLIED " 
      + INDENT[1] + "isDefinitelyWritable (true|false) #IMPLIED " 
      + INDENT[1] + "isNullable (true|false) #IMPLIED " 
      + INDENT[1] + "isReadOnly (true|false) #IMPLIED " 
      + INDENT[1] + "isSearchable (true|false) #IMPLIED " 
      + INDENT[1] + "isSigned (true|false) #IMPLIED " 
      + INDENT[1] + "isWritable (true|false) #IMPLIED" 
      + INDENT[1] + "> " + LINE_FEED 
      + INDENT[1] + "<!--  " 
      + INDENT[1] + "The number of columns should be the same in each row,  " 
      + INDENT[1] + "and should be the same as the number of columns indicated " 
                + "in the ResultSetMetaData. " 
      + INDENT[1] + "There must be at least 1 column.  " 
      + INDENT[1] + "-->  " + LINE_FEED2
      + "<!ELEMENT ResultSetData (Row)*> " + LINE_FEED
      + "<!ELEMENT Row (Column)+> " + LINE_FEED
      + "<!ELEMENT Column (#PCDATA)> " + LINE_FEED 
      + INDENT[1] + "<!--  " 
      + INDENT[1] + "If a column's value is null, "
                + "the 'null' attribute should be set to true. "
      + INDENT[1] + "In this case, any value specified is ignored. "
      + INDENT[1] + "Other column attributes might be appropriate: " 
                + "truncated, overflow, underflow, etc. " 
      + INDENT[1] + "--> " + LINE_FEED2
      + "<!ATTLIST Column "
      + INDENT[1] + "null (true | false) \"false\"  " 
      + INDENT[1] + "name CDATA #IMPLIED "
      + INDENT[1] + ">" +LINE_FEED2 ;


      
}

