//
//  Copyright (c) 1999, 2001
//  Sybase, Inc
//  Emeryville, CA 94608
//  All Rights Reserved
//
package jcs.xml.resultset;
import jcs.util.StringUtil;
import jcs.xml.*;

import org.w3c.dom.*;

import java.io.*;
import java.sql.*;
import java.util.*;

/**
 *  A class whose static method converts an XML document that 
 *  contains an SQL result set into an SQL script that has a 
 *  <code>create</code> statement and a series of <code>insert</code>  
 *  statements that re-create the SQL result set  in an SQL table.
 *  This static method is the body of the <code>toSqlScript</code>
 *  method of the <code>ResultSetXml</code> class.
 */
public class  XR implements java.io.Serializable  {

   /**  
    *  The XR class contains only static methods, and cannot be instantiated.
    */     
   private XR() {}  


   /**  
    *  Returns an SQL script with a <code>create</code> statement
    *  and a list of <code>insert</code> statements that re-create
    *  the SQL data contained in the XML document.
    *  This static method is the body of the <code>toSqlScript</code>
    *  method of the <code>ResultSetXml</code> class.
    *
    *  <p>The <code>goOption</code> parameter indicates whether you want
    *  the output script to include <code>go</code> commands after
    *  the <code>create</code> and <code>insert</code> statements.
    *  This depends on how you plan to execute the script:  ISQL requires
    *  the <code>go</code> commands, and JDBC doesn't allow them. 
    *  @param xml  a string that is the input XML document.  This is
    *              the actual document, not a name or reference.
    *              The document must be a valid representation of an
    *              SQL result set, using the JavaSqlResultSet DTD.
    *  @param resultTableName   a string that is the SQL table name to
    *                           specify in the output <code>create</code> and 
    *                           <code>insert</code>  statements
    *  @param columnPrefix  a string to use as a prefix when generating
    *                       column names in the output script.  Such generated
    *                       names are needed when the result set has columns
    *                       with no names or with duplicate names.
    *  @param goOption    a string indicating whether the output script should
    *               include <code>go</code> commands. This is indicated by
    *               <code>yes</code> or <code>no</code>.  See above.
    *  @throws  Exception   Thrown for unrecognized nodes.
    */  

   public static String toSqlScript(Document doc, 
                    String resultTableName, String columnPrefix, String goOption) 
				throws Exception {


      String goVar
        = StringUtil.validateArg(goOption, "yes,no", 
                    "Invalid 'goOption' arg ('yes' or 'no'): ");

      // The Properties variable "ct" is a dictionary for column types:  e.g. 
      //      (("name", "char"),  ("salary", "decimal"), ("age", "integer")).
      // We also use 'ct' for the "resultTableName" and "columnPrefix" 
      // parameters, so that they don't have to be passed through 
      // to low level routines as separate parameters.
      // I.e. the "ct" dictionary contains all info that would normally be
      // placed in static variables.  We use the "ct" dictionary instead 
      // of static variables in order to make the class more suitable 
      // for use in the server.

      Properties ct = new Properties();
      ct.put("resultTableName", resultTableName);
      ct.put("columnPrefix", columnPrefix);
      ct.put("goOption", (goVar.equals("yes"))?"go":"");

      Element root = (Element)DomTree.topNode(doc, "ResultSet");
      // Call 'doResultSet' to translate the parsed XML to an SQL script.
      String script = doResultSet(root, ct, 0) + indent_[0];    
      
      return script;
   }

   /**  
    *  Processes the parsed XML result set and returns an equivalent SQL script.
    *  @param node  the DOM node that is the root of the XML document
    *  @param ct    a Properties that has the parameters of 
    *               <code>XR.resultSet2Sql</code>.  Such a Properties object 
    *               is a convenient way to pass assorted option parameters to
    *               a series of subroutines.
    *  @param indent  an int with the current level of indentation.  This value
    *                 is used to pretty-print the format of the output script.
    *  @return a string with the script of <code>create</code> and <code>insert</code>
    *          statements
    *  @throws Exception  Thrown for unrecognized nodes.
    */  

   private static String doResultSet(Node node,  Properties ct, int indent) 
              throws Exception {

      //  The variable 'moreScript' is a work variable to accumulate the SQL script.

      if (node==null) return "";
      if (node.getNodeType()==Node.TEXT_NODE 
         && StringUtil.isNothing(node.getNodeValue()))  {
            return "" ;
      }
      String gNN = node.getNodeName();
      if (!gNN.equals("ResultSet")) {
         throw  new Exception ("Unexpected ("+gNN+")");
      }

      // Position on the "ResultSetMetaData" node of the XML result set document.
      Node resultSetMetaData = DomTree.findNode(node, "ResultSetMetaData");
      String createScript = "";
      if (resultSetMetaData!=null) {

         // Process the ResultSetMetaData.
         // The 'doResultSetMetaData' method will return the CREATE TABLE statement.
         createScript = doResultSetMetaData(resultSetMetaData, ct, 0) ;
      } else {
         throw new Exception("The 'resultSet2Sql' method requires <ResultSetMetaData>"
                +" in order to know which columns are strings");
      }
      // Position on the "ResultSetData" node of the XML result set document.
      Node resultSetData = DomTree.findNode(node, "ResultSetData");
      String insertScript="";
      if (resultSetData != null) {
         // Process the ResultSetData.
         // The 'doResultSetData' method will return a sequence of INSERT statements.
         insertScript = doResultSetData(resultSetData, ct, 0);
      }

      return createScript + lineFeed2_ + insertScript;

   }   
   

   /**  
    *  Processes the "ResultSetMetaData" node and generates the <code>create</code>
    *  statement.
    *  @param node  the DOM node that is the root of the XML document
    *  @param ct    a Properties that has the parameters of 
    *               <code>XR.resultSet2Sql</code>.  Such a Properties object 
    *               is a convenient way to pass assorted option parameters to
    *               a series of subroutines.
    *  @param indent  an int with the current level of indentation.  This value
    *                 is used to pretty-print the format of the output script.
    *  @return      a string with the <code>create</code> statement
    *  @throws Exception  Thrown for unrecognized nodes.
    */  

   private static String doResultSetMetaData
                 (Node resultSetMetaData, Properties ct, int indent) 
           throws Exception {

      // Obtain the table-name for the resulting SQL table from the properties.
      String resultTableName = ct.getProperty("resultTableName");

      // Generate the CREATE TABLE portion of the create-table.
      //  The initial 'set quoted_identifier on' command enables
      //  SQL quoted identifiers, such as "rate of pay".
      String createTable  = indent_[indent] 
                          + "set quoted_identifier on \n"
                          + "create table " + resultTableName + " ( ";

      //  Extract the "columnCount" value.  
      //  We don't use that value in this translation.
      String columnCount = DomTree.getAttribute(resultSetMetaData, "getColumnCount");

      //  Get the child nodes of the ResultSetMetaData node.
      NodeList nl = resultSetMetaData.getChildNodes();
      int columnNumber=0;

      //  Step through the child nodes of the RowMetaData, 
      //  and call 'doColumnMetaData' for each "ColumnMetaData" child
      //  that is a Column node.
      //  In particular, skip any #text nodes with whitespace.
      //  In fact, we'll skip any sort of child node except for "ColumnMetaData".
      //  Use the variable "columnNumber" for the true column number,
      //  since the variable 'i' will include #text whitespace nodes.
      //  The column number is needed when looking up the datatype of a column.


      int nextNode = DomTree.nextGivenChild(nl, 0, "ColumnMetaData");
      while (nextNode !=-1) {
         columnNumber++;     
         createTable
             += doColumnMetaData(nl.item(nextNode), ct, columnNumber, indent+1) ;
         nextNode=DomTree.nextGivenChild(nl, nextNode+1, "ColumnMetaData");
      }

      // Close the CREATE TABLE parentheses, 
      // and generate a Sybase "go" command after the CREATE TABLE statement.
      createTable +=   indent_[0] + ")" + goCommand(ct);
       
      return createTable ;
   }

   /**  
    *  Processes a ColumnMetaData node, and generates an SQL column declaration for it.
    *  @param columnMetaData  the DOM node that is the <code>columnMetadata</code> node
    *  @param ct    a Properties that has the parameters of 
    *               <code>XR.resultSet2Sql</code>.  Such a Properties object 
    *               is a convenient way to pass assorted option parameters to
    *               a series of subroutines.
    *  @param columnNumber  the current column number 
    *  @param indent  an int with the current level of indentation.  This value
    *                 is used to pretty-print the format of the output script.
    *  @return  a string that is a single column declaration
    *  @throws Exception  Thrown for unrecognized nodes.
    */  

   private static String doColumnMetaData
                 (Node columnMetaData , Properties ct, int columnNumber,  int indent) 
               throws Exception {

      //  The column name.
      String columnName = DomTree.getAttribute(columnMetaData, "getColumnName");
      columnName = processColumnName(columnName, columnNumber, ct);

      //  The column datatype.
      String columnTypeCode = DomTree.getAttribute(columnMetaData, "getColumnType");
      String columnType = lookUpTypeCode(columnTypeCode);

      // The precision, scale, displaysize (i.e. length), and nullable attributes.
      String precision = DomTree.getAttribute(columnMetaData, "getPrecision"); 
      String scale  = DomTree.getAttribute(columnMetaData, "getScale"); 
      String displaySize = DomTree.getAttribute(columnMetaData, "getColumnDisplaySize"); 
      String isNullable = DomTree.getAttribute(columnMetaData, "isNullable"); 

      // Assemble the column declaration in 'columnString'.
      // E.g. 'empno char(5)', or 'salary decimal (5,2)'

      // The column name.
      String columnString= comma(columnNumber)+indent_[indent]+ columnName + " " ;

      // The basic column datatype.
      columnString += columnType + " ";

      // The length, precision, scale, etc, attributes,
      // depending on the specific column datatype.
      if (isLengthType(columnType)) {
          columnString+="("+displaySize+") ";
      }
      
      if (columnType.equals("binary") || columnType.equals("varbinary")) {
          // For binary and varbinary, the 'displaysize' is twice the declared size
          int half = Integer.parseInt(displaySize)/2;
          columnString+="("+half+") ";
      }    
      if (isPrecScaleType(columnType)) {
          columnString += "(" + precision + ", " + scale + ") ";
      }
      if (isPrecType(columnType)) {
          columnString += "(" + precision + ") ";
      }

      // The "null" or "not null" attribute (don't rely on defaults).
      if (isNullable.equals("false")) {
         columnString += "not null ";
      } else {
         columnString += "null ";
      }

      //  Record the column number and column type in the 'ct' properties table.
      //  This will be needed when processing column values.
      //  E.g. if a column is CHAR or TEXT, etc, then the value must be quoted, etc.
      ct.put("type"+columnNumber, columnType);
      return columnString;     
   }

   /**  
    *  
    *  Processes a column name.    
    *  If a given column name is empty or is a duplicate, then replaces it
    *  with a generated column name.
    *  If the column name is not a valid SQL identifier, then returns it
    *  as a quoted identifier.
    *  Otherwise, returns the column name.
    *  @param columnName  the column name to be examined
    *  @param columnNumber  the current column number
    *  @param ct    a Properties that has the parameters of 
    *               <code>XR.resultSet2Sql</code>.  Such a Properties object 
    *               is a convenient way to pass assorted option parameters to
    *               a series of subroutines.
    *  @return a string with the effective column name
    *  @throws Exception  Thrown for unrecognized nodes.
    */  

   private static String processColumnName(String columnName, 
                               int columnNumber, Properties ct)
                        throws Exception {
      String columnPrefix = ct.getProperty("columnPrefix");

      String qt = "\"";
      boolean fold = true;
      if (columnName==null || columnName.length()==0 
            || columnName.length() > 30  ) {
         return saveAndReturn(columnPrefix+columnNumber, columnNumber, ct);
      } else if (isReservedId(columnName)&&columnName.length()<=26) {
         return  checkDuplicate(qt+columnName+qt, columnNumber, ct, fold);    
      } else if (isRegularIdentifier(columnName)) {
         return checkDuplicate(columnName,  columnNumber, ct, fold);
      } else {
         String work = doubleQuotes(columnName, qt);
         if (work.length()<26) {
            return checkDuplicate(qt+work+qt,  columnNumber, ct, !fold);
         } else { 
            return saveAndReturn(columnPrefix+columnNumber, columnNumber, ct);
         }
      }
   }

   /**  
    *  
    *  Checks whether a column name is a duplicate of a previous column name.
    *  If a given column name is a duplicate of the column name
    *  of a previous column, returns a generated name.
    *  Otherwise, returns the column name.
    *  @param columnName  the column name to be examined
    *  @param columnNumber  the current column number
    *  @param ct    a Properties that has the parameters of 
    *               <code>XR.resultSet2Sql</code>.  Such a Properties object 
    *               is a convenient way to pass assorted option parameters to
    *               a series of subroutines.
    *  @param fold  a boolead indicating whether the column name is a normal
    *               SQL column name (case insensitive) or a quoted column name
    *               (case sensitive), for use in checking duplicates
    *  @return      a string with the effective column name
    *  @throws Exception  Thrown for unrecognized nodes.
    *  
    */  

   private static String checkDuplicate(String columnName, 
                       int columnNumber, Properties ct, boolean fold)
                throws Exception {
      String columnPrefix = ct.getProperty("columnPrefix");
      for (int i=1; i<columnNumber; i++) {
          String ni = ct.getProperty("name"+i);
          if (ni == null) {
             throw new Exception("Internal error - null column name");
          } else if (fold && ni.toLowerCase().equals(columnName)) {
             return saveAndReturn(columnPrefix+columnNumber, columnNumber, ct);
          } else if (!fold && ni.toLowerCase().equals(columnName)) {
             return saveAndReturn(columnPrefix+columnNumber, columnNumber, ct);
          }
      }
      return saveAndReturn(columnName, columnNumber, ct);
   }

   /**  
    *  Records the column number and column name in the Properties table ct,
    *  after the final column name is finally determined (after checking
    *  for unnamed columns and duplicate columns).
    *  @param columnName  the column name to be examined
    *  @param columnNumber  the current column number
    *  @param ct    a Properties that has the parameters of 
    *               <code>XR.resultSet2Sql</code>.  Such a Properties object 
    *               is a convenient way to pass assorted option parameters to
    *               a series of subroutines.
    *  @return      a string with the column name
    */  
   private static String saveAndReturn(String columnName, int columnNumber,
                                 Properties ct) {
      ct.put("name"+columnNumber, columnName);
      return columnName;
   }

   /** 
    *  Tests for a datatype that requires quotes around the value.
    *  LONGVARCHAR is a JDBC type not supported by Sybase.
    *  TEXT is a Sybase type equivalent to LONGVARCHAR
    *  @param t  a string that is an SQL datatype
    *  @return  a boolean indication whether the type quotes
    */
   protected static boolean isQuotedType(String t) {
      if (t.equals("char") || t.equals("varchar") 
         || t.equals("longvarchar") || t.equals("text")) {

         return true;
      } else {
         return false;
      }
   }

   /**  
    *  Tests for a datatype that requires a length in the declaration.
    *  This is all of the char types except for TEXT.
    *  @param t  a string that is an SQL datatype
    *  @return  a boolean indication whether the type requires a length in the declaration
    */  
   private static boolean isLengthType(String t) {
      if (t.equals("char") || t.equals("varchar") 
         || t.equals("longvarchar") ) {

         return true;
      } else {
         return false;
      }
   }

   /**  
    *  Tests for a binary datatype, which requires a <code>0x</code> infront of the value.
    *  LONGVARBINARY is a JDBC type not supported by Sybase.
    *  IMAGE is a Sybase type equivalent to LONGVARBINARY
    *  @param t  a string that is an SQL datatype
    *  @return  a boolean indication whether the type requires <code>0x</code>
    */  

   private static boolean isBinaryType(String t) {
      if (t.equals("binary") || t.equals("varbinary") 
         || t.equals("longvarbinary") || t.equals("image")) {
         return true;
      } else {
         return false;
      }
   }


   /**  
    *  Tests for a datatype that requires the PRECISION attribute and not SCALE.
    *  @param t  a string that is an SQL datatype
    *  @return  a boolean indication whether the type requires PRECISION
    */  

   private static boolean isPrecType(String t) {
      if (t.equals("float")) {
         return true;
      } else {
         return false;
      }
   }

   /**  
    *  Tests for a datatype that requires both PRECISION and SCALE.
    *  @param t  a string that is an SQL datatype
    *  @return  a boolean indication whether the type requires PRECISION and SCALE
    */  

   private static boolean isPrecScaleType(String t) {
      if (t.equals("decimal") || t.equals("numeric") ){
         return true;
      } else {
         return false;
      }
   }

   /**  
    *  Tests for a DATE or TIME datatype.
    *  Note:  Sybase ASE supports only "datetime".  The others are JDBC types.
    *  @param t  a string that is an SQL datatype
    *  @return  a boolean indication whether the type is DATE or TIME
    */  
   private static boolean isDateType(String t) {
      if (t.equals("date") || t.equals("time") || t.equals("timestamp")
                      || t.equals("datetime")) {
         return true;
      } else {
         return false;
      }
   }

   /**  
    *  
    *  Returns the datatype keyword for a datatype code.
    *  @param t  a string that is a JDBC datatype code
    *  @return  string that is the corresponding SQL datatype name
    */  

   protected static String lookUpTypeCode(String tc) {
      if (tc.equals(String.valueOf(Types.BIT))) return "bit"; 
      if (tc.equals(String.valueOf(Types.TINYINT))) return "tinyint"; 
      if (tc.equals(String.valueOf(Types.SMALLINT))) return "smallint"; 
      if (tc.equals(String.valueOf(Types.INTEGER))) return "integer"; 
      if (tc.equals(String.valueOf(Types.BIGINT))) return "bigint"; 
      if (tc.equals(String.valueOf(Types.FLOAT))) return "float"; 
      if (tc.equals(String.valueOf(Types.REAL))) return "real"; 
      if (tc.equals(String.valueOf(Types.DOUBLE))) return "double precision"; 
      if (tc.equals(String.valueOf(Types.NUMERIC))) return "numeric"; 
      if (tc.equals(String.valueOf(Types.DECIMAL))) return "decimal"; 
      if (tc.equals(String.valueOf(Types.CHAR))) return "char"; 
      if (tc.equals(String.valueOf(Types.VARCHAR))) return "varchar"; 
      if (tc.equals(String.valueOf(Types.LONGVARCHAR))) return "text"; 
         // Note:  JDBC LONGVARCHAR is equivalenced to Sybase TEXT
      if (tc.equals(String.valueOf(Types.DATE ))) return "date"; 
      if (tc.equals(String.valueOf(Types.TIME ))) return "time"; 
      if (tc.equals(String.valueOf(Types.TIMESTAMP ))) return "datetime"; 
        //  Note:  JDBC TIMESTAMP is equivalenced to Sybase DATETIME
      if (tc.equals(String.valueOf(Types.BINARY ))) return "binary"; 
      if (tc.equals(String.valueOf(Types.VARBINARY ))) return "varbinary"; 
      if (tc.equals(String.valueOf(Types.LONGVARBINARY ))) return "image"; 
        //  Note:  JDBC LONGVARBINARY is equivalenced to Sybase IMAGE
      return "unknown("+tc+")";
   }   

   /**  
    *      
    * Processes the ResultSetData node and returns a sequence of <code>insert</code> 
    * statements to store the result set data into an SQL table.
    *  @param resultSetData  the DOM node that is the resultSetData node
    *  @param ct    a Properties that has the parameters of 
    *               <code>XR.resultSet2Sql</code>.  Such a Properties object 
    *               is a convenient way to pass assorted option parameters to
    *               a series of subroutines.
    *  @param indent  an int with the current level of indentation.  This value
    *                 is used to pretty-print the format of the output script.
    *  @return      a string with the <code>insert</code> statements
    *  @throws Exception  Thrown for unrecognized nodes.
    */  

   private static String doResultSetData
                 (Node resultSetData,  Properties ct, int indent) 
             throws Exception {

      // Get the child nodes of the ResultSetData node.      
      NodeList nl = resultSetData.getChildNodes();
      String inserts ="";

      int nextNode = DomTree.nextGivenChild(nl, 0, "Row");
      while (nextNode !=-1) {
         inserts
             +=  doRow(nl.item(nextNode), ct, indent) ;
         nextNode=DomTree.nextGivenChild(nl, nextNode+1, "Row");
      }

      // Generate a Sybase "go" command after the list of INSERT statements.
      inserts +=  goCommand(ct) ;

      return inserts;
   }

   /**  
    *  
    *  Processes a Row node and returns an SQL <code>insert</code> statement
    *  for the row.
    *  @param Row   a Row node
    *  @param ct    a Properties that has the parameters of 
    *               <code>XR.resultSet2Sql</code>.  Such a Properties object 
    *               is a convenient way to pass assorted option parameters to
    *               a series of subroutines.
    *  @param indent  an int with the current level of indentation.  This value
    *                 is used to pretty-print the format of the output script.
    *  @return      a string with a single <code>insert</code> statement
    *  @throws Exception  Thrown for unrecognized nodes.
    */  

   private static String doRow
                 (Node Row,  Properties ct,  int indent) throws Exception {

      //  Retrieve the result table name from the 'ct' properties table.
      //  This table name will be specified in the generated INSERT statements.
      String resultTableName = ct.getProperty("resultTableName");

      //  Variable "insertColumns" is for the INSERT statement.
      //  We won't actually initialize it until we get the first column,
      //  to deal with the case of an empty row <Row> in the XML doc.
      String insertColumns =  "";

      //  Get the child nodes of the Row node.

      NodeList nl = Row.getChildNodes();
      int columnNumber=0;

      //  Step through the child nodes of the Row, 
      //  and call 'doColumn' for each child that is a Column node.
      //  I.e. skip any #text nodes with whitespace.
      //  In fact, we'll skip any child of <Row> that isn't a <Column>.
      //  Use the variable "columnNumber" for the true column number,
      //  since the variable 'i' will include #text nodes.
      //  The column number is needed when looking up the datatype of a column.

      int nextNode = DomTree.nextGivenChild(nl, 0, "Column");
      while (nextNode !=-1) {
         columnNumber++; 
         if (insertColumns=="") {
            insertColumns = indent_[0] + "insert into " 
                              + resultTableName +  " values (";
         }  
         insertColumns += comma(columnNumber) + indent_[indent+1] 
                      +doColumn(nl.item(nextNode),columnNumber, ct, indent+1);
         nextNode=DomTree.nextGivenChild(nl, nextNode+1, "Column");
      }

      if (insertColumns!="")  {
         insertColumns += indent_[indent+1] + ")";
      }
      return insertColumns;

   }

   /**  
    *  
    *  Returns an <code>insert-list</code> entry for a column value.
    *  @param column  a Column node
    *  @param ct    a Properties that has the parameters of 
    *               <code>XR.resultSet2Sql</code>.  Such a Properties object 
    *               is a convenient way to pass assorted option parameters to
    *               a series of subroutines.
    *  @param indent  an int with the current level of indentation.  This value
    *                 is used to pretty-print the format of the output script.
    *  @return      a string with a single insert-list entry
    *  @throws Exception  Thrown for unrecognized nodes.
    *  
    */  

   private static String doColumn(Node column, int columnNumber,  
                                Properties ct, int indent) throws Exception {
      String columnNull = DomTree.getAttribute(column, "null");
      String columnValue="";
      //  Step through the child nodes of the Column.
      //  These should all be "#text" nodes, 
      //  with at most one whose value is non-whitespace.
      //  We will use the first non-whitespace (if any) and ignore any others.
      NodeList nl = column.getChildNodes();
      int valueNode = DomTree.nextNonWhiteSpace(nl, 0);
      if (valueNode != -1) {
         columnValue = nl.item(valueNode).getNodeValue();
      }

      String insertColumn = handleType(columnNumber, columnValue, columnNull, ct);
      return insertColumn;      
   }


   /**  
    *  Determines what special treatment (if any) a given column value 
    *  needs to make it an INSERT list entry.
    *  If the value is null, then the INSERT list entry is just "null".
    *  If the value is a string, then it must be quoted, 
    *  and any contained quotes must be doubled.
    *  If the value is binary, then it needs the "0x" prefix.
    *  @param columnNumber  an int that is the number of this column
    *  @param columnValue   a string that is a candidate to be placed in
    *                       the insert list.
    *  @param columnNull    an indication whether the column value is null
    *  @param ct    a Properties that has the parameters of 
    *               <code>XR.resultSet2Sql</code>.  Such a Properties object 
    *               is a convenient way to pass assorted option parameters to
    *               a series of subroutines.
    *  @return    the string to be placed in the insert list
    */  
   private static String handleType(int columnNumber,
                           String columnValue, String columnNull, 
                                                     Properties ct) 
	throws Exception {
      String work;
      String t = ct.getProperty("type"+columnNumber);
      if (columnNull.equals("true")) {
         return "null"; 
      } else if (isQuotedType(t) || isDateType(t)) {
         return "'"+doubleQuotes(columnValue, "'")+"'";
      } else if (isBinaryType(t)) {
         return "0x"+columnValue;
      }  else {
         return columnValue;
      }
 
   }

   /**  
    *  Returns a comma for arguments 2 and above.  For list separators.
    *  @param i  the index of this column.
    *  @return  an empty string for argument value 1, otherwise a comma
    */  

   private static String comma (int i) {
      return (i==1)?"": ", ";
   }

   /**  
    *  Doubles the quotes in a string.   For example, 
    *  <pre><code>doubleQuotes('AB"CD"EF', '"')</code></pre>
    *  returns 'AB""CD""EF'.
    *  The second parameter is the quote-mark you want to double.
    *  I.e. a true quote-mark or an apostrophes.
    *  @param val  a string whose contained quotes are to be doubled
    *  @param quote   the particular quote mark to consider
    *  @return  the quote-doubled string
    */  

   private static String doubleQuotes (String val,  String quote) {

     if (val == null || val.length()==0 ) return val;

     String work ="";

     String doubleQuote = quote+quote;
     int i=0;
     while (i<=val.length() && val.indexOf(quote, i)!=-1) {
        int j = val.indexOf(quote, i);
        work += val.substring(i, j);
        work += doubleQuote;
        i=j+1;
     }
     work += val.substring(i);
     return work;
     }

   /**  
    *  Tests whether an identifier follows the SQL rules for regular identifier.
    *  @param id   a string that is the identifier
    *  @return  an indication whether it is an SQL regular identifier  
    */  

   private static boolean isRegularIdentifier(String id) {

       if (id==null || id.length()==0 || id.length() > 30 
                                    || isReservedId(id)) return false;      
       char[] ca = id.toCharArray();
       if (!Character.isLetter(ca[0])) return false;
       for (int i=0; i<ca.length; i++)  {

         if (!Character.isLetterOrDigit(ca[i]) 
              && "_@$#".indexOf(id.substring(i, i+1)) == -1) return false;
       }
       return true;
   }

   /**  
    *  
    * Tests whether an identifier is an SQL reserved word. 
    * @param  id  a string that it the identifier to be considered
    * @return  an indication whether it is an SQL reserved word 
    */  

   private static boolean isReservedId(String id) {

      String[] reserved 
         = {"add", "all", "alter", "and", "any", "arith_overflow", "as",
            "asc", "at", "authorization", "avg", 
            "begin", "between", "break", "browse", "bulk", "by", 
            "cascade", "char_convert", "check", "checkpoint", "close", "clustered", 
            "commit", "compute", "confirm", "constraint", "continue", "controlrow", 
            "convert",  "count", "create", "current", "cursor",
            "data_pgs", "database", "dbcc", "deallocate", "declare", "default", 
            "delete", "desc", "disk", "distinct", "double", "drop", "dummy", "dump",
            "else", "end", "endtran", "errlvl", "errorexit", "escape", "except",
            "exec", "execute", "exists", "exit", 
            "fetch", "fillfactor", "for", "foreign", "from", 
            "goto", "grant", "group", 
            "having", "holdlock", 
            "identity", "identify_insert", "if", "in", "index", "insert", "intersect",
            "into", "is", "isolation", "key", "kill",        
            "level", "like", "lineno", "load", 
            "max", "min", "mirror", "mirrorexit", 
            "national", "noholdlock", "nonclustered", "not", "null", "numeric_truncation", 
            "of", "off", "offsets", "on", "once", "only", "open", "option", 
            "or", "order", "over", 
            "perm", "permanent", "plan", "precision", "prepare", "primary", 
            "print", "privileges", "proc", "procedure", "processexit", "public",
            "raiserror", "read", "readtext", "reconfigure", "references", "replace",
            "reserved_pgs", "return", "revoke", "role", "rollback", "rowcnt", "rowcount",
            "rows", "rule",
            "save", "schema", "select", "set", "setuser", "shared", "shutdown",
            "some", "statistics", "stripe", "sum", "syb_identity", "syb_restree",
            "table", "temp", "temporary", "textsize", "to", "tran", "transaction",
            "trigger", "truncate", "tsequal", "union", "unique", "update", 
            "used_pgs", "user", "user_option", "using", "values", "varying", 
            "view", "waitfor", "where", "while", "with", "work", "writetext"};

       for (int i=0; i<reserved.length; i++) {
           if (id.toLowerCase().equals(reserved[i])) return true;
       }

       return false;
   }

   /**  
    *  Returns a Sybase "go" command for insertion into the ourput script.
    *  @param ct    a Properties that has the parameters of 
    *               <code>XR.resultSet2Sql</code>.  Such a Properties object 
    *               is a convenient way to pass assorted option parameters to
    *               a series of subroutines.
    *  @return  a string that is either a <code>go</code> command or empty
    *  
    */  

   private static String goCommand(Properties ct) {
       String goOption = ct.getProperty("goOption");
       return lineFeed2_ + goOption + lineFeed2_;
   }

   /**  
    *  A double line-feed for pretty-print formatting of the output script.
    */  
   static final String lineFeed2_ = "\n\n";
   /**  
    *  An array of indentation levels for pretty-print formatting of the output script.
    */  
   static final String[ ] indent_ = {"\n", "\n   ", "\n      ", "\n         "} ;

}
//  ------------------------------------------------------------------------------


