//
//  Copyright (c) 1999, 2001
//  Sybase, Inc
//  Emeryville, CA 94608
//  All Rights Reserved
//
import java.sql.*;             // JDBC
/**  
 *  An example class illustrating the use of JDBC facilities 
 *  with the Java-in-SQL feature of ASE 12.0.
 *  <p>The methods of this class perform a range of SQL operations.
 *  These methods can be invoked either from a Java client,
 *  using the "main" method, or from the SQL server, using
 *  the "internalMain" method.
 */  

public class JDBCExamples {

   /**  
    *  The JDBCExamples class contains only static methods, and cannot be instantiated.
    */     
   private JDBCExamples() {}  

   /**  
    *  Invokes the other methods, based on the 2nd command line
    *  parameter.
    *  <p>Usage:
    *  <pre>
    *  java JDBCExamples server-name:port-number action
    *  </pre>
    *  <ul>  
    *  <li>The first parameter is the server-name, port-number,
    *  user-name, and password.  
    *  For example 
    *  <pre>
    *  <code>"antibes:4000?user=smith&password=xxx"</code>.
    *  <code>"antibes:4000?user=sa"</code>.
    *  </pre>
    *  <p>In the second example above, the <code>password</code>
    *  defaults to the empty string.
    *  <li>The second parameter is a keyword indicating the <i>action</i>
    *  to be performed.  This parameter is one of the following:
    *  <pre>  
    *     connect, createtable, createproc, drop, 
    *     insert, select, update, call
    *  </pre>
    *  <p>Examples of command-line client invocation:
    *  <pre>
    *  java JDBCExamples  "antibes:4000?user=sa" insert
    *  java JDBCExamples  "antibes:4000?user=sa" select 
    *  java JDBCExamples  "antibes:4000?user=sa"  update
    *  java JDBCExamples  "antibes:4000?user=sa"  select
    *  java JDBCExamples  "antibes:4000?user=sa"  call
    *  java JDBCExamples  "antibes:4000?user=sa"  select
    *  </pre>
    *  <p>Examples of SQL server invocation:
    *  <pre>
    *  select JDBCExamples.serverMain('insert')
    *  select JDBCExamples.serverMain('select')
    *  select JDBCExamples.serverMain('update')
    *  select JDBCExamples.serverMain('select')
    *  select JDBCExamples.serverMain('call')
    *  select JDBCExamples.serverMain('select')
    *  </pre>
    *  @param  args  a string array that is the command-line arguments
    */  
    public static void main(String args[]) {

        if (args.length!=2) {
            System.out.println("\n Usage:    " 
                 + "java JDBCExamples server-name:port-number action ");
            System.out.println("  The action is connect, createtable, createproc, drop, "
                                     + "insert, select, update, or call \n" );
            return;
        }

        try{
           String server = args[0]; 
           String action = args[1].toLowerCase();
           Connection con = connecter(server);
           String workString = doAction( action, con, "client");                 
           System.out.println("\n" + workString + "\n");

        }  catch (Exception e) {
           System.out.println("\n Exception: ");
           e.printStackTrace();
        }  
    }

   /**  
    * A JDBCExamples method equivalent to the main method, 
    * to be called from SQL or Java in the server
    */  

    public static String serverMain(String action) {
        try {
           Connection con = connecter("default");
           String workString = doAction(action, con, "server");
           return workString;
        }  catch ( Exception e ) {
             if (e.getMessage().equals(null)) {
               return "Exc: " + e.toString();
             } else {
               return "Exc - " + e.getMessage();
             }  
        }  
    }

   /**  
    *  A JDBCExamples method to get a connection.  
    *  It can be called from the server with argument "default",
    *  or from a client, with an argument that is the server name.
    */  

    public static  Connection connecter(String server) 
             throws Exception, SQLException, ClassNotFoundException  {

       String forName="";
       String url="";

       if (server=="default") {  // server connection to current server
           forName = "sybase.asejdbc.ASEDriver";  
           url = "jdbc:default:connection";
       } else if (server!="default") {  //client connection to server
           forName= "com.sybase.jdbc.SybDriver";          
           url = "jdbc:sybase:Tds:"+ server;
       }
       // Load the driver
       Class.forName(forName);
       // Get a connection
       Connection con = DriverManager.getConnection(url);
       return con;
    }

   /**  
    *  A JDBCExamples method to route to the action to be performed
    */  
    public static String doAction(String action, Connection con, String locale) 
          throws Exception {

        String createProcScript =          
             " create proc inoutproc @id int, @newname varchar(50), @newhome Address, " 
           + "   @oldname varchar(50) output,  @oldhome Address output as " 
           + " select @oldname = name, @oldhome = home from xmp where id=@id "
           + " update xmp set name=@newname, home = @newhome where id=@id ";
        String createTableScript =
            " create  table xmp (id int, name varchar(50), home Address)" ;

        String dropTableScript = "drop   table xmp  ";
        String dropProcScript = "drop  proc inoutproc ";
        String insertScript = "insert  into xmp " 
           + "values (1, 'Joe Smith', new Address('987 Shore', '12345'))";

        String workString = "Action (" + action + ") ";
        if (action.equals("connect")) {
             workString += "performed";
        } else if (action.equals("createtable")) {
             workString += doSQL(con, createTableScript );
        } else if (action.equals("createproc")) {
             if (locale.equals("server")) {
                 throw new Exception ("CreateProc cannot be performed in the server");
             } else {
                 workString += doSQL(con, createProcScript );
             }
        } else if (action.equals("droptable")) {
             workString += doSQL(con, dropTableScript );
        } else if (action.equals("dropproc")) {
             if (locale.equals("server")) {
                 throw new Exception( "DropProc cannot be performed in the server");
             } else {
                 workString += doSQL(con, dropProcScript );
             }
        } else if (action.equals("insert")) {
             workString += doSQL(con, insertScript );
        } else if (action.equals("update")) {
             workString += updater(con); 
        } else if (action.equals("select")) {
             workString += selecter(con);
        } else if (action.equals("call")) {
             workString += caller(con);
        } else if (action.equals("delete")) {
             workString += doSQL(con, "delete from xmp");
        } else { return "Invalid action: " + action;
        }                   
        return workString;
    }

   /**  
    *  A JDBCExamples method to execute an SQL statement.
    */  

    public static String doSQL (Connection con, String action)  
           throws Exception {

        Statement stmt = con.createStatement();
        int res = stmt.executeUpdate(action);
        return "performed";

    }

   /**  
    *  A method that updates a certain row of the "xmp" table.
    *  This method illustrates prepared statements and parameter markers.
    */  


    public static String updater(Connection con)  
           throws Exception {

        String sql = "update xmp set name = ?, home = ? where id = ?";
        int id=1;
        Address home = new Address("123 Main", "98765");  
        String name = "Sam Brown";
        PreparedStatement pstmt = con.prepareStatement(sql);
        pstmt.setString(1, name);
        pstmt.setObject(2, home);
        pstmt.setInt(3, id);
        int res = pstmt.executeUpdate();
        return "performed";

    }

   /**  
    *  A JDBCExamples method to retrieve a certain row of the "xmp" table.
    *  This method illustrates prepared statements, parameter markers, 
    *  and result sets.
    */  

    public static String selecter(Connection con)  
           throws Exception {

        String sql = "select name, home from xmp where id=?";
        int id=1;
        Address home = null;
        String name = "";
        String street = "";
        String zip = "";
        PreparedStatement pstmt = con.prepareStatement(sql);
        pstmt.setInt(1, id);
        ResultSet rs = pstmt.executeQuery();
        if (rs.next()) {
           name = rs.getString(1);
           home = (Address)rs.getObject(2);                 
           if (rs.next()) {
              throw new Exception("Error:  Select returned multiple rows");
           } else { // No action
           }         
        } else { throw new Exception("Error:  Select returned no rows"); 
        }                  
        return "- Row with id=1:  name("+ name + ")" 
                 + " street(" + home.street + ") zip("+ home.zip + ")";
    }

   /**  
    *  A JDBCExamples method to call a stored procedure, 
    *  passing input and output parameters of datatype String and Address.
    *  This method illustrates callable statements, parameter markers, 
    *  and result sets.
    */  

    public static String caller(Connection con)  
           throws Exception {

        CallableStatement cs 
               = con.prepareCall("{call inoutproc (?, ?, ?, ?, ?)}");
        int id = 1;
        String newName = "Frank Farr";
        Address newHome = new Address("123 Farr Lane", "87654");
        cs.setInt(1, id);
        cs.setString(2, newName);
        cs.setObject(3, newHome);
        cs.registerOutParameter(4, java.sql.Types.VARCHAR);
        cs.registerOutParameter(5, java.sql.Types.JAVA_OBJECT); 
        int res = cs.executeUpdate();
        while (cs.getMoreResults())  {
           cs.getUpdateCount(); 
        }        
        String oldName = cs.getString(4);
        Address oldHome = (Address)cs.getObject(5);
        return "- Old values of row with id=1:  name("+oldName+") street(" 
               + oldHome.street + ") zip("+ oldHome.zip + ")";
    }
}