//
//  Copyright (c) 2001
//  Sybase, Inc
//  Emeryville, CA 94608
//  All Rights Reserved
//

import java.lang.*;
import java.sql.*;
import java.math.*;

/**  
 *   <p>A class with methods to illustrate SQLJ facilities.</p>
 */  

public class SQLJExamples {

   /**  
    *  A method that returns a code for the geographic region of a given state.
    *  <p>This method is intended for use with an SQLJ <b>create function</b>
    *  such as the following:</p>
    *  <pre>
    *  create function regionof(state char(20)) 
    *		returns integer
    *  language java parameter style java
    *  external name 'Routines1.region(java.lang.String)'
    *  </pre>
    *
    *  <p>That SQLJ function "<i>regionof</i>" would be called in SQL as follows:</p>

    *  <pre>
    *  select name, regionof(state) as region
    *  	from salesemps
    *  where regionof(state)=3
    *  </pre>
    *
    *  @param s         the given state abbreviation
    *  @returns         an int region code
    */

	public static int region(String s) 
			throws SQLException {
		s = s.trim();
		if (s.equals("MN") || s.equals("VT") ||
			s.equals("NH") ) return 1;
		if (s.equals("FL") || s.equals("GA") ||
			s.equals("AL") ) return 2;
		if (s.equals("CA") || s.equals("AZ") ||
			s.equals("NV") ) return 3;
		else throw new SQLException
			("Invalid state code", "X2001");

	}



   /**  
    *  A method that updates the "<i>salesemp</i>" table to correct
    *  the spelling of a given state.
    *  <p>This method is intended for use with an SQLJ <b>create procedure</b>
    *  such as the following:</p>
    *  <pre>
    *  create procedure correctstates(old char(20), 
    *		notold char(20))
    *	modifies sql data
    *	language java parameter style java
    *	external name
    *		'Routines1.correctStates(java.lang.String
    *		java.lang.String)'
    *
    *  </pre>
    *  <p>That SQLJ procedure 'correctstates' would be called in SQL as follows:</p>
    *  <pre>
    *  execute correctstates 'GEO', 'GA'
    *  </pre>
    *  @param oldSpelling         the incorrect spelling, e.g. "Calif"
    *  @param newSpelling         the correct spelling, e.g. "CA"
    */

	public static void correctStates
		(String oldSpelling, String newSpelling)
		throws SQLException {

		Connection conn = null;
		PreparedStatement pstmt = null;
		try {
			Class.forName
				("sybase.asejdbc.ASEDriver");
			conn = DriverManager.getConnection("jdbc:default:connection");
		} 
		catch (Exception e) {
			System.err.println(e.getMessage() 
				+ ":error in connection");
		}
		try {
			pstmt = conn.prepareStatement
				("UPDATE salesemps SET state = ? "
				 + " WHERE state = ?");
			pstmt.setString(1, newSpelling);
			pstmt.setString(2, oldSpelling);
			pstmt.executeUpdate();
		}
		catch (SQLException e) {
			System.err.println("SQLException: " + e.getErrorCode() + e.getMessage());
		}
	return;
	}


   /**  
    *  A method that provides the column data for the top two employees 
    *  whose region is above a given value.
    *  <p>The method is intended for an SQLJ <b>create procedure</b>
    *  such as the following:</p>
    *
    *  <pre>
    *  create procedure best2
    *  	(out n1 varchar(50), out id1 varchar(5), out s1 decimal(6,2), out r1 integer, 
    *  	out n2 varchar(50), out id2 varchar(50), out r2 integer, out s2 decimal(6,2), 
    *  	in region integer)
    *  	language java parameter style java
    *  	external name 'Routines2.bestTwoEmps(java.lang.String[],
    *  		java.lang.String[], int[],
    *  		java.math.BigDecimal[], java.lang.String[],
    *  		java.lang.String[], int[],
    *  		java.math.BigDecimal[], int)'
    *  </pre>
    *  <p>That SQLJ procedure "<i>best2</i>" would be called in SQL as follows:</p>
    *  <pre>
    *  declare @n1 varchar(50), @id1 varchar(5), 
    *  	@s1 decimal (6,2), @r1 integer, @n2 varchar(50), 
    *  	@id2 varchar(50), @r2 integer, @s2 decimal(6,2),
    *  	@region integer
    *  select @region = 3
    *  execute best2 @n1 out, @id1 out, @s1 out, @r1 out,
    *  	@n2 out, @id2 out, @r2 out, @s2 out, @region 
    *  </pre>
    *  @param n1        a string array for the first output name
    *  @param id1       a string array for the first output id
    *  @param r1       a string array for the first output region
    *  @param s1       a string array for the first output sales
    *  @param n2        a string array for the second output name
    *  @param id2       a string array for the second output id
    *  @param r2       a string array for the second output region
    *  @param s2       a string array for the second output sales
    *  @param regionParm  an int for the given region
    */  

	public static void bestTwoEmps
		(String[] n1, String[] id1,  BigDecimal[] s1, int[] r1,
		 String[] n2, String[] id2,  BigDecimal[] s2, int[] r2,
		int regionParm) throws SQLException {

	n1[0] = "****"; n2[0] = "****";
	id1[0] = ""; id2[0] = "";
	r1[0] = 0; r2[0] = 0;
	s1[0] = new BigDecimal(0); 
	s1[0] = new BigDecimal(0);

	try {
                Class.forName("sybase.asejdbc.ASEDriver"); 
		Connection conn = DriverManager.getConnection("jdbc:default:connection"); 
		java.sql.PreparedStatement stmt =
			conn.prepareStatement("SELECT name, id, "
			     + "regionof(state) as region, sales "
			     + " FROM salesemps WHERE regionof(state)>? "
                             + " AND sales IS NOT NULL ORDER BY sales DESC");
		stmt.setInt(1, regionParm);
		ResultSet r = stmt.executeQuery();

		if(r.next()) {
			n1[0] = r.getString("name");
			id1[0] = r.getString("id");
			r1[0] = r.getInt("region");

			s1[0] = r.getBigDecimal("sales");
		}
		else return;

		if(r.next()) {
			n2[0] = r.getString("name");
			id2[0] = r.getString("id");
			r2[0] = r.getInt("region");
			s2[0] = r.getBigDecimal("sales");
		}
		else return;
	} catch (SQLException e) {
		System.err.println(" SQLException:"  
			+ e.getErrorCode() + e.getMessage());
	}catch (Exception e)  {
		System.err.println(" Exception:"  
			+ e.getMessage());
     
        }
    }		

  /**
   *  An example method that returns a result set.
   *  <p>This method is intended for an SQLJ <b>create procedure</b> such as the following:</p>
   *  <pre>
   *  create procedure rankedemps(region integer)
   *  	dynamic result sets 1
   *  	language java parameter style java
   *  	external name 'Routines3.orderedEmps(int)'  
   *  </pre>
   *  <p>That SQLJ procedure "<i>rankedemps</i>"  would be called as a stored procedure
   *  by an SQL client.  For example, a Java client such as the following:</p>
   *  <pre>
   *  java.sql.CallableStatement stmt =
   *  	conn.prepareCall({call rankedemps(?)});
   *  stmt.setInt(1,3);
   *  ResultSet rs = stmt.executeQuery();
   *  while (rs.next()) {
   *  	String name = rs.getString(1);
   *  	int.region = rs.getInt(2);
   *  	BigDecimal sales = rs.get.BigDecimal(3);
   *  	System.out.print(Name =  + name);
   *  	System.out.print(Region =  + region);
   *  	System.out.print(Sales =  + sales);
   *  	System.out.printIn():
   *  }
   *  </pre>
   *  <p>The <i>main</i> method of the Java class <i>CallRankedEmps</i> contains such
   *  a call of "<i>rankedemps</i>".</p>
   *
   *  @param int         an int with a region value
   *  @param rs          the result set that will be returned on the SQLJ call
   */
	public static void orderedEmps
			(int regionParm, ResultSet[] rs) throws
			SQLException {		

		Connection conn = null;
		PreparedStatement pstmt = null;

		try {
			Class.forName("sybase.asejdbc.ASEDriver");
			conn = DriverManager.getConnection("jdbc:default:connection");
		} 
		catch (Exception e) {
			System.err.println(e.getMessage() + ": error in connection" );
		}

		try {
			java.sql.PreparedStatement 
				stmt = conn.prepareStatement 
				("SELECT name, regionof(state) as region, "
				  + "sales FROM salesemps " 
  				  + "WHERE regionof(state) > ? "
				  + " AND sales IS NOT NULL ORDER BY sales DESC");
			stmt.setInt(1, regionParm);
			rs[0] = stmt.executeQuery();
			return;
		}
		catch (SQLException e) {
			System.err.println("SQLException: "
			 + e.getErrorCode() + e.getMessage());
		}
		return;
	} 

   /**  
    *  A method that returns the job name for a given job code.
    *  <p>This method is intended to illustrate the handling of nulls
    *  with the SQLJ <b>null on null input</b> clause.  You would define
    *  an SQLJ function on this method, as follows:</p>
    *  <pre>
    *  create function jobof22(jc integer) returns varchar(20)
    *  returns null on null input
    *  language java parameter style java
    *  external name 'Routines5.job2(int)'    
    *  </pre>
    *  <p> A call of the SQLJ "<i>job22</i>" function will implicitly 
    *  return null (without calling "<i>Routines5.job2</i>" if the parameter value
    *  is null.</p>
    *
    *  @param jc       an int with the given job code
    *  @returns        a string with the job name
    */  

	public static String job(int jc)
			throws SQLException {
	if (jc==1) return( "Admin");

 	else if (jc==2) return ("Sales");
	else if (jc==3) return ("Clerk");
	else return ("unknown jobcode");
	}


}
