//
//  Copyright (c) 1999, 2001
//  Sybase, Inc
//  Emeryville, CA 94608
//  All Rights Reserved
//
package jcs.util;

import java.io.*;
import java.util.*;

/**  
 *  A class with miscellaneous methods:
 *  <pre>
 *  doCommandArgs:   Validates an array of command-line arguments and tags,
 *                   and merges the arguments into an array of defaults.
 *  validateArg:     Checks a keyword arg against permissible values.
 *  markupString:    Returns the argument string with XML markup replaced
 *                   with entity symbols (such as "&lt;" for "<").
 *  isNothing:       Determines if a string is null or whitespace.
 *  sqlSubstring:    A substring method that uses the SQL rules 
 *                   for out-of-range args.
 *  stackTrace2String:  Returns a String with the exception stack trace.
 *  stream2String:   Returns the argumentInputStream contents as a String.
 *  string2Stream:   Returns an InputStream on the argument String.
 *  multiply:        Returns a given String replicated a given number of times.
 *  genString:       Returns a given string replicated to a given length.
 *                   
 *  </pre>
 */  

public class StringUtil {

   /**  
    *  The StringUtil class contains only static methods, and cannot be instantiated.
    */     
   private StringUtil() {}  

   /**  
    *  Checks the command line arguments and merges them with default
    *  values for any omitted arguments. 
    *  <p>Example Java code:
    *  <pre>
    *  public static void main (String args[]) {
    *  String[] argTags = {"-I", "-O", "-ROW", "-CODE",  "-NAMES", "-UPDATE"};
    *  String[] argVals = {"", "", "0", "0", "", "no"};
    *  StringUtil.doCommandArgs(args, argTags, argVals);
    *  </pre>
    *  <ul>
    *  <li>The <code>args</code> array is the command line <code>String[]</code>. 
    *  <li>The <code>argTags</code> are the permissible "tags" for command line
    *  arguments.   
    *  <li>The <code>argVals</code> is an array of the default values for the
    *  command line arguments, in the same order as the <code>argTags</code>.
    *  </ul>
    *  A call of the <code>doCommandArgs</code> method merges the supplied values
    *  in the <code>args</code> array into the default values in the <code>argVals</code>
    *  array, using the <code>argTags</code> array to determine the permissible tags
    *  and the order of arguments in <code>argVals</code>.
    *  <p>For example, assume that the above <code>main</code> method is in a class named
    *  <code>Sample</code>.
    *  The following shows a series of calls of <code>Sample</code>, followed by
    *  the values that would be in <code>argVals</code> after the call 
    *  <code>StringUtil.doCommandArgs(args, argTags, argVals)</code>.
    *  <pre>
    *  Call:  java Sample -ROW 7 -NAMES yes -I /usr/u/smith/input.txt
    *  Resulting argVals:  {"/usr/u/smith/input.txt", "", "7", "0", "yes", "no"}
    *    
    *  Call:  java Sample -O "result.txt" -UPDATE no -CODE "K 7"
    *  Resulting argVals:  {"", "result.txt", "0", "K 7", "", "no"}
    *  </pre>
    *  <ul>
    *  <li>The <code>args</code> may be specified in any order. 
    *  <li>The <code>doCommandArgs</code> method does no folding of tags or parameters,
    *  and no checking of parameter values or combinations.  
    *  <li>The string following a tag may be quoted or not, but must be quoted 
    *  if it contains spaces.  
    *  </ul>
    *  @param    args       the string array of user-supplied arguments,
    *                       as provided in command-line invocation of 
    *                       <code>main</code>
    *  @param    argTags    an array of the legal argument tags, 
    *                       e.g. "-F". (case sensitive)
    *  @param    argVals    an array of the default values of the arguments 
    *                       (or empty strings for arguments with no default)
    *  @throws  Exception If an argument tag letter is not specifies by the
    *                       <code>argTags</code>, or the <code>argTags</code>
    *                       and <code>argVals</code> arrays are not the same length.
    */  
           
   public static void doCommandArgs(String[] args, String[]argTags, 
                                    String[]argVals )  throws Exception {
             
      if (argTags.length != argVals.length) {
            throw new Exception("\nInternal error:  " 
                                + "argTags and argVals must be same length\n");
      } 
            
      for (int i = 0; i<args.length; i++) {
         boolean found=false;
         tag: for (int j=0; j<argTags.length;j++) {
            if (args[i].equals(argTags[j])) {
               if (args[i].length()>argTags[j].length()){
                  argVals[j] = args[i].substring(argTags[j].length(), args[i].length());
               } else if (args.length> (i+1)) {
                  i++;
                  argVals[j] = args[i];
               }
               found=true;
               break tag;   
            } 
         }         
         if (! found) {
            throw new Exception ( "Unrecognized arg tag (" 
                      + args[i]+ ")");
         }   
      } 
   }


   /**  
    *  Tests whether the argument is null, empty, or all whitespace.
    *  This is a convenience method for a common test.
    *  @param  val  a string to be tested for vacuity
    *  @return a boolean indicating the vacuity of <code>val</code> 
    */  
   public static boolean isNothing(String val) { 
      return (val==null || val.trim().equals(""));
   }


   /**  
    *  Verifies that the <code>arg</code> is in the <code>permissible</code> values.
    *  The "permissible" string contains the permissible keyword values.
    *  <p>Example Java code:
    *  <pre>
    *       validateArg("yes", "yes,no,maybe", "Arg must be yse, no, or maybe")
    *  </pre>
    *  @param arg  a string with the value to be validated
    *  @param permissible  a string with the valid values.  The method
    *                      simply checks whether the index of <code>arg</code>
    *                      in <code>permissible</code> is -1.
    *  @param message  a string that will be included in the exception message
    *  @throws IllegalArgumentException  Raised if the <code>arg</code>
    *                   is not in the <code>permissible</code> values
    */  
   public static String validateArg(String arg, String permissible, String message) 
              throws IllegalArgumentException {

      String argVar = (arg!=null)?arg.trim().toLowerCase():"";
      String permissibleVar = (permissible!=null)?permissible:"";
      String messageVar = (message!=null)?message:"";
          
      if (argVar.equals("") 
            || (permissibleVar.indexOf(argVar) == -1)) {
        throw new IllegalArgumentException(messageVar+argVar);        
      } else {
        return argVar;
      }
   }

   /**  
    *  Returns a substring of the given string.  This method uses SQL rules 
    *  for the FROM and TO values:
    *  <pre><code>
    *         sqlSubstring("abc", S, L)
    *  </code></pre>
    *  <ul>
    *  <li>"S" is the start, beginning at "1", not "0".
    *  <li>"L" is the result length, not the end position.
    *  <li>SQL rules apply when S and or L are outside the string.
    *  </ul>
    *  <p>This method is a literal transcription of the rules in the SQL standard,
    *  and the variable names are the symbols used in the SQL standard (in caps).
    *  @param C  a string that is the source for the substring values
    *  @param startArg  a string that is the initial position of the substring
    *  @param lengthArg  an int that is the length of the substring
    *  @return a string with the substring result
    *  @throws Exception  If the end is less than the start
    */  

   public static String sqlSubstring(String C, int startArg, int lengthArg)  
        throws Exception {
        
      int S = startArg;
      int L = lengthArg;
      int LC = C.length();
      int E = S + L;
      if (C==null) return null; 
      else if ( E < S ) throw new Exception("sqlSubstring: "
                                   +"end less than start");
      else if ( S > LC || E < 1) return "";
      else {
         int S1 = (S>1)?S:1;
         int E1 = (E< LC+1)?E:(LC+1);
         int L1 = E1-S1;
         return C.substring(S1-1, E1-1);                             
      }
   }        

   /**  
    *  Returns a String with the stack trace of the given Exception.<br>
    *  This is an alternative to using <code>e.printStackTrace()</code>, 
    *  for cases where you want to save the exception info rather than
    *  print it.
    *  @param  e  an exception whose stack trace is to be returned
    *  @return    a string with the stack trace
    */  

   public static String stackTrace2String (Exception e ) {
      StringWriter sw = new StringWriter();
      e.printStackTrace(new PrintWriter(sw));
      return sw.toString();
   }


   /**  
    *  Translates a String containing XML markup characters,
    *  (&gt;, &lt;, &amp;, &apos;, and &quot;) to a String with the corresponding
    *  entities (&amp;gt;, &amp;lt;, &amp;amp;, &amp;apos;, and &amp;quot;).
    *  The <i>deEntityize</i> method does the inverse.
    *  <br><table align="center" border="3" cellpadding="3" bgcolor="00ffff">
    *  <thead><th>Call</th><th>Result</th></thead><tbody>
    *  <tr><td><pre>
    *  entityize(&quot;A&gt;B &amp;&amp; C&lt;D&quot;) 
    *  </pre></td><td><pre>
    *  &quot;A&amp;gt;B &amp;amp;&amp;amp; C&amp;lt;D&quot;
    *  </pre></td></tr><tr><td><pre>
    *  deEntityize(&quot;A&amp;gt;B &amp;amp;&amp;amp; C&amp;lt;D&quot;)
    *  </pre></td><td><pre> &quot;A&gt;B &amp;&amp; C&lt;D&quot;
    *  </pre></tr></table>
    *  @param  val  a string that is to be examined for XML markup
    *  @return  a string that is a copy of <code>val</code> with XML
    *           markup characters suitably substitutued
    */  
   public static String entityize(String val) {
      if (val==null) {
         return null;
      }
      String markups = "<>&\"'";
      String[ ] entities = {"&lt;", "&gt;", "&amp;", "&quot;", "&apos;"};
      StringBuffer result = new StringBuffer(1000);
      for (int i=0; i<val.length(); i++) {
         int j = markups.indexOf(val.substring(i, i+1));
         if ( j != -1) {
            result.append(entities[j]);
         } else {
            result.append( val.substring(i, i+1));
         }
      }
      return result.toString();
   }

   /**  
    *  Translates a String containing the five special XML entities 
    *  (&amp;gt;, &amp;lt;, &amp;amp;, &amp;apos; and &amp;quot;).
    *  to a String with the corresponding characters
    *  (&gt;, &lt;, &amp;, &apos;, and  &quot;). 
    *  The <i>entityize</i> method does the inverse.
    *  <br><table align="center" border="3" cellpadding="3" bgcolor="00ffff">
    *  <thead><th>Call</th><th>Result</th></thead><tbody>
    *  <tr><td><pre>
    *  entityize(&quot;A&gt;B &amp;&amp; C&lt;D&quot;) 
    *  </pre></td><td><pre>
    *  &quot;A&amp;gt;B &amp;amp;&amp;amp; C&amp;lt;D&quot;
    *  </pre></td></tr><tr><td><pre>
    *  deEntityize(&quot;A&amp;gt;B &amp;amp;&amp;amp; C&amp;lt;D&quot;)
    *  </pre></td><td><pre> &quot;A&gt;B &amp;&amp; C&lt;D&quot;
    *  </pre></tr></table>
    *  @param  val  a string that is to be examined for XML entities.
    *  @return  a string that is a copy of <code>val</code> with XML
    *           entity characters suitably substituted
    */  
   public static String  deEntityize(String val) throws Exception {      

      if (val==null) {return null;
      }
      StringBuffer result = new StringBuffer(1000);
      int i=0;  int j = 0; 
      int valLength = val.length();
      while (i<valLength) {
         j=val.indexOf("&", i);
         if(j == -1) {
            result.append(val.substring(i));
            i = valLength+1;
         } else {   
           result.append(val.substring(i, j));
           String work6 = sqlSubstring(val, j+1, 6);
           String work5 = sqlSubstring(work6, 1, 5);
           String work4 = sqlSubstring(work5, 1, 4);
	   if (work6.toLowerCase().equals("&apos;")) {
		result.append("\'"); i=j+6;
	   }else if (work6.toLowerCase().equals("&quot;")) {
		result.append("\""); i=j+6;
	   }else if (work5.toLowerCase().equals("&amp;")) {
		result.append("&"); i=j+5;
	   }else if (work4.toLowerCase().equals("&gt;")) {
		result.append(">"); i=j+4;
	   }else if (work4.toLowerCase().equals("&lt;")) {
		result.append("<"); i=j+4;
           }else {
		result.append("&"); i+=1;
           }
         }

      }
      return result.toString();
   }

   /**  
    *  Returns the contents of the argument InputStream as a String.
    *  @param  iStream   an InputStream to be returned as a String
    *  @return      the contents of the InputStream
    */  
   public static String stream2String(InputStream iStream) throws Exception {
      if (iStream==null) {
         return null;
      } 
      byte[] ba = new byte[1000];
      int len=0;
      StringBuffer sb = new StringBuffer(1000);
      while((len=iStream.read(ba))!= -1) {
         sb.append(new String(ba, 0, len));
      }
      return sb.toString();

   }

   /**  
    *  Returns the stream on the argument string.
    *  @param  iString   a String to be returned as a stream
    *  @return      the contents of the InputStream
    */  
   public static InputStream string2Stream(String iString) throws Exception {
      if (iString==null) {
         return null;
      } else {
         return new ByteArrayInputStream(iString.getBytes());
      }
   }


   /**  
    *   Repeats a given string a given number of times 
    *   (mainly for test generation).
    *     multiply(3, "abc") returns "abcabcabc"
    *     multiply(0, "abc") returns ""
    *     multiply(n, "") returns ""
    *     multiply(null, n)  returns null
    *  @param times   the number of replications
    *  @param seed  the string that will be replicated to generate
    *               the returned string
    *  @return      a string with the given number of replications

    */  

   public static String multiply (int times,  String seed){

      if (seed==null) {
         return null;
      }
      StringBuffer result = new StringBuffer(1000);
      for (int i=0; i<times; i++) {
         result.append(seed);
      }
      return result.toString();
   }

   /**  
    *  Repeats a given string a given number of times 
    *  (mainly for test generation).
    *  <p><code>multiply(String, int)</code> is simply 
    *  <code>multiply(int, String)</code>, for convenience.
    */  
   public static String multiply (String seed, int times){
      return multiply(times, seed);
   }    

   /**  
    *  Generates a string of given length (mainly for test generation).
    *    genString(6 "abc") returns "abcabc"
    *    genString(5, "abc") returns "abcab"
    *    genString(4, "abc") returns "abca"
    *    genString(0, "abc") returns ""
    *    genString(n, "") returns ""
    *    genString(n, null) returns null
    *  @param len   the length of the returned string
    *  @param seed  the string that will be replicated to generate
    *               the returned string
    *  @return      a string of the given length
    */  
          
   public static String genString(int len, String seed) {
      if (seed == null) return null;
      int seedLen = seed.length();
      if (seedLen == 0 || len <= 0) return "";
      StringBuffer result= new StringBuffer(1000);
      while (result.length() <= len-seedLen){
         result.append(seed);
      }

      result.append( seed.substring(0, len-result.length()));
      return result.toString();
   }

   /**  
    *  Generates a string of given length (mainly for test generation).
    *  <p><code>genString(String, int)</code> is simply 
    *  <code>genString(int, String)</code>, for convenience.
    */  
   public static String genString(String seed, int len) {
      return genString(len, seed);
   }

}