//
//  Copyright (c) 1999, 2001
//  Sybase, Inc
//  Emeryville, CA 94608
//  All Rights Reserved
//
package jcs.xml;

import java.io.*;

// JAXP
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;

// DOM
import org.w3c.dom.Document;
import org.w3c.dom.DocumentType;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import org.xml.sax.InputSource;
import org.xml.sax.EntityResolver;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;


   /**  
    *  A class for XML documents.  
    *  <ul>
    *  <li>The class contains an XML document
    *  as a Java String field.   
    *  <li>The primary method of the class is a <code>parse()</code> 
    *  method.  This method internally uses the Java(tm) Project X 
    *  Technology Release 1.
    *  See the <a href
    *  ="http://developer.java.sun.com/developer/earlyAccess/xml/index.html">
    *  Project X</a> web site.
    */  

public class JXml implements java.io.Serializable {

   /**  
    *  The character text of the XML document contained 
    *  in the JXml object.   
    *  <p>For a simple JXml instance, the xmlText is not validated by the
    *  JXml constructor.  Subclasses of JXml will commonly validate the
    *  xmlText in their constructors.  This is the reason for making the
    *  xmlText field private.
    *  @serial  A String value containing XML text.
    */  

   protected String xmlText;

   /**  
    *  Constructs a <code>JXml</code> instance from the argument text.
    *  The constructor does no validation of the argument.  If it is
    *  not well-formed XML, then a subsequent invocation of the 
    *  <code>parse</code> method will raise exceptions.
    *  <p>All validation of XML documents stored in JXML instances
    *  is defined by subclasses of JXml, such as 
    *  {@link jcs.xml.order.OrderXml OrderXml} 
    *  or {@link jcs.xml.resultset.ResultSetXml ResultSetXml}.
    */  

   public JXml (String xmlTextArg) {
      xmlText = xmlTextArg;
      
   }

   /**  
    *  Return the content of the JXml instance.
    */  

   public String toString ()  {
      return xmlText;
   }

   /**  
    *  A dummy <code>resolveExternal</code> method.  This method throws an exception
    *  whenever invoked.
    *  <p>Assume that <code>SomeXml</code> is either the class <code>JXml</code>
    *  or a subclass thereof, and that <code>someXml</code> is of type
    *  <code>SomeXml</code>.  Consider a call <code>someXml.parse()</code>.
    *  If the <code>parse</code> method of JXml has not been overridden,
    *  then it will parse the document contained in <code>someXml</code>.
    *  Suppose that this document contains one or more external entity references,
    *  such as:
    *  <pre> 
    *  &lt;!DOCTYPE SYSTEM "SomeXmlDoc.dtd"&gt;
    *  </pre> 
    *  <p>Normally, an XML parser would look for a file named 
    *  <code>SomeXmlDoc.dtd</code>.  However, the <code>parse</code> method of JXml 
    *  does not have access to operating system files, and instead resolves
    *  such external entity references by calling <code>this.resolveExternal(P, S)</code>,
    *  which returns the definition of the external reference.
    *  <p>In the current example, this call of <code>this.resolveExternal(P, S)</code>
    *  will invoke the <code>resolveExternal</code> method of the <code>SomeXml</code> 
    *  class,  which should then return the appropriate DTD for <i>SomeXml</i> documents.
    *  <p>If the call of <code>this.resolveExternal(P, S)</code> by the JXml 
    *  <code>parse</code> method ever invokes the <code>resolveExternal</code> method of
    *  <code>JXml</code> itself, then either the subclass of JXml did not define
    *  a <code>resolveExternal</code> method, or an instance of JXml is being parsed,
    *  and that instance contained an external entity reference.  The JXml
    *  class has no "built-in" or "implicit" DTD or external entity definitions,
    *  so any such external entity reference is undefined, and the 
    *  <code>resolveExternal</code>
    *  method of JXml will raise an exception.  
    *  <p>For examples of the <code>resolveExternal</code> methods of subclasses of
    *  <code>JXml</code>, see <a href="jcs.xml.example.OrderXml.html#resolveExternal">
    *  OrderXml.resolveExternal</a> and 
    *  <a href="jcs.resultset.ResultSetXml.html#resolveExternal">
    *  ResultSetXml.resolveExternal</a>.
    *  @param  publicId   the value (if any) specified with the PUBLIC keyword in an
    *                     external entity reference, or null.
    *  @param  systemId   the value specified with the SYSTEM keyword in an
    *                     external entity reference.
    *  @return  a string with the DTD or other definition of the given external entity.
    *  @throws SAXException  Thrown when the SYSTEM and PUBLIC names do not
    *                           define an external entity known by the method.
    *                           Note that this exception must be a SAXException.
    */  
  
   public String resolveExternal (String publicId, String systemId)  
                                           throws SAXException {
      throw new SAXException("JXml - Unknown external entity name "
           +"\nt publicID is (" + publicId + ") systemID is (" + systemId+")");
   }

   /**  
    *  Parses the XML document and returns the resulting 
    *  <code>Document</code> object.
    *  @return an Document object that is the parsed object
    *  @throws Exception  Any exceptions arising in the XML parse
    */  
   public Document parse () throws Exception {
      return parse(true);
   }
   /**  
    *  Parses the XML document and returns the resulting 
    *  <code>Document</code> object.
    *  @param  validate  a boolean indication of whether (true) or not (false)
    *                    the parse should validate
    *  @return an Document object that is the parsed object.
    *  @throws Exception  Any exceptions arising in the XML parse
    */  
   public Document parse (boolean validate) throws Exception {


      try{      

         // Get Document Builder Factory
         DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

         // Set validation, and turn off namespaces
         factory.setValidating(false);
         factory.setNamespaceAware(false);
         factory.setIgnoringComments(true);
         factory.setIgnoringElementContentWhitespace(true);
         factory.setExpandEntityReferences(false);
         factory.setCoalescing(false);
         DocumentBuilder builder = factory.newDocumentBuilder();

         //  Supply an entity resolver for the jdbc DTD.
         ServerEntityResolver ser = new ServerEntityResolver(this);
  	 builder.setEntityResolver (ser);
         //  Supply a default ErrorHandler.
	 builder.setErrorHandler (new ErrorHandler ());

         Document doc = builder.parse(jcs.util.StringUtil.string2Stream(xmlText));

         return doc;

      } catch (Exception err) {
         throw new Exception( "Class JXml:  Parsing error \n" + err.getMessage());
      }
   }
}
   
