//
//  Copyright (c) 1999, 2001
//  Sybase, Inc
//  Emeryville, CA 94608
//  All Rights Reserved
//
package jcs.xml;

import org.xml.sax.InputSource;
import org.xml.sax.EntityResolver;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;


/**  
 *  A class to resolve external entity references for the XML parser.
 *  This class is used internally by the JXml class.
 *  It provides the mechanism for resolving entity references such as:
 *  <pre>
 *     <code>&lt;!DOCTYPE ResultSet SYSTEM  "ResultSet.dtd"&gt;</code>
 *  </pre>
 *  <p>The <code>parse</code> method of the <code>JXml</code> class provides 
 *  an instance of this class to the XML parser, which invokes the 
 *  <code>resolveEntity</code> of that instance to resolve a reference 
 *  such as <code>ResultSet.dtd</code>.  
 *  <p>The XML parser would normally resolve such a reference by looking for a file 
 *  named "ResultSet.dtd" in the "current" directory.  But, when the XML parser 
 *  is called in the SQL server,  it cannot access operating system files, 
 *  and therefore needs some other way to resolve external entity references.
 *  <p>This class is only used internally in the <code>parse</code> method
 *  of the <code>JXml</code class.  The function of this class is subsumed
 *  by the <code>resolveExternal</code> methods of <code>JXml</code> and subclasses
 *  thereof. 
 */  

public class ServerEntityResolver implements EntityResolver {

   JXml thisJXmlDoc;

   public ServerEntityResolver(JXml thisJXmlDocArg) {
      thisJXmlDoc = thisJXmlDocArg;
   }

   /**  
    *  Returns the value for a given external ID.  This method is called
    *  internally by the XML parser to resolve external entity references.
    *  @param  publicId  a string that either is null, or is the value
    *                    specified as <code>PUBLIC</code> in an external
    *                    entity reference. 
    *  @param  systemId  a string that either is null, or is the value
    *                    specified as <code>SYSTEM</code> in an external
    *                    entity reference. 
    *  @return an InputSource containing the resolution of the external entity
    *          denoted by the <code>publicId</code> 
    *           and/or <code>systemId</code> values.
    *  @throws SAXException  Thown when the given external entity is not
    *                           resolvable, or when an IOException occurs
    *                           internally while constructing the 
    *                           <code>InputSource</code> on the entity's resolution.
    */  

   public InputSource resolveEntity (String publicId, String systemId)  
            throws  SAXException{

      String entityDef = thisJXmlDoc.resolveExternal(publicId, systemId);
      if (entityDef==null) {
         throw new SAXException ("ServerEntityResolver: "
           +"Unrecognized external reference: "
           +"publicID is (" + publicId 
           + ") systemID is (" + systemId+")");
      }
      try{
         InputSource entityDefSource 
	   = new InputSource(jcs.util.StringUtil.string2Stream(entityDef));
         return entityDefSource;
      }catch(Exception e) {
	throw new SAXException ("ServerEntityResolver: "
		+ " Invalid resolution ("+entityDef+")");
      }

   }
}


