//
//  Copyright (c) 1999, 2001
//  Sybase, Inc
//  Emeryville, CA 94608
//  All Rights Reserved
//
package jcs.xml.order;
import jcs.xml.*;
import jcs.util.*;

import java.io.*;
import java.sql.*;

import org.w3c.dom.*;
import org.xml.sax.SAXException;

/**  
 *  A subclass of the <code>JXml</code> class, for the "Order" DTD.
 *  This subclass validates the documents with the Order DTD.
 *  The "Order" DTD is a simple example DTD for purchase-order type
 *  data.  
 */  

public class OrderXml extends JXml implements java.io.Serializable{

   /**  
    *  Returns the DTD for "Order" XML documents.  
    *  <p>Any subclass of the JXml class is expected to have a <code>resolveExternal</code>
    *  method such as this.  For a description of the role of this method,
    *  see the {@link JXml#resolveExternal resolveExternal} method of JXml.
    *  @param  publicId   the value (if any) specified with the PUBLIC keyword in an
    *                     external entity reference, or null.
    *  @param  systemId   the value specified with the SYSTEM keyword in an
    *                     external entity reference.
    *  @return  a string with the DTD or other definition of the given external entity.
    *  @throws SAXException  Thrown when the SYSTEM and PUBLIC names do not
    *                           define an external entity known by the method.
    *                           Note that this exception must be a SAXException.
    */  

   public String resolveExternal(String publicId, String systemId)  throws SAXException {
      if ( (systemId!=null && systemId.toLowerCase().indexOf(DTD_NAME)!= -1)
		||  (publicId!=null && publicId.toLowerCase().indexOf(DTD_NAME)!= -1)) {
         return DTD;     
      } else {
         throw new SAXException ("OrderXml exception - Invalid external reference:"
           +"\n\t publicID is (" + publicId + ") systemID is (" + systemId+")"
           +")\n\t expected name containing ("+ DTD_NAME +")" );
      }
   }

   /**  
    *  A constructor whose parameter is a String containing a valid XML
    *  "Order" document.
    *  <p>This constructor overrides the corresponding constructor of the JXML
    *  super class, and performs document validation for the Order DTD.
    *  @param  OrderXmlText    a string that is the Order XML document.
    *  @throws Exception   Thrown if the argument document is not valid
    */  

   public OrderXml(String orderXmlText) throws Exception {
      super(orderXmlText);
      this.parse(true);
   }

   /**  
    *  A static final line feed for pretty-print formatting.
    */  
   static final String LINE_FEED = "\n";
   
   /**  
    *  The static final DTD name, "order.dtd".
    */  
   static final String DTD_NAME = "order.dtd";

   /**  
    *  The constant DTD for the example document type "Order".
    *  Notes:
    *  <ul>
    *  <li>Don't include any characters after the markup. 
    *  For example, no "LINE_FEED".
    *  </ul>
    */  

   static final String DTD  = 
              "<!ELEMENT Order (Date, CustomerId, CustomerName, Item*)>" + LINE_FEED
            + "<!ELEMENT Date (#PCDATA)>" + LINE_FEED
            + "<!ELEMENT CustomerId (#PCDATA)>" + LINE_FEED
            + "<!ELEMENT CustomerName (#PCDATA)>" + LINE_FEED
            + "<!ELEMENT Item (ItemId, ItemName, Quantity)>" + LINE_FEED
            + "<!ELEMENT ItemId (#PCDATA)>" + LINE_FEED
            + "<!ELEMENT ItemName (#PCDATA)>" + LINE_FEED
            + "<!ELEMENT Quantity (#PCDATA)>" + LINE_FEED
            + "<!ATTLIST Quantity units CDATA #IMPLIED>";
   
   /**  
    *  Constructs an Order for a given customer and date with
    *  Order data from specified SQL tables.
    *  <p>This method assumes the following SQL tables:
    *  <pre> 
    *  create table customers 
    *     (customer_id varchar(5) not null unique, 
    *     customer_name varchar(50) not null)
    *  create table orders 
    *     (customer_id varchar(5) not null, 
    *  order_date datetime not null, 
    *     item_id varchar(5) not null, 
    *     quantity int not null)
    *  create table items 
    *     (item_id varchar(5) unique, 
    *     item_name varchar(20),
    *     unit smallint)
    *  </pre> 
    *
    *  @param orderDate      a string that is the <code>order_date</code> 
    *                        of the requested order.  This string must be in an SQL
    *                        format, such as "1999/05/07".
    *  @param customerId     a string that is the <code>customer_id</code> 
    *                        of the requested order
    *  @param  server  a string that identifies the SQL server in which to execute
    *                  the query.  See the 
    *                  {@link ExecSql#connectTo(java.lang.String) connectTo} 
    *                  method for a description of this parameter.
    *  @return a string with the output XML document that contains the XML "Order"
    *          representation of the order
    *  @throws ClassNotFoundException  Thrown when the XML parser class 
    *                                     cannot be loaded.
    *  @throws SQLException  Thrown for any exceptions raised during execution
    *                           of the SQL statement
    *  @throws Exception  Thrown for invalid parameters and other conditions
    */  

   public OrderXml(String date, String customerId, String server) throws Exception {
       super(sql2Order(date,  customerId,  server));    
   }

   /**  
    *  Returns a string with an empty XML Order for a given customer and date.
    *  The OrderXml has no &lt;Item&gt;s.
    *  @param date         the &lt;Date&gt; for the new OrderXml
    *  @param customerId    the &lt;CustomerId&gt; for the new OrderXml
    *  @param customerName  the &lt;CustomerName&gt; for the new OrderXml
    *  @throws Exception   Thrown when the document can't be created.
    */  

   public static String  emptyOrder(String date, String customerId, String customerName) 
			throws Exception {

      //  Create the output XML document, "doc".
      Document doc = jcs.xml.DomTree.newDocument();

      //  Create an "&lt;Order&gt;" element and attach it to the doc.
      Node orderNode = DomTree.appendChild(doc, doc, "Order");
      DomTree.appendChild(doc, orderNode, "Date", date, false);
      DomTree.appendChild(doc, orderNode, "CustomerId", customerId, false);
      DomTree.appendChild(doc, orderNode, "CustomerName", customerName, false);

      return jcs.xml.DomTree.doc2String(doc);
                  

   }

   /**  
    *  Returns a specified header element from the OrderXml,
    *  or an empty string if the element doesn't exist.
    *  The <code>elementName</code> is one of <code>"Date", "CustomerId",
    *  or "CustomerName". </code>
    *  @param  elementName   the name of one of the above elements of the Order DTD
    *  @return  a string with the text of that element or empty.
    *  @throws  Exception  Thrown if the document is not valid.
    */  

   public String getOrderElement(String elementName) throws Exception {
      Document doc = this.parse();
      Node orderNode = DomTree.topNode(doc, "Order");        
      Node element = DomTree.findNode(orderNode, elementName);
      return DomTree.getText(element);
   }

   /**  
    *  Sets a specified header element from the Order, and returns the updated OrderXml.
    *  The <code>elementName</code> is one of <code>"Date", "CustomerId",
    *  or "CustomerName". 
    *  @param  elementName   the name of one of the above elements of the Order DTD
    *  @param  newValue  the new text of the element
    *  @return  an OrderXml object with the updated document
    *  @throws  Exception  Thrown if the elementName is not valid
    */  

   public void setOrderElement(String elementName, String newValue) 
                                             throws Exception {

      Document doc = this.parse();
      Node order = DomTree.topNode(doc, "Order");        
      Node targetNode = DomTree.findNode(order, elementName);
      if (targetNode==null) {
         throw new Exception("OrderXml exception:  specified element doesn't exist.");
      }
      DomTree.setText(doc, targetNode, newValue);   
      this.xmlText = jcs.xml.DomTree.doc2String(doc);    
   }


   /**  
    *  Returns a specified item element from the OrderXml, or an empty string
    *  if the item element doesn't exist.
    *  The <code>itemNumber</code> is the number of an item in the document
    *  (starting with 0). The <code>elementName</code> is one of <code>"ItemId", 
    *  "ItemName",or "Quantity".
    *  @param  itemNumber  the number of an Item in the OrderXml 
    *  @param  elementName   the name of one of the above elements of the Order DTD
    *  @return  a string with the text of that element, or null if it doesn't exist.
    *  @throws  Exception if the document is not valid.
    */  

   public String getItemElement(int itemNumber, String elementName) throws Exception {
      Document doc = this.parse();
      Node orderNode = DomTree.topNode(doc, "Order");   
      Node itemNode = DomTree.findNode(orderNode, "Item", itemNumber);
      Node Element = DomTree.findNode(itemNode, elementName); 
      return DomTree.getText(Element);
   }

   /**  
    *  Sets a specified item element from the OrderXml.
    *  The <code>itemNumber</code> is the number of an item in the document
    *  (starting with 0). The <code>elementName</code> is one of <code>"ItemId", 
    *  "ItemName",or "Quantity".  The <code>newValue</code> is the replacement 
    *  value for the element.
    *  @param  itemNumber  the number of an Item in the OrderXml 
    *  @param  elementName   the name of one of the above elements of the Order DTD
    *  @param  newValue  the new value for the specified element
    *  @return  an OrderXml with the updated document
    *  @throws  Exception  Thrown if the item element doesn't exist.
    */  

   public void setItemElement(int itemNumber, String elementName, String newValue) 
                                             throws Exception {

      Document doc = this.parse();
      Node orderNode = DomTree.topNode(doc, "Order"); 
      Node itemNode = DomTree.findNode(orderNode, "Item", itemNumber);    
      Node element = DomTree.findNode(itemNode, elementName);
      if (element==null) {
         throw new Exception("OrderXml exception:  specified item element doesn't exist.");
      }
      DomTree.setText(doc, element, newValue);
      this.xmlText = jcs.xml.DomTree.doc2String(doc);
   }

   /**  
    *  Returns a specified item attribute from the OrderXml, or an empty string
    *  if the attribute doesn't exist.
    *  The <code>itemNumber</code> is the number of an item in the document
    *  (starting with 0). The <code>elementName</code> is one of <code>"ItemId", 
    *  "ItemName",or "Quantity".  The <code>attributeName</code> is the
    *  name of an attribute of the element named <code>elementName</code>.
    *  In the Order DTD, only one element has an attribute, namely the
    *  <code>unit</code> attribute of the <code>quantity</code> element.
    *  This method is written for the general case, for illustration.
    *  @param  itemNumber  the number of an Item in the OrderXml 
    *  @param  elementName   the name of one of the above elements of the Order DTD
    *  @return  a string with the text of that element
    *  @throws  Exception  Thrown if the elementName is not valid
    */  

   public String getItemAttribute(int itemNumber, String elementName,
                           String attributeName) throws Exception {

      Document doc = this.parse();
      Node orderNode = DomTree.topNode(doc, "Order");   
      Node itemNode = DomTree.findNode(orderNode, "Item", itemNumber);
      Node Element = DomTree.findNode(itemNode, elementName);         
      return DomTree.getAttribute(Element, attributeName);
   }

   /**  
    *  Sets a specified attribute of an item element from the OrderXml.
    *  The <code>itemNumber</code> is the number of an item in the document
    *  (starting with 0). The <code>elementName</code> is one of <code>"ItemId", 
    *  "ItemName",or "Quantity".  The <code>attributeName</code> is the
    *  name of an attribute of the element <code>elementName</code>.
    *  The <code>newValue</code> is the replacement  value for the attribute.
    *  In the Order DTD, only one element has an attribute, namely the
    *  <code>unit</code> attribute of the <code>quantity</code> element.
    *  This method is written for the general case, for illustration.
    *  @param  itemNumber  the number of an Item in the OrderXml 
    *  @param  elementName   the name of one of the above elements of the Order DTD
    *  @param  newValue  the new value for the specified element
    *  @return  an OrderXml with the updated document
    *  @throws  Exception  Thrown if the elementName is not valid
    */  

   public void setItemAttribute(int itemNumber, String elementName, 
             String attributeName,   String newValue)  throws Exception {
                                   
      Document doc = this.parse();
      Node orderNode = DomTree.topNode(doc, "Order"); 
      Node itemNode = DomTree.findNode(orderNode, "Item", itemNumber);    
      Node Element = DomTree.findNode(itemNode, elementName);
      DomTree.setAttribute( (Element)Element, attributeName, newValue);
      this.xmlText = jcs.xml.DomTree.doc2String(doc);
   }

   /**  
    *  Appends a new &lt;Item&gt; to the OrderXml.
    *  @param  newItemId     the &lt;ItemId&gt; for the new &lt;Item&gt;
    *  @param  newItemName   the &lt;ItemName&gt; for the new &lt;Item&gt;
    *  @param  newQuantity   the &lt;Quantity&gt; for the new &lt;Item&gt;
    *  @param  unit          the "unit" attribute for the &lt;Quantity&gt;
    *  @throws  Exception  Thrown if the elementName is not valid
    */  

   public void  appendItem (String newItemId, String newItemName, 
                         String newQuantity, String unit) throws Exception {

      Document doc = this.parse();
      Node orderNode = DomTree.topNode(doc, "Order");                   
      Element itemNode = DomTree.appendChild(doc, orderNode, "Item");
      DomTree.appendChild(doc, itemNode, "ItemId", newItemId, false);
      DomTree.appendChild(doc, itemNode, "ItemName", newItemName, false);
      Element quantity 
          =  DomTree.appendChild(doc, itemNode, "Quantity", newQuantity, false);
      if (Integer.parseInt(unit)!=1) {
            DomTree.setAttribute(quantity, "unit", unit);
      }
      this.xmlText = jcs.xml.DomTree.doc2String(doc);
   }

   /**  
    *  Deletes a specified &lt;Iitem&gt; from the OrderXml. 
    *  <p>The <code>itemNumber</code> is the number of an item in the document
    *  (starting with 0).
    *  @param  itemNumber  the number of an Item in the OrderXml 
    *  @throws  Exception  Thrown if the elementName is not valid
    */  

   public void  deleteItem (int itemNumber) throws Exception {
      Document doc = this.parse();
      Node orderNode = DomTree.topNode(doc, "Order");                         
      Node itemNode = DomTree.findNode(orderNode, "Item", itemNumber); 
      if (itemNode==null)  {
         throw new Exception("OrderXml exception - Item number "+itemNumber+ " not found");
      } else {
         orderNode.removeChild(itemNode);
         this.xmlText = jcs.xml.DomTree.doc2String(doc);
      }   
   }

   /**  
    *  Returns an XML "Order" with the data for a given <code>customer_id</code>
    *  and <code>order_date</code>.
    *  <p>This method assumes the following SQL tables:
    *  <pre> 
    *  create table customers 
    *     (customer_id varchar(5) not null unique, 
    *     customer_name varchar(50) not null)
    *  create table orders 
    *     (customer_id varchar(5) not null, 
    *      order_date datetime not null, 
    *      item_id varchar(5) not null, 
    *      quantity int not null)
    *  create table items 
    *     (item_id varchar(5) unique, 
    *     item_name varchar(20),
    *     unit smallint)
    *  </pre> 
    *  @param orderDate      a string that is the <code>order_date</code> 
    *                        of the requested order.  This string must be in an SQL
    *                        format, such as "1999/05/07".
    *  @param customerId     a string that is the <code>customer_id</code> 
    *                        of the requested order
    *  @param  server  a string that identifies the SQL server in which to execute
    *                  the query.  See the 
    *                  {@link ExecSql#connectTo(java.lang.String) connectTo} 
    *                  method for a description of this parameter.
    *  @return a string with the output XML document that contains the XML "Order"
    *          representation of the order
    *  @throws ClassNotFoundException  Thrown when the XML parser class 
    *                                     cannot be loaded.
    *  @throws SQLException  Thrown for any exceptions raised during execution
    *                           of the SQL statement
    *  @throws Exception  Thrown for invalid parameters and other conditions
    */  

   public static String sql2Order(String orderDate, String customerId, String server)
         throws ClassNotFoundException, SQLException, Exception {  

      String orderQuery =  "select customer_name, o.item_id, i.item_name, "
                      + "quantity, i.unit "
                      + "from customers c, orders o, items i"
                      + " where o.customer_id = ? and o.order_date = ?"
                      + " and c.customer_id=o.customer_id and i.item_id=o.item_id";                      
      //  Evaluate the SQL query to get the result set rs.
      Connection con = ExecSql.connectTo(server);
      PreparedStatement pstmt = con.prepareStatement(orderQuery);
      pstmt.setObject(1, customerId);
      pstmt.setObject(2, orderDate);
      ResultSet rs = pstmt.executeQuery();
      String  dateVar ;
      String customerIdVar ;
      String customerNameVar ;
      String itemIdVar;
      String itemNameVar;
      String quantityVar;
      String unitVar;

      // Create a new empty Order for the given date and customer id.
      // Note:  we don't yet know the customer name.
      String emptyOrder = OrderXml.emptyOrder
                       (orderDate, customerId, "***");
      OrderXml newOrder  = new OrderXml(emptyOrder);

      Document doc = newOrder.parse();
      Node orderNode= DomTree.topNode(doc, "Order");
      int row = 0;
      while (rs.next()) {
         if (++row == 1) {
            customerNameVar = rs.getString(1);
            Node customerNode = DomTree.findNode(orderNode, "CustomerName");
            DomTree.setText(doc, customerNode, customerNameVar);
         }

         itemIdVar = rs.getString(2);
         itemNameVar = rs.getString(3);
         quantityVar = rs.getString(4);
         unitVar = rs.getString(5);
         Element itemNode = DomTree.appendChild(doc, orderNode, "Item");
         DomTree.appendChild(doc, itemNode, "ItemId", itemIdVar, false);
         DomTree.appendChild(doc, itemNode, "ItemName", itemNameVar, false);
         Element quantity 
            =  DomTree.appendChild(doc, itemNode, "Quantity", quantityVar, false);
         if (Integer.parseInt(unitVar)!=1) {
            DomTree.setAttribute(quantity, "unit", unitVar);
         }
      }
      return jcs.xml.DomTree.doc2String(doc);        
   }

   /**  
    *  Stores the data from an XML Order into a specified SQL table.
    *  <p>The name of the table is a parameter, but the column
    *  names and datatypes are assumed to be as follows
    *  (using the parameter names for the table names):
    *  <pre> 
    *  create table ordersTableName
    *    (customer_id varchar(5) not null, 
    *    order_date datetime not null, 
    *    item_id varchar(5) not null, 
    *    quantity int not null,
    *    unit smallint)
    *  </pre> 
    *  <p>The <code>insert</code> statements that this method performs 
    *  specify the inserted column names, so the actual tables can have
    *  other columns not shown above (such columns must allow nulls).
    *  <p>For a description of the <code>server</code> parameter, 
    *  see the {@link ExecSql#connectTo(java.lang.String) connectTo} method.
    *  @param ordersTableName     a string that is the name of the 'Orders' table
    *  @param server              a string that identifies the server 
    *                             in which to execute the statement
    *  @throws Exception   Thrown for parse errors.
    */  

   public void order2Sql(String ordersTableName, String server) 
                                            throws Exception  {

      Document doc = this.parse();
      Node orderNode = DomTree.topNode(doc, "Order");

      Node dateNode = DomTree.findNode(orderNode, "Date");
      String dateVal =  DomTree.getText(dateNode);

      Node customerIdNode = DomTree.findNode(orderNode, "CustomerId");
      String customerIdVal =  DomTree.getText(customerIdNode);

      NodeList nl =  orderNode.getChildNodes();
      int nextNode = DomTree.nextGivenChild(nl, 0, "Item");
      
      while (nextNode !=-1) {
         Node itemNode = nl.item(nextNode);
         Node itemIdNode = DomTree.findNode(itemNode, "ItemId");
         String itemIdVal = DomTree.getText(itemIdNode);
         Node itemNameNode = DomTree.findNode(itemNode, "ItemName");
         String itemNameVal = DomTree.getText(itemNameNode);
         Node quantityNode = DomTree.findNode(itemNode, "Quantity");
         String quantityVal = DomTree.getText(quantityNode);
         String unitVal = DomTree.getAttribute(quantityNode, "unit");
         unitVal = (StringUtil.isNothing(unitVal))?"1":unitVal;
         String insertOrder = " insert into "+ordersTableName  
             +  " (customer_id, order_date, item_id, quantity, unit) values ( '"
             +  customerIdVal+"', '" + dateVal +"', '"
             + itemIdVal+"', " + quantityVal+ ", " + unitVal +") ";
         ExecSql.statement(insertOrder,server);
               
         nextNode=DomTree.nextGivenChild(nl, nextNode+1, "Item");
      }      
   }

   /**  
    *  Creates an SQL table suitable for "Orders" data.  The table name
    *  is a parameter.  The column names and datatypes are fixed:
    *  <pre> 
    *  create table ordersTableName
    *    (customer_id varchar(5) not null, 
    *    order_date datetime not null, 
    *    item_id varchar(5) not null, 
    *    quantity int not null,
    *    unit smallint)
    *  </pre> 
    *  <p>This method is a convenience for creating the sort of table
    *  needed for the <code>storeOrder</code> method.
    *  <p>For a description of the <code>server</code> parameter, 
    *  see the {@link ExecSql#connectTo(java.lang.String) connectTo} method.
    *  @param ordersTableName     a string that will be the name of the 'Orders' table
    *  @param server              a string that identifies the server in which to execute 
    *                             the statement.
    *  @throws Exception   Thrown for SQL errors.
    */  

   public static void createOrderTable(String ordersTableName, String server)   
                                                throws Exception {

      String createOrders = 
                 " if not exists (select name from sysobjects where name= '"
                         + ordersTableName + "' and type = 'U' )"
                 + " exec ( '" 
                 + "  create table " + ordersTableName 
                 + "    (customer_id varchar(5) not null, "
                 + "    order_date datetime not null, "
                 + "    item_id varchar(5) not null, "
                 + "    quantity int not null, " 
                 + "    unit smallint not null)  ')" ;
      ExecSql.statement(createOrders, server);
   }


}
   