//
//  Copyright (c) 1999, 2001
//  Sybase, Inc
//  Emeryville, CA 94608
//  All Rights Reserved
//
package jcs.xml.resultset;

import jcs.util.*;
import jcs.xml.resultset.*;
import jcs.xml.*;

import java.io.*;

/**  
 *  
 *  A client command line shell to invoke
 *  the <code>ResultSetXml</code> class.  
 *  <p>This command line interpreter invokes the 
 *  <code>toSqlScript</code> method that translates a 
 *  <code>ResultSetXml</code> document to an SQL script 
 *  with a <code>create</code> statement and a list of 
 *  <code>insert</code> statements to re-create
 *  the result set.  That method can be invoked in either
 *  a client or server environment.  This class
 *  <code>DoXml2Sql</code> invokes the method in a client
 *  environment, using client files for the input and results.
 */  



public class DoXml2Sql {

   /**  
    *  The DoXml2Sql class contains only static methods, and cannot be instantiated.
    */    
   private DoXml2Sql() {}  

   /**  
    *  Invokes the <code>resultSet2Script</code> method from a client 
    *  command line.
    *  <p>Usage:
    *  <pre>
    *  java jcsexec.DoXml2Sql -I inFile -O outFile 
    *       [-T resultTableName] [-X columnPrefix] [-G go]
    *  </pre>
    *  <p>For example, assume that <code>emps.xml</code> is a file containing
    *  an XML result set, as produced by <code>RX.query2Xml</code> or
    *  by <code>ResultSetXml</code>.    
    *  <p>Example command line:
    *  <pre> 
    *  java jcs.resultset.DoXmlSql -I emps.xml  -O emps.script      \
    *           -T emps_copy  -X "column_"  -G go
    *  </pre> 
    *  <p>This call will translate the XML result set contained in
    *  <code>emps.xml</code> into an SQL script containing a <b>create table</b>
    *  statement for table name <code>emps_copy</code> and a series of
    *  <b>insert</b> statements into that table name, which re-create the
    *  result set data.
    *  <p>The parameters tags are as follows.  For a detailed description
    *  of the <code>tableName</code>, <code>columnPrefix</code>, 
    *  and <code>go</code> parameters, 
    *  see the <code>jcs.resultset.ResultSetXml</code> constructor,
    *  which this command-line routine invokes.
    *  <ul>
    *  <li><b>-I</b>:  the name of a client file containing the XML document
    *  <li><b>-O</b>:  the name of a client file for the output script
    *  <li><b>-T</b>:  the <code>resultTableName</code> parameter
    *  <li><b>-X</b>:  the <code>columnPrefix</code> parameter
    *  <li><b>-G</b>:  the <code>go</code> parameter
    *  </ul>
    *  @param args  a string array with the command line arguments    
    */  


   public static void main (String args[]) {
   
      //  argTags and argVals are the argument tag letters
      //  and the default argument values.
      //  
      String[] argTags = { "-I", "-O", "-T", "-X",  "-G"};
      String[] argVals = { "", "", "T", "column",  "yes"};



      try{
         //  Call doCommandArgs to merge the user-supplied arguments
         //  with the defaults.
         StringUtil.doCommandArgs(args, argTags, argVals);

         //  Copy the resulting supplied/defaulted args to named variables.
         String inFile = argVals[0];
         String outFile = argVals[1];
         String resultTableName = argVals[2];
         String columnPrefix = argVals[3];
         String go = argVals[4];

         // Check for omitted input or output file names.
         if (inFile.equals("") || outFile.equals("")) {
               throw new IllegalArgumentException("The -I and -O parameters "
                         +  "are required. ");
         }

         // Copy the XML document from the given file to a string variable.
         String source = FileUtil.file2String(inFile);                 

         // Translate the XML document to SQL.
         ResultSetXml rsx = new ResultSetXml(source);
         String outSQL = rsx.toSqlScript(resultTableName, columnPrefix,  go);

         // Copy the SQL script to the specified output file.
         FileUtil.string2File(outFile, outSQL);

         System.out.println("\nInfo:  " 
                 + "Generated SQL script in '"+ outFile + "'");

      } catch (IllegalArgumentException iae) {
         System.out.println("\nInvalid arguments: " + iae.getMessage() + "\n" 
            + "Usage:  java jcsexec.DoXml2Sql -I file-name -O file-name "
            + "[-T table-name] [-X column-prefix]\n");
            
      } catch (FileNotFoundException notFound) {
            System.err.println("\n Error: File not found (" 
                                  + notFound.getMessage()+ ")\n");
            return;
     
      } catch (Exception e) {
            System.out.println("\n Error: " + e.getMessage() + "\n");
            return;
      }  
  }

}