//
//  Copyright (c) 1999, 2001
//  Sybase, Inc
//  Emeryville, CA 94608
//  All Rights Reserved
//
package jcs.xml.resultset;

import jcs.util.*;
import jcs.xml.*;

import java.io.*;
import java.sql.*;
import java.util.*;

/**
*
*  A class to contain a Java Vector with the values of an SQL result set.
*/

public class  ResultSetVector implements java.io.Serializable  {

   /**  
    *  The contents of the ResultSetVector.  This field is private
    *  because it is constrained to have square properties appropriate
    *  for an SQL result set.
    *  @serial  content is the rows and columns of the SQL result set
    *           that the ResultSetVector represents.  It is a vector of 
    *           vectors whose elements are SQL column values.
    */  
   private Vector content;

   /**  
    *  Constructs a ResultSetVector from a plain Vector.
    *  @param arg   a Vector that has the characteristics of an SQL result set.
    *               It must be a Vector of Vectors, with each sub-Vector
    *               having the same number of elements.
    *  @throws Exception  Thrown if the Vector does not have the required
    *                        characteristics.
    */  

   public ResultSetVector(Vector arg) throws Exception {
      String check = checkSquare(arg);
      if (check.equals("")) {
        content = arg;
      } else {
         throw new Exception(check);
      }
   }

   /**  
    *  Returns the <code>content</code> field.
    *  @return   the contained Vector.
    */  

   public Vector content() {
      return content;
   }

   /**  
    *  
    *  Checks that the arg vector has the shape of an SQL result set.
    *  <ul>
    *  <li>The arg must be a vector of vectors.  
    *  <li>A null arg is invalid.  I.e. you can have a null ResultSetVector, 
    *  but the 'content' field of a ResultSetVector cannot be null.
    *  <li>A null or empty row is invalid.
    *  <li>All rows must have the same size.
    *  <li>Null columns are OK.
    *  </ul>
    *  @param arg   a Vector that has the characteristics of an SQL result set.
    *  @return  a string that is empty if the parameter is OK, and is otherwise
    *           a string that will be included in the exception
    *  @throws Exception   To be added
    */  

   public static String checkSquare (Vector arg) throws Exception {
      if (arg==null) {
         return "Value is null";  // 
      } 
      int tableColumnCount=0;
     
      for (int i=0; i<arg.size(); i++) {
         Vector row = null;
         try {
            row = (Vector)arg.elementAt(i);
         } catch (Exception e) {
            return "Row ("+i+") not a Vector";
         }
         if (row==null) {
           return "null row ("+ i +")";
         } 
         int rowColumnCount = row.size();       
         if (i==0) {
            tableColumnCount = rowColumnCount;
         }
         if (rowColumnCount==0) {
           return "empty row (" + i + ")";
         } 
         if (rowColumnCount != tableColumnCount) {
            return "Not square:  tableColumnCount("+tableColumnCount+") "
                  +" row("+i+")  rowColumnCount("+rowColumnCount+")";
         }
       }
       return "";
   }

   /**  
    *  Returns the number of rows. 
    *  @return  an int that is the number of rows
    */  

   public int rowCount() {
      return content.size();      
   }
   /**  
    *  Returns the number of columns. 
    *  @return  an int that is the number of columns
    */  

   public int columnCount() {
      Vector r0 = (Vector)content.elementAt(0);
      return (r0==null)?0:r0.size();
   }

   /**  
    *  Returns the value of row i column j as a String
    *  @param i   an int that is the number of a row of the result set Vector
    *  @param j   an int that is the number of a column of the result set Vector
    *  @return    a string that is the valud of the given column of the given row
    */  

   public String getColumn(int i, int j) {
      if (i<0 || j<0 || i>this.rowCount() || j>this.columnCount()) {  
         return null;
         //  Note:  This screens invalid 'i' values.
         //  So, we don't need to check for null 'this.row(i)' below.
      } else {
         return (String)this.rowVector(i).elementAt(j);
      }
   }

   /**  
    *  Overrides the <code>Object.toString</code> method.
    *  @return   a string that is a display of the result set Vector
    */  

   public String toString() {
      return this.toStringTable(" \n ", ",  ");
   }


   /**  
    *  A 'toString' method for the entire result set.
    *  The string parameters are separators to be placed
    *  between row or column elements.
    *  @param  rowSeparator   a string to be inserted between rows in the output
    *  @param  columnSeparator   a string to be inserted between columns in the output
    *  @return  a String that is a display of the result set Vector
    */  

   public String toStringTable(String rowSeparator,  String columnSeparator) {
      String tableString = "";
      for (int i = 0; i<this.rowCount(); i++) {
         if (i==0) {
            tableString += this.toStringRow(i, columnSeparator);
         } else {
            tableString += rowSeparator + this.toStringRow(i, columnSeparator);
         }
      }     
      return tableString;
   }

   /**  
    *  Returns a string representation of a given row of the result set.
    *  The string parameter is a separator to be placed
    *  between columns of the row.
    *  @param  i  an int that is the number of a row of the result set
    *  @param  columnSeparator   a string to be inserted between columns in the output
    *  @return  a String that is a display of given row of the result set
    */  


   public String toStringRow(int i, String  columnSeparator) {
      String[] rowArray = this.rowArray(i);
      String rowString = "";
      for (int j=0; j<this.columnCount(); j++) {
         if (j==0) {
            rowString += rowArray[j];
         } else {
            rowString += columnSeparator + rowArray[j]; 
         }           
      }
      return rowString;
   }
   /**  
    *  Returns a string representation of a given column of the result set.
    *  The string parameter is a separator to be placed
    *  between the elements of the column.
    *  @param  j  an int that is the number of a column of the result set
    *  @param  columnSeparator   a string to be inserted between columns in the output
    *  @return  a String that is a display of the given column of the result set
    */  
   public String toStringColumn(int j, String  columnSeparator) {
      String[] columnArray = this.columnArray(j);
      String columnString = "";
      for (int i=0; i<this.rowCount(); i++) {
         if (i==0) {
            columnString += columnArray[i];
         } else {
            columnString += columnSeparator + columnArray[i]; 
         }           
      }
      return columnString;
   }


   /**  
    *  Returns the i-th row as a Vector.
    *  @param  i  an int that is the number of a row of the result set
    *  @return  a Vector that is the given row of the result set
    */  

   public Vector rowVector(int i) {
      if (i<0 || i > this.rowCount()) {
         return null;
      } else {
         return (Vector)content.elementAt(i);
      }
   }

   /**  
    *  Returns the j-th column as a Vector.
    *  @param  j  an int that is the number of a column of the result set
    *  @return  a Vector that is the given column of the result set
    */  

   public Vector columnVector(int j) {
      if (j<0 || j>this.columnCount()) {
         return null;
      } else {
         Vector columnVector = new Vector(10,10);
         for (int i=0; i<this.rowCount(); i++) {
            Vector row = (Vector)content.elementAt(i);
            columnVector.addElement(row.elementAt(j));
         }
         return columnVector;
      }
   }

   /**  
    *  Returns the i-th row as a String array.
    *  @param  i  an int that is the number of a row of the result set
    *  @return  a String array that is the given row of the result set
    */  

   public String[] rowArray(int i) {
       return vector2StringArray(this.rowVector(i));
   }

   /**  
    *  Returns the j-th column as a String array.
    *  @param  j  an int that is the number of a column of the result set
    *  @return  a String array that is the given column of the result set
    */  

   public String[] columnArray(int j) {
       return vector2StringArray(this.columnVector(j));
   }

   /**  
    *  Returns a String array equivalent of the Vector parameter.
    *  @param  v   a Vector whose elements are Strings
    *  @return   a String array containing the elements of the Vector 
    */  


   private static String [] vector2StringArray(Vector v)  {
       if (v==null) {return null;};
       String[] workArray = new String[v.size()];
       v.copyInto(workArray);
       return workArray;   
   }         

   /**  
    *  Constructs a <code>ResultSetVector</code> from the SQL result set 
    *  returned by the argument query.
    *  <p>Example Java code:
    *  <pre> 
    *  ResultSetVector rsv     //  A client-side call
    *     = new ResultSetVector("select * from systypes", "antibes:4000?user=sa");
    *  ResultSetVector rsv2    //  A server-side call
    *     = new ResultSetVector("select * from emp, dept where emp.dno=dept.dno", "");
    *  </pre> 
    *  @param query  a string that is the SQL query whose result set is to be
    *                returned as a ResultSetVector
    *  @param  server  a string that identifies the SQL server in which to execute
    *                  the query.  See the {@link jcs.util.ExecSql#connectTo connectTo} method 
    *                  for a description of this parameter.
    *  @throws ClassNotFoundException  Thrown when the XML parser class 
    *                                     cannot be loaded.
    *  @throws SQLException  Thrown for any exceptions raised during execution
    *                           of the SQL statement
    *  @throws Exception  Thrown for invalid parameters and other conditions
    */  

   public ResultSetVector(String query, String server) 
         throws ClassNotFoundException, SQLException, Exception {  

      //  Evaluate the SQL query to get the result set rs.
      ResultSet rs  = ExecSql.query(query, server );
 
      //  Get the result set meta data
      ResultSetMetaData rsmd = rs.getMetaData();

      //  Get the column count, 
      int columnCount =  rsmd.getColumnCount();

      content = queryResultSetData(rs, columnCount );

   }

   /**  
    *  
    *   Processes the data rows.
    *   @param rs  a java.sql.ResultSet that is the result of the query
    *   @param  columnCount  the number of columns in the result set
    *   @return a Vector containing the rows and columns of the result set
    *   @throws Exception   To be added
    */  

   private static Vector queryResultSetData(ResultSet rs, int columnCount )  
                   throws Exception {

      // Work variable
      String xString = "";

      //  Create a Vector that will contain a Vector for each row.
      Vector rowsVector = new Vector(10, 10);

      //  Process the rows of the result set
      while (rs.next()) {

         //  Create a Vector that will contain an element for each column.
         Vector rowVector = new Vector();

         //   Process the columns of this row.         
	 for(int i=1; i<=columnCount; i++) {

            //  Get the i-th column
            xString = rs.getString(i);
            boolean wasNull = rs.wasNull();
            xString = (xString==null || wasNull)?null:xString;              

            //  Append the i-th column to the rowVector.  
            rowVector.addElement(xString);

         }  // End of for-loop for columns in a row.

         //  Append this row to the rowsVector.
         rowsVector.addElement(rowVector);

      } //  End of while-loop for rows in the result set

      return rowsVector;

   }


   /**  
    *  Returns the universal quantification of a given comparison
    *  to a given column of the <code>ResultSetVector</code>.
    *  <p>Example Java code:
    *
    *  <pre>rsv.allString(i, "<=", "xxx")</pre>
    *
    *  returns true iff for all elements ci of column i of rsv, 
    *  <pre>ci <= "xxx"</pre>
    *
    *  The <code>allString</code> method performs only String comparisons.
    *  @param columnNumber   the number of a column of the result set
    *  @param  compOp   a string that is one of "=,==,<,>,<>,!=, >=,<=".
    *  @param  comparand  a string that will be the second operand of the compares.
    *  @return  a boolean indicating whether the universal quantification is true
    *  @throws Exception    To be added 
    */  
   
   public Boolean allString(int columnNumber, String compOp, String comparand)  
                           throws Exception {
      String someOps = "= ,==,< ,> ,<>,!=,>=,<=";
      String allOps  = "!=,!=,>=,<=,==,==,< ,> ";
      if (compOp==null) {
         throw new Exception ("Null compOp");
      }
      int someOpIndex = someOps.indexOf(compOp.trim());
      if (someOpIndex==-1) {
         throw new Exception("Invalid compOp: "+ compOp);   
      }
      String compOpAll = allOps.substring(someOpIndex, someOpIndex+2);
      Boolean some = this.someString(columnNumber, compOpAll, comparand);  
      return (some==null)?null:new Boolean(!some.booleanValue());
   }



   /**  
    *  Returns the existential quantification of a given comparison
    *  to a given column of the <code>ResultSetVector</code>.
    *  <p>Example Java code:
    *
    *  <pre>rsv.someString(i, "<=", "xxx")</pre>
    *
    *  returns true iff for some element ci of column i of rsv, 
    *  <pre>ci <= "xxx"</pre>
    *
    *  The <code>someString</code> method performs only String comparisons.
    *  @param columnNumber   the number of a column of the result set
    *  @param  compOp   a string that is one of "=,==,<,>,<>,!=, >=,<=".
    *  @param  comparand  a string that will be the second operand of the compares.
    *  @return  a boolean indicating whether the existential quantification is true
    *  @throws Exception    To be added 
    */  
   
   public Boolean someString(int columnNumber, String compOp, String comparand)  
                           throws Exception {
      if (this.content==null || comparand==null) {
         return null;
      }
      String[] columnArray = this.columnArray(columnNumber);
      if (columnArray == null) {
         return null;
      }
      if (compOp==null) {
         return null;
      }
      //  The following are meanings of compareTo results 
      //  that are negative, zero, or positive,
      //  for each compOp.
      boolean[] equals = {false, true, false};
      boolean[] notEquals = {true, false, true};
      boolean[] lessThan = {true, false, false};
      boolean[] greaterThan = {false, false, true};
      boolean[] lessThanOrEquals = {true, true, false};
      boolean[] greaterThanOrEquals = {false, true, true};       

      //  "relation[]" will be the meaning of a compareTo result 
      //  that is negative, zero, or positive.  
      boolean[] relation = null;   

      String compOpVar = compOp.trim();
      if ("==".indexOf(compOpVar) != -1) {
         relation = equals; 
      } else if (compOpVar.equals("!=") || compOpVar.equals("<>")) {
         relation = notEquals;
      } else if (compOpVar.equals("<=")) {
         relation = lessThanOrEquals;
      } else if (compOpVar.equals(">="))  {
         relation = greaterThanOrEquals;
      } else if (compOpVar.equals("<")) {
         relation = lessThan;
      } else if (compOpVar.equals(">")) {
         relation = greaterThan;
      }
      int comparison=0;
      boolean someElementNull = false;
      String comparandVar = comparand.trim(); 
      for (int i=0; i<columnArray.length; i++) {
         if (columnArray[i]!=null) {
            comparison = columnArray[i].trim().compareTo(comparandVar);
            if (comparison<0 && relation[0]) {
               return new Boolean(true);
            } if (comparison==0 && relation[1]) {
               return new Boolean(true);
            } if (comparison>0&& relation[2]) {
               return new Boolean(true);
            }

         } else {
            someElementNull = true;
         }
      }
      return (someElementNull?null:new Boolean(false));
   }



}
//  ------------------------------------------------------------------------------


