//
//  HandleObject.java  
//
//  Copyright (c) 1998 
//  Sybase, Inc 
//  Emeryville, CA 94608
//  All Rights Reserved
//             

package sample;
import java.sql.*;        
import java.util.Properties;

/**
 * The HandleObject class demonstrates how to insert, update, and delete 
 * data stored as a Java Object in a Sybase database.
 *
 * Command line arguments for this sample:<br>
 * -U username<br>
 * -P password<br>
 * -S server<br>                           
 *
 * This sample can only work for Sybase servers which have the ablility to
 * store Java Objects.  Server which have been tested against this sample
 * are; Adaptive Server Anywhere (ASA) Version 6.001 and higher, and
 * Adaptive Server Enterprise (ASE) Version 12.0 and higher.
 *
 * This sample requires the 'sample.Address' class to be installed on the
 * targeted server.  Use the following instructions to install 
 * sample.Address.
 * 
 * 1) Build the samples using the Makefile (for Unix) or make_nt.bat (NT).
 * 2) Place the sample/Address.class file into a jar file:
 *    a) go to the $JDBC_HOME/classes directory
 *    b) run the jar command: 
 *       jar cvf0 Address.jar sample/Address.class
 * 4) Install it onto the server
 *    a) for ASE (SQL Server), use: 
 *       $SYBASE/bin/installjava -f <dir>/Address.jar -new -U sa -P passwd
 *    b) for ASA (Anywhere):  
 *       install java new from file '<dir>/Address.jar' 
 * 
 * When running this sample, add the 'Address.jar' to the classpath.
 *
 */

public class HandleObject
{
    // Constants
    final static String TABLE_NAME = "emps";
    final static String DRIVER_PREFIX = "jdbc:sybase:Tds:";
    final static String USAGE = "Usage: java HandleObject " + 
        "-S <servername:portnumber>|<url>" +
        " -U <username> -P <password>";

    // members
    static String _server = "";
    static Connection _con;
    static Properties _props; // for "user" and "password"
    static String _url = "";

    public static void main(String args[])
    {
        _props = new Properties();

        if (!processCommandline(args))
        {
            System.out.println(USAGE);
            System.exit(0);
        }

        try
        {
            // Load the driver
            Class.forName("com.sybase.jdbc.SybDriver");
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        // Get a connection
        try
        {
            _con = DriverManager.getConnection(_url, _props);
            Statement stmt = _con.createStatement();

            // Get Database Metadata
            DatabaseMetaData md = _con.getMetaData();

            // Check if the table we are going to create already exists
            ResultSet rs = md.getTables(null, null, TABLE_NAME, null);

            // If there is a result we need to delete the table first
            int res = 0;

            if (rs.next())
            {
                res = stmt.executeUpdate("drop table " + TABLE_NAME);
                System.out.println("Table " + TABLE_NAME + " deleted");
            }
            // create the table
            res = stmt.executeUpdate("create table "+ TABLE_NAME +" ( name "+
                "varchar(30), home_addr sample.Address)");
            System.out.println("Table " + TABLE_NAME + " created");
            // Enter data
            // It can be done either by sending T-SQL commands
            enterObjectsBySQL();
            // or by creating the Object first and then just send that one
            enterRealObjects();
            // Finally we want to check the Data
            System.out.println("Select all");
            doSelect("select * from "+ TABLE_NAME);
            // Update the Object values
            updateAddressSQL("99999", "666 Heaven's Gate");
            // select again
            System.out.println("Select all");
            doSelect("select * from "+ TABLE_NAME);
        }
        catch (SQLException sqe)
        {
            sqe.printStackTrace();
        }
    }
    /** Updates the Address object 
         * @param set new value
         * @param where old value
         */
    public static void updateAddressSQL(String set, String where)
    {
        // NOTE: This is ASE syntax. It may be different on other backends
        // define a String for use with PreparedStatement
        String updateString = "update "+ TABLE_NAME +
            " set home_addr>>_zip = ? where home_addr>>_street = ?";
        try
        {
            // define Prepared Statement
            PreparedStatement pstmt = _con.prepareStatement(updateString);
            // Set the Name
            pstmt.setString(1, set);
            // Set the Address Object
            pstmt.setObject(2, where);
            // execute the Statement
            int res = pstmt.executeUpdate();
            System.out.println("update Object affected "+ res +" rows");
        }
        catch (SQLException sqe)
        {
            sqe.printStackTrace();
        }
    }

    /** Inserts the Address object by using setObject on a 
         * prepared statement
         */
    public static void enterRealObjects()
    {
        String name = "Joe Black";
        Address add = new Address ("666 Heaven's Gate","11111");
        // define a String for use with PreparedStatement
        String insertString = "insert into "+ TABLE_NAME +" values (?,?)";
        try
        {
            // define Prepared Statement
            PreparedStatement pstmt = _con.prepareStatement(insertString);
            // Set the Name
            pstmt.setString(1, name);
            // Set the Address Object
            pstmt.setObject(2, add);
            // execeute the Statement
            int res = pstmt.executeUpdate();
            System.out.println("Insert a Real Object affected "+ res +" rows");
        }
        catch (SQLException sqe)
        {
            sqe.printStackTrace();
        }
    }

    /** 
     *  Inserts the Address object by using plain Transact SQL commands
     */
    public static void enterObjectsBySQL()
    {
        // NOTE: This is ASE syntax. It may be different on other backends
        // declare the variable
        String sqlVar = "declare @A sample.Address select @A = " + 
            " new sample.Address('432 Post Lane', '99444')";
        // Insert it using a variable
        String insert1 = " insert into "+ TABLE_NAME +
            " values ('Don Green',@A)";
        // or using a constructor
        String insert2 = "insert into "+ TABLE_NAME + 
            " values ('Bob Blue', " +
            " new sample.Address('234 Stone Road', '99777'))";

        try
        {
            Statement stmt = _con.createStatement();
            // send the batch
            int res = stmt.executeUpdate(sqlVar + insert1);
            System.out.println("Insert with variable affected "+ res +" rows");
            // send the insert command
            res = stmt.executeUpdate(insert2);
            System.out.println(
                "Insert with constructor affected "+ res +" rows");
        }
        catch (SQLException sqe)
        {
            sqe.printStackTrace();
        }
    }

    /** 
     *  Selects the rows: name, Address object
     *  @param selectStmt Select Statement
     */
    public static void doSelect(String selectStmt)
    {
        try
        {
            Statement stmt = _con.createStatement();
            ResultSet rs = stmt.executeQuery(selectStmt);
            // Loop over the result set
            while (rs.next())
            {
                // Get the name
                String name = rs.getString(1);
                // Get the address
                Address obj = (Address)rs.getObject(2);
                System.out.println("Name: "+ name +", "+ obj.display());
            }
        }
        catch (SQLException sqe)
        {
            sqe.printStackTrace();
        }
    }
    /**
     *  Handles command line options for all samples.<br>
     *  Possible arguments include:
     *  <ul> 
     *     <li>  -U &ltusername&gt
     *     <li>  -P &ltpassword&gt 
     *     <li>  -S &ltservername&gt
     *  </ul>
     */
    public static boolean processCommandline(String args[])
    {
        boolean retVal = true;

        if (args.length != 6)
        {
            System.out.println("Wrong number of parameters");
            return false;
        }

        for (int i = 0; retVal == true && i < args.length; i++)       
        {
            String arg = args[i];
            if (arg.regionMatches(0, "-", 0, 1))
            {
                try
                {
                    switch (arg.charAt(1))
                    {
                        case 'U':
                            i++;
                            _props.put("user", args[i]);
                            break;                            
                        case 'P':
                            i++;
                            _props.put("password", args[i]);
                            break;                            
                        case 'S':
                            i++;
                            if (args[i].startsWith(DRIVER_PREFIX))
                            {
                                _url = args[i];
                            }
                            else
                            {
                                _url = DRIVER_PREFIX + args[i];
                            }
                            break;  
                        default:
                            System.out.println("Invalid Parameter: '" + 
                                arg.charAt(1) + "'");
                            retVal = false;
                            break;
                    }
                }
                catch (Exception ex)
                {
                    retVal = false;
                }
            }
        }

        return (retVal);
    }
}
